//-----------------------------------------------------------------------
// FCalcDoCalc
//-----------------------------------------------------------------------


//-----------------------------------------------------------------------
// System includes
#include <vcl.h>
#include <stdio.h>
#include <string>
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
// BCB (Borland C++ Builder) Hint
#pragma hdrstop
//-----------------------------------------------------------------------



//---------------------------------------------------------------------------
// application includes
#include "JrPlugin_GenericShell.h"

// calucator
#include "FCalcDoCalc.h"
#include "muparser_myextensions.h"

//
#include "AdvConfigFormUnit.h"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
mu::Parser *mparserp=NULL;
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
double afValBuf[MAX_EVARS];
int localvarcount = 0;
bool autocreatevars=false;
bool loadingvars=false;
//
// last result for adding to history
AnsiString LastResultStr;
AnsiString LastExpressionStr;
AnsiString LastPureResultStr;
double lastevalval=0;
double lastevalval_exposed=0;
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
bool CalcInit()
{
	// error check
	if (mparserp!=NULL)
		ShowMessage("Internal Error in FCalc - mparserp is not NULL");

	// create new parser
	mparserp=new mu::Parser();

	// add Questor's custom parsers
	mparserp->AddValIdent(isBinVal);
	mparserp->AddValIdent(isHexVal);

	// make sure we search in plugin dir for any dlls WE ourselves need
	mparserp->SetVarFactory(AddVariable, mparserp);

	// create built in vars
	CreateBuiltInVars();

	return true;
}


void CalcUnInit()
{
	// delete the parser
	// ShowMessage("ATTN: in calcuninit!");
	if (mparserp==NULL)
		{
		// already freed
		return;
		//ShowMessage("Internal Error in FCalc - mparserp is NULL");
		}
	delete mparserp;
	mparserp=NULL;
	localvarcount=0;
}


void ClearAllLocalVariables()
{
	mparserp->ClearVar();
	localvarcount=0;
	CreateBuiltInVars();
}


void ClearOneLocalVariables(char *varname)
{
	mparserp->RemoveVar(varname);
	--localvarcount;
}


void CreateBuiltInVars()
{
	// set our builtin variables
	mparserp->DefineVar("lv", &lastevalval_exposed);
}
//---------------------------------------------------------------------------







//---------------------------------------------------------------------------
// Factory function for creating new parser variables
// This could as well be a function performing database queries.
double* AddVariable(const mu::char_type *a_szName, void *a_pUserData)
{
  // I don't want dynamic allocation here, so i used this static buffer
  // If you want dynamic allocation you must allocate all variables dynamically
  // in order to delete them later on. Or you find other ways to keep track of 
  // variables that have been created implicitely.


//  mu::console() << _T("Generating new variable \"")
//                << a_szName << _T("\" (slots left: ")
//                << 99-iVal << _T(")")
//                << _T(" User data pointer is:")
//                << std::hex << a_pUserData <<endl;

  if (!loadingvars && (!autocreatevars || LastExpressionStr.Pos("=")==0))
	{
	std::string estr="Unknown variable '"+std::string(a_szName)+"'.";
	throw mu::ParserError( estr.c_str() );
	}

  if (localvarcount>=MAX_EVARS)
    throw mu::ParserError( _T("FCalc Error -- Too many local variables created.") );

  // only allow vars starting with v?
  if (false)
    {
    if (a_szName[0]!='v')
	throw mu::ParserError( _T("Variables must begin with the letter v.") );
    }


  afValBuf[localvarcount] = 0;
  return &afValBuf[localvarcount++];
}
//---------------------------------------------------------------------------





//---------------------------------------------------------------------------
AnsiString DoCalcExpression(AnsiString expressionstr, int triggermethod)
{
	AnsiString retvstr;

	if (triggermethod==0)
		autocreatevars=false;
	else
		autocreatevars=true;

	try
		{
	lastevalval = ParserDoEval(expressionstr);
		retvstr=AnsiString(lastevalval);
		}
	catch(mu::Parser::exception_type &e)
		{
		//	resultstr = AnsiString(e.GetMsg().c_str()) + "["+e.GetCode()+"]";
		retvstr = AnsiString("[ERR: ")+AnsiString(e.GetMsg().c_str())+AnsiString("]");
		}
	catch(Exception &e)
		{
		retvstr = e.Message;
		}

	if (!autocreatevars && triggermethod==0)
		{
		// not done typing yet
		if (retvstr.Pos("Unexpected end of formula")>0 || retvstr.Pos("Unknown variable")>0)
			{
			// dont complain yet, user isnt done
			retvstr="";
			}
		}

	// reset
	autocreatevars=false;

	return retvstr;
}
//---------------------------------------------------------------------------






//---------------------------------------------------------------------------
double ParserDoEval(AnsiString str)
{
	// should we smartly replace , with . when outside ( for locale hack?
	str=SmartLocaleCommaReplace(str);
	str=SmartFixupExpression(str);

	mparserp->SetExpr(str.c_str());
	double dretv=mparserp->Eval();

	return dretv;
}
//---------------------------------------------------------------------------





//---------------------------------------------------------------------------
AnsiString SmartLocaleCommaReplace(AnsiString str)
{
	// should we smartly replace , with . when outside ( for locale hack?
	int parendepth=0;
	bool inquote=false;
	char c;
	int len=str.Length();

	bool swapall=AdvConfigForm->CheckBoxSwapCommas->Checked;

	for (int i =1;i<=len;++i)
		{
		c=str[i];
		if (c=='\"')
			inquote=!inquote;
                else if (c=='(')
			++parendepth;
                else if (c==')')
			--parendepth;
		else if (c==',')
			{
			if (!inquote && (parendepth==0 || swapall))
				{
				// change to .
				str[i]='.';
				}
			}
		else if (c==';')
			{
			if (!inquote && swapall)
				{
				// change to ,
				str[i]=',';
				}
			}
		}
	return str;
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
AnsiString SmartFixupExpression(AnsiString str)
{
	// should we smartly replace , with . when outside ( for locale hack?
	int len=str.Length();
	if (len<1)
		return str;

	// we care about first character of + or -
	char c,firstc;
	firstc=str[1];
	if (firstc!='+' && firstc!='-' && firstc!='*' && firstc!='/')
		{
		// no firstchar so return
		return str;
		}

	bool inquote=false;
	bool nonletter=false;

	for (int i =2;i<=len;++i)
		{
		c=str[i];
		if (c=='\"')
			inquote=!inquote;
                else if (inquote || (c>='A' && c<='Z') || (c>='a' && c<='z') || (c>='0' && c<='9'))
			{
                        // ok its a norma letter or number, so ignore it
			}
		else
			nonletter=true;
		}

	if (!nonletter)
		{
		// when we have no nonletter characters in the expression, then its just like +43 so we fixup by ADDING lastval in front
		str="lv"+str;
		}
	else
		{
		// there is some other characters, in which case we only fixup a starting +
		if (firstc=='+')
			str=str.SubString(2,len-1);
		}

	return str;
}
//---------------------------------------------------------------------------











