//---------------------------------------------------------------------------
// JRbcbConfigCollection
//  manage a set of config files in a menu
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#include "JRConfigCollection.h"
//---------------------------------------------------------------------------



//-----------------------------------------------------------------------
ConfigEntry::ConfigEntry(std::string in_FileName,bool relativenicelabelflag,std::string BaseDirectory)
{
	FileName=in_FileName;
	LabelString=MakeLabelFromFileName(FileName,relativenicelabelflag,BaseDirectory);
	isDupeLabel=false;
}
//-----------------------------------------------------------------------


//---------------------------------------------------------------------------
std::string ConfigEntry::MakeLabelFromFileName(std::string fullfilename,bool relativenicelabelflag,std::string BaseDirectory)
{
	// make label from full filename
	std::string labelstr=fullfilename;;
	const char* charp=fullfilename.c_str();
	char c;
	int len=strlen(charp);
	int startpos=0;
	int endpos=len;

	if (relativenicelabelflag)
		{
		// new is just everything relative to basedir
		int len =BaseDirectory.length();
		if (strncmp(BaseDirectory.c_str(),fullfilename.c_str(),len)==0)
			{
			// got a left match
			char *charp2 = (char*)(fullfilename.c_str()) + len;
			if (*charp2=='\\')
				++charp2;
			labelstr=std::string(charp2);
			}
		}

	if (!relativenicelabelflag)
		{
		// standard old way - get end filename
		// find last \ and first following .
		for (int i=0;i<len;++i)
			{
			c=charp[i];
			if (c=='\\' || c=='/')
				{
				startpos=i+1;
				endpos=len;
				}
			else if (c=='.' && endpos==len)
				endpos=i;
			}

		// get labelstring
		if (endpos-startpos>0)
			labelstr=std::string(&charp[startpos],endpos-startpos);
		}

	// return it
	return labelstr;
}
//---------------------------------------------------------------------------








//---------------------------------------------------------------------------
void JrConfigManager::Clear()
{
	// walk through and delete
	std::vector<ConfigEntry*> ::iterator iter;
	for (iter=ConfigEntries.begin();iter!=ConfigEntries.end();++iter)
		{
		// delete the pointed to node
		delete (*iter);
		}
	ConfigEntries.clear();
}
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
void JrConfigManager::RescanForFiles()
{
	// clear first
	Clear();

	// call templated objectmember function callback to find files
	RecursiveFileFinder<JrConfigManager>(this,BaseDirectory,FileMask,true);
}
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
bool JrConfigManager::AddAFile(std::string fullfilename)
{
	// callback for discovering files

	// create it
	ConfigEntry* entryp=new ConfigEntry(fullfilename,relativenicelabelflag,BaseDirectory);
	// is it a dupe
	entryp->set_isDupeLabel(IsLabelDuped(entryp));
	// add it
	ConfigEntries.push_back(entryp);
	return true;
}


bool JrConfigManager::IsLabelDuped(ConfigEntry* entryp)
{
	// see if any other entries have same label
	std::vector<ConfigEntry*> ::iterator iter;
	for (iter=ConfigEntries.begin();iter!=ConfigEntries.end();++iter)
		{
		// delete the pointed to node
		if (*iter==entryp)
			continue;
		if ((*iter)->get_LabelString() == entryp->get_LabelString())
			return true;
		}
	return false;
}
//---------------------------------------------------------------------------





//-----------------------------------------------------------------------
int JrConfigManager::FindEntryIndexByFileName(std::string fullfilename)
{
	// try to find the entry
	int index=0;
	std::vector<ConfigEntry*> ::iterator iter;
	for (iter=ConfigEntries.begin();iter!=ConfigEntries.end();++iter)
		{
		if ((*iter)->get_FileName() == fullfilename)
			return index;
		++index;
		}
	return -1;
}
//-----------------------------------------------------------------------

