
//-----------------------------------------------------------------------
#pragma package(smart_init)
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
// regular expression via pcre library
#include <pcre.h>
//-----------------------------------------------------------------------



//---------------------------------------------------------------------------
#include "JrBookmarkManager.h"
//---------------------------------------------------------------------------


//-----------------------------------------------------------------------
#include <fstream>
#include <stdlib>
#include <stdio.h>
//
#include <windows.h>
#include <ShlObj.h>
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
// BCB (Borland C++ Builder) Hint
#pragma hdrstop
//-----------------------------------------------------------------------



//-----------------------------------------------------------------------
JrBookmarkManagerT::JrBookmarkManagerT()
{
	// reload interval = 1 hour
	debugout << "FarrFoxx JrBookmarkManagerT A1";
	processupdateinterval=60*60;
	debugout << "FarrFoxx JrBookmarkManagerT A2";
	SetupRegexes();
	debugout << "FarrFoxx JrBookmarkManagerT A3";
	bookmarksloaded=false;
	debugout << "FarrFoxx JrBookmarkManagerT A4";
	// init urlfinder callback pointer
	urlfilefinderp = new JrURLFileFinder();
	debugout << "FarrFoxx JrBookmarkManagerT A5";
	urlfilefinderp->set_bookmarkmanagerp(this);
	debugout << "FarrFoxx JrBookmarkManagerT A6";
}


JrBookmarkManagerT::~JrBookmarkManagerT()
{
	debugout << "FarrFoxx ~JrBookmarkManagerT 1";

	// ATTN: this is causing trouble
	if (false)
		ShutdownRegexes();

	debugout << "FarrFoxx ~JrBookmarkManagerT 2";
	FreeAllBookmarks();
	debugout << "FarrFoxx ~JrBookmarkManagerT 3";
	// delete urlfinder
	delete urlfilefinderp;
	debugout << "FarrFoxx ~JrBookmarkManagerT 4";
}
//-----------------------------------------------------------------------



//-----------------------------------------------------------------------
bool JrBookmarkManagerT::SetupRegexes()
{
	// set up one time regex compiling, etc
	PcrepCompile(pcrep_url, pcre_extrap_url, JrFirefoxBookmarks_Regex_URL);
	PcrepCompile(pcrep_title, pcre_extrap_title, JrFirefoxBookmarks_Regex_Title);
	PcrepCompile(pcrep_shortcut, pcre_extrap_shortcut, JrFirefoxBookmarks_Regex_Shortcut);
	PcrepCompile(pcrep_postdata, pcre_extrap_postdata, JrFirefoxBookmarks_Regex_PostData);
	// success
	return true;
}


bool JrBookmarkManagerT::ShutdownRegexes()
{
	// shut down regex compiling
	PcrepUncompile(pcrep_url, pcre_extrap_url);
	PcrepUncompile(pcrep_title, pcre_extrap_title);
	PcrepUncompile(pcrep_shortcut, pcre_extrap_shortcut);
	PcrepUncompile(pcrep_postdata, pcre_extrap_postdata);
	// success
	return true;
}
//-----------------------------------------------------------------------



//-----------------------------------------------------------------------
bool JrBookmarkManagerT::PcrepCompile(pcre* &pcrep, pcre_extra* &pcre_extrap, char* regexstrbuf)
{
	// compile the regex (which is in regexstr)

	// options and stuff
	int options=0;
	char *errorstr;
	char *erroroffset;
	char *pattern;
	int erroffset;
	char *tableptr;
	int extraoptions=0;
	//
	options |= PCRE_CASELESS;

	pcrep = pcre_compile(regexstrbuf, options, (const char **) &errorstr, &erroffset, NULL);
	// study it
	if (pcrep!=NULL)
		{
		pcre_extrap = pcre_study(pcrep, extraoptions, (const char **) &errorstr);
		return true;
		}
	pcre_extrap = NULL;
	return false;
}


void JrBookmarkManagerT::PcrepUncompile(pcre* &pcrep, pcre_extra* &pcre_extrap)
{
	// destruct the regex
	if (pcrep!=NULL)
		{
		free(pcrep);
		pcrep=NULL;
		}
	if (pcre_extrap!=NULL)
		{
		free(pcre_extrap);
		pcre_extrap=NULL;
		}
}
//-----------------------------------------------------------------------























//-----------------------------------------------------------------------
bool JrBookmarkManagerT::DoesNeedReloadBookmarks()
{
	// get current time so we can keep track of when to reload
	time(&curprocessupdatetime);

	// already loaded recently and so dont need to reload?
	if (bookmarksloaded)
		{
		if (!option_rescanregularly || curprocessupdatetime-lastprocessupdatetime<=processupdateinterval)
			return false;
		}

	// yes we need to reload
	return true;
	}


bool JrBookmarkManagerT::LoadBookmarksIfAppropriate()
{
	if (DoesNeedReloadBookmarks())
		{
		// load bookmarks
		LoadAllBookmarks();
		return true;
		}

	// not reloaded
	return false;
}
//-----------------------------------------------------------------------












//-----------------------------------------------------------------------
bool JrBookmarkManagerT::FreeAllBookmarks()
{
	// walk through and delete all bookmark pointer data
	int bcount = jrbookmark.size();
	for (int i=0;i<bcount;++i)
		delete jrbookmark[i];

	// after freeing, clear
	jrbookmark.clear();

	// clear
	bookmarksloaded=false;

	// success
	return true;
}
//-----------------------------------------------------------------------



//-----------------------------------------------------------------------
bool JrBookmarkManagerT::LoadAllBookmarks()
{
	// load all bookmarks

	// free first
	FreeAllBookmarks();

	// now for each browser
	if (browserenable_firefox)
		{
		// load all firefox bookmarks
		LoadAllBookmarks_FireFox();
		}

	if (browserenable_opera)
		{
		// load all opera bookmarks
		LoadAllBookmarks_Opera();
		}

	if (browserenable_ie)
		{
		// now IE bookmarks
		LoadAllBookmarks_IE("");
		}

	// set flag so we dont reload
	bookmarksloaded=true;

	// record time of last load
	lastprocessupdatetime=curprocessupdatetime;

	// success
	return true;
}
//-----------------------------------------------------------------------









//-----------------------------------------------------------------------
int JrBookmarkManagerT::LoadAllBookmarksInFolder(const char *fname)
{
	// this is used when user specifies a custom folder
	int readcount=0;

	// ie
	if (browserenable_ie)
		LoadAllBookmarks_IE(std::string(fname));

	// ATTN: unfinished - search through subfolders for opera and ff

	return readcount;
}
//-----------------------------------------------------------------------






//-----------------------------------------------------------------------
int JrBookmarkManagerT::LoadAllBookmarksInFile(const char *fname)
{
	std::string linestr;
	int readcount=0;

	// open file
	std::ifstream infile(fname, std::ios_base::in);
	if (!infile)
		return 0;


	// NEW: first line tells us something (opera vs. firefox)
	if (getline(infile, linestr, '\n'))
		{
		if (strstr(linestr.c_str(),"Opera")!=NULL)
			{
			if (browserenable_opera)
				readcount=LoadBookmarksFromOpenFile_Opera(&infile);
			}
		else if (strstr(linestr.c_str(),"NETSCAPE")!=NULL)
			{
			if (browserenable_firefox)
				readcount=LoadBookmarksFromOpenFile_FireFox(&infile);
			}
		else
			{
			// how about calling a default file that grabs urls?
			}
		}

	// infile will close on exit of func

	// return count
	return readcount;
	}
//-----------------------------------------------------------------------





//-----------------------------------------------------------------------
int JrBookmarkManagerT::LoadBookmarksFromOpenFile_FireFox(std::ifstream *infilep)
{
	std::string linestr;
	JrBookmark* jbp;
	const char *linebufp;
	int readcount=0;
	//
	int options = 0;
	int ovector[DEF_OVECCOUNT];
	int pcre_matches;
	int arg1posi=2;
	int arg1endi=3;
	//
	const char *out_result_charp;
	int out_result_len;
	//
	std::string nurl,ntitle,nshortcut,npostdata,iconfpath;

	// default icon
	iconfpath=iconfpath_firefox;

	// walk through it
	// this regex parsing is based on Foxy.cpp from a Launchy plugin called Foxy
	while (getline(*infilep, linestr, '\n'))
		{
		// do regex for url
		linebufp=linestr.c_str();
		pcre_matches = pcre_exec(pcrep_url, pcre_extrap_url, linebufp, strlen(linebufp), 0, options, ovector, DEF_OVECCOUNT);
		// no match?
		if (pcre_matches<0)
			{
			continue;
			}

		// ok got a match
		 ++readcount;

		// url data from ovector
		out_result_charp=linebufp+ovector[arg1posi];
		out_result_len=ovector[arg1endi]-ovector[arg1posi];
		if (out_result_len>0 && out_result_charp[out_result_len-1]=='\"')
			--out_result_len;
		nurl=std::string(out_result_charp,out_result_len);

		// title - get more info
		pcre_matches = pcre_exec(pcrep_title, pcre_extrap_title, linebufp, strlen(linebufp), 0, options, ovector, DEF_OVECCOUNT);
		// no match?
		if (pcre_matches<0)
			ntitle="";
		else
			{
			// data from ovector
			out_result_charp=linebufp+ovector[arg1posi];
			out_result_len=ovector[arg1endi]-ovector[arg1posi];
			ntitle=std::string(out_result_charp,out_result_len);
			}

		// shortcut - get more info
		pcre_matches = pcre_exec(pcrep_shortcut, pcre_extrap_shortcut, linebufp, strlen(linebufp), 0, options, ovector, DEF_OVECCOUNT);
		// no match?
		if (pcre_matches<0)
			nshortcut="";
		else
			{
			// data from ovector
			out_result_charp=linebufp+ovector[arg1posi];
			out_result_len=ovector[arg1endi]-ovector[arg1posi];
			nshortcut=std::string(out_result_charp,out_result_len);
			}

		// shortcut - get more info
		pcre_matches = pcre_exec(pcrep_postdata, pcre_extrap_postdata, linebufp, strlen(linebufp), 0, options, ovector, DEF_OVECCOUNT);
		// no match?
		if (pcre_matches<0)
			npostdata="";
		else
			{
			// data from ovector
			out_result_charp=linebufp+ovector[arg1posi];
			out_result_len=ovector[arg1endi]-ovector[arg1posi];
			npostdata=std::string(out_result_charp,out_result_len);
			}

		//save it
		jbp = new JrBookmark(nurl,ntitle,nshortcut,npostdata,iconfpath);
		jrbookmark.push_back(jbp);
		}

	// return # red
	return readcount;
}
//-----------------------------------------------------------------------








//-----------------------------------------------------------------------
int JrBookmarkManagerT::LoadBookmarksFromOpenFile_Opera(std::ifstream *infilep)
{
	std::string linestr;
	JrBookmark* jbp;
	char *linebufp;
	char* cp;
	int readcount=0;
	bool foldernotfile=false;
	//
	std::string currentfolder;
	//
	std::string nurl,ntitle,nshortcut,npostdata,iconfpath;
	std::string slashstring=std::string("\\");

	// default icon
	iconfpath=iconfpath_opera;

	// walk through it
	// this regex parsing is based on Foxy.cpp from a Launchy plugin called Foxy
	while (getline(*infilep, linestr, '\n'))
		{
		// do regex for url
		linebufp=(char*) linestr.c_str();

		if (linebufp[0]=='-')
			{
			// pop up a folder dir
			int len=currentfolder.length()-2;
			while (len>=0)
				{
				if (currentfolder[len]=='\\')
					{
                                        // truncate here
					currentfolder=currentfolder.substr(0,len);
					break;
					}
				--len;
				}
			if (len<0)
				currentfolder="";
			}
		else if (strstr(linebufp,"#FOLDER")!=NULL)
			foldernotfile=true;
		else if (strstr(linebufp,"#URL")!=NULL)
			foldernotfile=false;
		else if ((cp=strstr(linebufp,"NAME="))!=NULL)
			{
			// get url
			ntitle=std::string(cp+5);
			if (foldernotfile)
				currentfolder+=ntitle+slashstring;
			}
		else if ((cp=strstr(linebufp,"URL="))!=NULL)
			{
			// get url
			nurl=std::string(cp+4);
			//save it
			if (option_folderintitle)
				jbp = new JrBookmark(nurl,currentfolder+ntitle,nshortcut,npostdata,iconfpath);
			else
				jbp = new JrBookmark(nurl,ntitle,nshortcut,npostdata,iconfpath);
			jrbookmark.push_back(jbp);
			}
		}

	// return # red
	return readcount;
}
//-----------------------------------------------------------------------




















//---------------------------------------------------------------------------
std::string JrBookmarkManagerT::JR_MyGetShellFolderPath(const int FolderID,const std::string &poststr)
{
	std::string retstring;
	char Buffer[MAX_PATH];

	LPITEMIDLIST pidl;
	HRESULT hr = SHGetSpecialFolderLocation(NULL, FolderID, &pidl);
	if (SUCCEEDED(hr))
		{
		if (SHGetPathFromIDList(pidl, Buffer))
			{
			Buffer[MAX_PATH-1]='\0';
			retstring=std::string(Buffer);
			return retstring+poststr;
			}
		}
	return "";
}
//---------------------------------------------------------------------------























//-----------------------------------------------------------------------
bool JrBookmarkManagerT::LoadAllBookmarks_FireFox()
{
	// loop through all bookmark files and load bookmarks from each
	std::string profiledir;

	// locate profile dir
//	profiledir=JR_MyGetShellFolderPath(CSIDL_APPDATA,"\\Mozilla\\Firefox\\Profiles");
	profiledir=JR_MyGetShellFolderPath(CSIDL_APPDATA,"\\Mozilla\\Firefox");

	// read all files in dir
	bool bretv=LoadBookmarkFilesFromPath(profiledir,"bookmarks.html");
	return bretv;
}
//-----------------------------------------------------------------------






//-----------------------------------------------------------------------
bool JrBookmarkManagerT::LoadAllBookmarks_Opera()
{
	// loop through all bookmark files and load bookmarks from each
	std::string profiledir;

	// locate profile dir
	profiledir=JR_MyGetShellFolderPath(CSIDL_APPDATA,"\\opera\\");
	// read all files in dir
	bool bretv=LoadBookmarkFilesFromPath(profiledir,"opera6.adr");

	// locate profile dir
	profiledir=JR_MyGetShellFolderPath(CSIDL_PROGRAM_FILES,"\\opera\\");
	// read all files in dir
	bretv |= LoadBookmarkFilesFromPath(profiledir,"opera6.adr");

	return bretv;
}
//-----------------------------------------------------------------------









//-----------------------------------------------------------------------
bool JrBookmarkManagerT::LoadBookmarkFilesFromPath(std::string dirname, std::string filemask)
{
	// config file thing
	cfgmanager.Clear();
	cfgmanager.set_FileMask(filemask.c_str());
	cfgmanager.set_BaseDirectory(dirname.c_str());
	cfgmanager.RescanForFiles();

	// loop through files
	std::string fname;
	ConfigEntry* entryp;
	int entrycount=cfgmanager.get_EntryCount();
	for (int index=0;index<entrycount;++index)
		{
		entryp=cfgmanager.get_Entry(index);
		fname=entryp->get_FileName();
		LoadAllBookmarksInFile(fname.c_str());
		}

	// success
	return true;
}
//-----------------------------------------------------------------------





//-----------------------------------------------------------------------
bool JrBookmarkManagerT::LoadAllBookmarks_IE(std::string basedir)
{
	// load all IE bookmarks (just .url files)
	std::string BaseDirectory;
	std::string FileMask;

	if (basedir!="")
		BaseDirectory=basedir;
	else
		{
		// default profile dir
		BaseDirectory=JR_MyGetShellFolderPath(CSIDL_FAVORITES,"");
		}
	set_basedirlen(BaseDirectory.length()+1);

	// mask
	FileMask="*.url";

	// call templated objectmember function callback to find files
	RecursiveFileFinder<JrURLFileFinder>(urlfilefinderp,BaseDirectory,FileMask,true);

	// success
	return true;
}
//-----------------------------------------------------------------------



//-----------------------------------------------------------------------
bool JrBookmarkManagerT::AddFoundURLFile_CallBack(std::string fullfilename)
{
	// callback from file finder, add it as bookmark
	static std::string urlstring,urltitle,nshortcut,npostdata,iconfpath;

	// default icon
	iconfpath=iconfpath_ie;

	// url
	urlstring=GetUrlFromUrlFile(fullfilename);

	if (option_folderintitle)
		{
		// trim favorites base dir
		urltitle=fullfilename.substr(basedirlen,fullfilename.length()-basedirlen);
		}
	else
		{
		// trim all folder info
		char *rpos = strrchr((char*)(fullfilename.c_str()),'\\');
		if (rpos!=NULL)
			urltitle=std::string(rpos+1);
		}

	// trim extension from filename
	urltitle=urltitle.substr(0,urltitle.length()-4);

	//save it
	JrBookmark* jbp = new JrBookmark(urlstring,urltitle,nshortcut,npostdata,iconfpath);
	jrbookmark.push_back(jbp);

	// success
	return true;
}
//-----------------------------------------------------------------------







//-----------------------------------------------------------------------
#define MAXFLINELEN 512
std::string JrBookmarkManagerT::GetUrlFromUrlFile(const std::string &str)
{
	// get a url inside a .url file
	std::string urlstring;
	char aline[MAXFLINELEN];
	FILE *fp;
	int slen;
	int linecount=0;
	if ((fp=fopen(str.c_str(),"rb")))
		{
		for (;;)
			{
			fgets(aline,MAXFLINELEN,fp);
	                aline[MAXFLINELEN]='\0';
			slen=strlen(aline)-1;
			// trim crs
			while (slen>=1 && aline[slen]<=13)
				aline[slen--]='\0';
			// is it link to url
			if (strncmp(aline,"url=",4)==0 || strncmp(aline,"URL=",4)==0)
				{
				urlstring=std::string(&aline[4],strlen(aline)-4);
				break;
				}
			if ((strncmp(aline,"baseurl=",8)==0 || strncmp(aline,"BASEURL=",8)==0) && urlstring=="")
				{
				urlstring=std::string(&aline[8],strlen(aline)-8);
				// break;
				}
			if (++linecount>30 || feof(fp))
				break;
			}
		fclose(fp);
		}

	return urlstring;
}
//---------------------------------------------------------------------------

