//---------------------------------------------------------------------------
// JrFirefoxBookmarks
// Classes for managing and finding [firefox,opera,ie] bookmarks
//---------------------------------------------------------------------------


//-----------------------------------------------------------------------
#ifndef JrBookmarkManagerH
#define JrBookmarkManagerH
//-----------------------------------------------------------------------




//-----------------------------------------------------------------------
// forward declarations
class JrBookmark;
class JrBookmarkManagerT;
class JrURLFileFinder;
//-----------------------------------------------------------------------



//-----------------------------------------------------------------------
// system lib includes
#include <string>
#include <vector>
#include <time.h>
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
// regular expression via pcre library
#include <pcre.h>
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
// for finding files
#include "JRConfigCollection.h"
//-----------------------------------------------------------------------

//---------------------------------------------------------------------------
// include main debug header
#include "jrdebug.h"
//---------------------------------------------------------------------------


//-----------------------------------------------------------------------
// C++ Builder Stuff
#pragma hdrstop
//-----------------------------------------------------------------------





//-----------------------------------------------------------------------
// regex patterns used for bookmark stuff
#define JrFirefoxBookmarks_Regex_URL		"<A HREF=\"(.*?\")"
#define JrFirefoxBookmarks_Regex_Title		"\">(.*?)</A>"
#define JrFirefoxBookmarks_Regex_Shortcut	"SHORTCUTURL=\"(.*?)\""
#define JrFirefoxBookmarks_Regex_PostData	"POST_DATA"
//
#define DEF_OVECCOUNT 30
//-----------------------------------------------------------------------





//-----------------------------------------------------------------------
class JrBookmark
{
protected:
	std::string url;
	std::string title;
	std::string shortcut;
	std::string postdata;
	std::string iconfpath;
public:
	JrBookmark() {;};
	JrBookmark(const std::string &u, const std::string &t, const std::string &s, const std::string &p, const std::string &ifp) {url=u; title=t; shortcut=s; postdata=p; iconfpath=ifp;};
	~JrBookmark() {;};
public:
	const char* get_url_charp() {return url.c_str();};
	const char* get_title_charp() {return title.c_str();};
	const char* get_shortcut_charp() {return shortcut.c_str();};
	const char* get_postdata_charp() {return postdata.c_str();};
	const char* get_iconfpath_charp() {return iconfpath.c_str();};
};
//-----------------------------------------------------------------------




//-----------------------------------------------------------------------
class JrBookmarkManagerT
{
protected:
	long processupdateinterval;
	time_t lastprocessupdatetime,curprocessupdatetime;
	bool bookmarksloaded;
	int basedirlen;
protected:
	JrConfigManager cfgmanager;
	std::vector<JrBookmark*> jrbookmark;
	JrURLFileFinder* urlfilefinderp;
protected:
	pcre* pcrep_url;
	pcre_extra* pcre_extrap_url;
	pcre* pcrep_title;
	pcre_extra* pcre_extrap_title;
	pcre* pcrep_shortcut;
	pcre_extra* pcre_extrap_shortcut;
	pcre* pcrep_postdata;
	pcre_extra* pcre_extrap_postdata;
protected:
	bool browserenable_firefox,browserenable_opera,browserenable_ie,option_folderintitle;
	bool option_rescanregularly;
protected:
	std::string iconfpath_firefox, iconfpath_opera, iconfpath_ie;
public:
	JrBookmarkManagerT();
	~JrBookmarkManagerT();
public:
	bool FreeAllBookmarks();
	bool LoadAllBookmarks();
	bool LoadAllBookmarks_Opera();
	bool LoadAllBookmarks_FireFox();
	bool LoadAllBookmarks_IE(std::string basedir);
	bool LoadBookmarkFilesFromPath(std::string dirname, std::string filemask);
	//
	bool DoesNeedReloadBookmarks();
	bool LoadBookmarksIfAppropriate();
	//
	int LoadAllBookmarksInFolder(const char *fname);
	int LoadAllBookmarksInFile(const char *fname);
	int LoadBookmarksFromOpenFile_FireFox(std::ifstream *infilep);
	int LoadBookmarksFromOpenFile_Opera(std::ifstream *infilep);
public:
	bool SetupRegexes();
	bool ShutdownRegexes();
public:
	int get_bookmarkcount() {return jrbookmark.size();};
	JrBookmark* get_bookmarkp(int i) {return jrbookmark[i];};
protected:
	bool PcrepCompile(pcre* &pcrep, pcre_extra* &pcre_extrap, char* regexstrbuf);
	void PcrepUncompile(pcre* &pcrep, pcre_extra* &pcre_extrap);
protected:
	std::string JR_MyGetShellFolderPath(const int FolderID,const std::string &poststr);
public:
	bool AddFoundURLFile_CallBack(std::string fullfilename);
	void set_basedirlen(int len) {basedirlen=len;};
	std::string GetUrlFromUrlFile(const std::string &str);
public:
	void Set_BrowserEnable_FireFox(bool val) {browserenable_firefox=val;};
	void Set_BrowserEnable_Opera(bool val) {browserenable_opera=val;};
	void Set_BrowserEnable_IE(bool val) {browserenable_ie=val;};
	void Set_FolderInTitle(bool val) {option_folderintitle=val;};
	void Set_RescanRegularly(bool val) {option_rescanregularly=val;};
public:
	void set_iconfpath_firefox(std::string str) {iconfpath_firefox=str;};
	void set_iconfpath_opera(std::string str) {iconfpath_opera=str;};
	void set_iconfpath_ie(std::string str) {iconfpath_ie=str;};
};
//-----------------------------------------------------------------------




//-----------------------------------------------------------------------
class JrURLFileFinder
{
protected:
	JrBookmarkManagerT* bookmarkmanagerp;
public:
	JrURLFileFinder() {bookmarkmanagerp=NULL;};
public:
	void set_bookmarkmanagerp(JrBookmarkManagerT *p) {bookmarkmanagerp=p;};
public:
	// callback for discovering files
	bool AddAFile(std::string fullfilename) {return bookmarkmanagerp->AddFoundURLFile_CallBack(fullfilename);};
};
//-----------------------------------------------------------------------








//-----------------------------------------------------------------------
#endif
//-----------------------------------------------------------------------
