//---------------------------------------------------------------------------
// AdvConfigFormUnit.cpp
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// System includes and C++ Builder Stuff
#include <vcl.h>
#include <stdio.h>
#include <clipbrd.hpp>

#pragma hdrstop
//---------------------------------------------------------------------------
// Application includes
#include "JrPlugin_MyPlugin.h"
#include "AdvConfigFormUnit.h"
//---------------------------------------------------------------------------
// C++ Builder stuff
#pragma package(smart_init)
#pragma link "hkClipboardMonitor"
#pragma link "icSwavC"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
// Global Form Instance Pointer
TAdvConfigForm *AdvConfigForm=NULL;
//---------------------------------------------------------------------------
__fastcall TAdvConfigForm::TAdvConfigForm(TComponent* Owner)
	: TForm(Owner)
{
	// nothing to do in constructor
}
//---------------------------------------------------------------------------
void __fastcall TAdvConfigForm::FormCreate(TObject *Sender)
{
    MyFlags = MyFlags << rfReplaceAll << rfIgnoreCase;
}
//---------------------------------------------------------------------------
void __fastcall TAdvConfigForm::FormCloseQuery(TObject *Sender, bool &CanClose)
{
	// do nothing if this is called from a general close message
	if (!Visible)
		return;

    try
    {
    	SaveSettings();
        if (AdvConfigForm->lblCalledFromKKPrefs->Caption == "YES") {
            callbackfp_set_strval(hostptr,"window.show","");
            AdvConfigForm->lblCalledFromKKPrefs->Caption = "NO";
        }
        CanClose = true;
    }
    catch (...)
    {
	ReportErrorToHost("FormClose");
    }
}
//---------------------------------------------------------------------------
void __fastcall TAdvConfigForm::FormResize(TObject *Sender)
{
	// do nothing if this is called when form is not visible
	if (!Visible)
		return;

    try
    {
    	lvwClipboard->Column[0]->Width = lvwClipboard->Width - 20;
    }
    catch (...)
    {
	ReportErrorToHost("ErrorFormResize");
    }
}
//---------------------------------------------------------------------------
void TAdvConfigForm::OpenKlipKeeperDataFileToEntries()
{
    try
    {
        AnsiString appdir=AnsiString(dlldir);
        AnsiString OpenFileName = appdir+"KlipKeeper_entries.dat";

        char line[MAXLINELEN+1]; //Line read from file
        FILE *OpenEntries;
        if ((OpenEntries = fopen(OpenFileName.c_str(),"r")) != NULL)
        {
            // Set number of lines to 0; 
            NumLines = 0;
            // Start adding entries.
            while (fgets (line, MAXLINELEN+1, OpenEntries) != NULL)
            {
                AnsiString LineReplace = line;
                LineReplace.SetLength(LineReplace.Length() - 1);
                if (NumLines > MaxEntries)
                {
                    break;
                }
                else
                {
                    ListItemToAdd = lvwClipboard->Items->Add();
                    ListItemToAdd->Caption = LineReplace;
                    NumLines++;
                }
            }
            // Stop adding entries.
            fclose (OpenEntries);
        }

    }
    catch (...)
    {
	ReportErrorToHost("ErrorOpenDataFile");
    }
}
//---------------------------------------------------------------------------
void TAdvConfigForm::SaveEntriesToKlipKeeperDataFile()
{
    try
    {
        AnsiString appdir=AnsiString(dlldir);
        AnsiString SaveFileName = appdir+"KlipKeeper_entries.dat";
        FILE *SaveEntries;
        if ((SaveEntries = fopen(SaveFileName.c_str(),"w+")) != NULL)
        {
            EntryData = "";
            // Start adding entries.
            for (Entries = 0; Entries < AdvConfigForm->lvwClipboard->Items->Count; Entries++)
            {
                EntryData = AdvConfigForm->lvwClipboard->Items->Item[Entries]->Caption + "\n";
        	fputs(EntryData.c_str(), SaveEntries);
            }
            // End Adding entries.
            fclose(SaveEntries);
            EntryData = NULL;
        }
    }
    catch (...)
    {
	ReportErrorToHost("ErrorSaveDataFile");
    }
}
//-----------------------------------------------------------------------
bool TAdvConfigForm::SaveSettings()
{
	// save adv options to file
	bool bretv=false;
	AnsiString appdir=AnsiString(dlldir);
	AnsiString filename=appdir+"KlipKeeper_settings.cfg";
	AnsiString filenametemp=filename+".tmp";

	// open temp ini file
        TMemIniFile *ini;
        ini = new TMemIniFile(filenametemp);
        if (!ini)
                return false;

	// save settings to temp file
	try
        {
            // save info
            bretv=SaveConfigIni(ini);
        }
	catch (...)
        {
            // MyShowMessage("Error loading "+AnsiString(ThisPlugin_DisplayName)+" plugin configuration file '"+filename+"'.");
        }

	// close ini
	delete ini;

	// ok success, so now copy it over the real existing settings file
	if (bretv && FileExists(filenametemp))
        {
            if (FileExists(filename))
            {
                // make backup
                if (FileExists(filename+".bak"))
                        DeleteFile((filename+".bak").c_str());
                CopyFile(filename.c_str(),(filename+".bak").c_str(),true);
                DeleteFile(filename.c_str());
            }
            RenameFile(filenametemp.c_str(),filename.c_str());
        }

	// success
	return bretv;
}
//-----------------------------------------------------------------------
bool TAdvConfigForm::SaveConfigIni(TMemIniFile *ini)
{
	// save ini stuff
	ini->WriteString("Information","File Identification",AnsiString(ThisPlugin_DisplayName));
	ini->WriteString("Information","Version",AnsiString(ThisPlugin_VersionString));
	ini->WriteBool("Settings","Klip.Keeper.IsEnabled",AdvConfigForm->cmdStart->Enabled);
	ini->WriteString("Settings","Klip.MaxNumber", AdvConfigForm->txtMaxNumberClips->Text);
	ini->WriteString("Settings","Klip.MaxLength",AdvConfigForm->txtMaxLengthClips->Text);
	ini->WriteBool("Settings","Klip.Sound.Play",AdvConfigForm->chkPlaySound->Checked);
	ini->WriteString("Settings","Klip.Sound.WavFile",AdvConfigForm->txtPlaySound->Text);
        if (FileExists(AdvConfigForm->txtPlaySound->Text))
        {
            AdvConfigForm->WaveFile->Loadfile(AdvConfigForm->txtPlaySound->Text);
        }
        else
        {
            ini->WriteBool("Settings","Klip.Sound.Play", false);
        }
	// success
	return true;
}
//-----------------------------------------------------------------------
bool TAdvConfigForm::LoadSettings()
{
    // load adv options from file
    bool bretv=false;
    AnsiString appdir=AnsiString(dlldir);
    AnsiString filename=appdir+"KlipKeeper_settings.cfg";
    AnsiString soundfile = appdir+"default.wav";
    
    if (!FileExists(filename))
    {
        AdvConfigForm->ClipboardMonitor->Enabled = true;
        AdvConfigForm->cmdStart->Enabled = false;
        AdvConfigForm->cmdStop->Enabled = true;
        AdvConfigForm->txtMaxNumberClips->Text = "100";
        AdvConfigForm->txtMaxLengthClips->Text = "1000";
        AdvConfigForm->chkPlaySound->Checked = false;

        AdvConfigForm->txtPlaySound->Text = soundfile;
        if (FileExists(AdvConfigForm->txtPlaySound->Text))
        {
            AdvConfigForm->WaveFile->Loadfile(AdvConfigForm->txtPlaySound->Text);
        }
        else
        {
            AdvConfigForm->chkPlaySound->Checked = false;
        }
        return false;
    }

    TMemIniFile *ini;
    ini = new TMemIniFile(filename);
    if (!ini) /* file not found */
    {
        AdvConfigForm->ClipboardMonitor->Enabled = true;
        AdvConfigForm->cmdStart->Enabled = false;
        AdvConfigForm->cmdStop->Enabled = true;
        AdvConfigForm->txtMaxNumberClips->Text = "100";
        AdvConfigForm->txtMaxLengthClips->Text = "1000";
        AdvConfigForm->chkPlaySound->Checked = false;
        AnsiString soundfile = appdir+"default.wav";
        AdvConfigForm->txtPlaySound->Text = soundfile;
        if (FileExists(AdvConfigForm->txtPlaySound->Text))
        {
            AdvConfigForm->WaveFile->Loadfile(AdvConfigForm->txtPlaySound->Text);
        }
        else
        {
            AdvConfigForm->chkPlaySound->Checked = false;
        }
        return false;
    }

    // load settings
    try
    {
        // load info
        AnsiString version;
        version=ini->ReadString("Information","File Identification","");

        if (version!=AnsiString(ThisPlugin_DisplayName))
        {
            AnsiString ErrorWrongConfig;
            ErrorWrongConfig = "Error: "+filename+" is not a configuration file for this plugin. Please delete it if this problem persists.";
            Application->MessageBox(ErrorWrongConfig.c_str(), "Error Wrong Configuration File!", MB_OK | MB_ICONERROR);
        }
        else
        {
            bretv=LoadConfigIni(ini);
        }
    }
    catch (...)
    {
    	AnsiString ErrorLoading;
        ErrorLoading = "Error loading "+AnsiString(ThisPlugin_DisplayName)+" plugin configuration file '"+filename+"'.";
        Application->MessageBox(ErrorLoading.c_str(), "Load Configuration Error!", MB_OK | MB_ICONERROR);
    }

    // close ini
    delete ini;

    // success
    return bretv;
}
//-----------------------------------------------------------------------
bool TAdvConfigForm::LoadConfigIni(TMemIniFile *ini)
{
	// load ini stuff
	AnsiString FileVersion=ini->ReadString("Information","Version","");
	//
        if (ini->ReadBool("Settings","Klip.Keeper.IsEnabled", 0) == 0)
        {
            AdvConfigForm->ClipboardMonitor->Enabled = true;
            AdvConfigForm->cmdStart->Enabled = false;
            AdvConfigForm->cmdStop->Enabled = true;
        }
        else
        {
            AdvConfigForm->ClipboardMonitor->Enabled = false;
            AdvConfigForm->cmdStart->Enabled = true;
            AdvConfigForm->cmdStop->Enabled = false;
        }
        AdvConfigForm->txtMaxNumberClips->Text = ini->ReadString("Settings", "Klip.MaxNumber", "100");
        AdvConfigForm->txtMaxLengthClips->Text = ini->ReadString("Settings", "Klip.MaxLength", "1000");
        AdvConfigForm->chkPlaySound->Checked = ini->ReadBool("Settings", "Klip.Sound.Play", 0);
        AdvConfigForm->txtPlaySound->Text = ini->ReadString("Settings", "Klip.Sound.WavFile", "");
        if (FileExists(AdvConfigForm->txtPlaySound->Text))
        {
            AdvConfigForm->WaveFile->Loadfile(AdvConfigForm->txtPlaySound->Text);
        }
        else
        {
            AdvConfigForm->chkPlaySound->Checked = false;
        }

	// success
	return true;
}
//-----------------------------------------------------------------------
void TAdvConfigForm::SettingsHaveChanged()
{
	// settings have changed, now deal with it
}
//-----------------------------------------------------------------------
void __fastcall TAdvConfigForm::ClipboardMonitorChange(TObject *Sender)
{
	if (shuttingdown)
		return;

    try
    {
        if (Clipboard()->HasFormat(CF_TEXT))
        {
            if (AdvConfigForm->ClipboardMonitor->Enabled == true && AdvConfigForm->lblRefreshClipboardMonitor->Caption == "NO")
            {
                // Pull in the text that was sent to the clipboard.
                ClipboardText = Clipboard()->AsText;

        	// Get klip max length from configuration.
                ClipLineLength = AdvConfigForm->txtMaxLengthClips->Text.ToDouble();

        	// Make sure the klip at least has some data.
                if (ClipboardText.Length() > 0)
                {

                    // Replace the line breaks in each line for
                    // KlipKeeper format which will get replaced
                    // when copied back out.
                    ClipboardText = StringReplace(ClipboardText, "\r" , Def_SlashrReplacement, MyFlags);
                    ClipboardText = StringReplace(ClipboardText, "\n" , Def_SlashnReplacement, MyFlags);

                    // Current Klip Length
                    ClipboardTextLength = ClipboardText.Length();

                    // Check to see if it is within the max
                    // length specified in configuration.
                    if (ClipboardTextLength <= ClipLineLength)
                    {
                    	// Seek and destory any previous KlipKeeper
                        // entry of the same klip then add again.
                        for (FindItem = 0; FindItem < AdvConfigForm->lvwClipboard->Items->Count; FindItem++)
                        {
                            if (AdvConfigForm->lvwClipboard->Items->Item[FindItem]->Caption == ClipboardText)
                            {
                                AdvConfigForm->lvwClipboard->Items->Delete(FindItem);
                                break;
                            }
                        }

                        // Add new klip to KlipKeeper.
                        ListItemToAdd = AdvConfigForm->lvwClipboard->Items->Add();
                        ListItemToAdd->Caption = ClipboardText;

                        // Play sound if enabled.
                        if (AdvConfigForm->chkPlaySound->Checked == true)
                        {
                            // Check if wav file exists before playing.
                            if (FileExists(AdvConfigForm->txtPlaySound->Text))
                            {
                            	// Plays loaded wav file.
                                AdvConfigForm->WaveFile->Play();
                            }
                            else
                            {
                            	// Uncheck play sound.
                                AdvConfigForm->chkPlaySound->Checked = false;
                            }
                        }

                        // Clear ClipboardText string.
                        ClipboardText = "";

                        // If max entries reached delete oldest.
                        MaxEntries = AdvConfigForm->txtMaxNumberClips->Text.ToDouble();
                        if (AdvConfigForm->lvwClipboard->Items->Count > MaxEntries)
                        {
                            AdvConfigForm->lvwClipboard->Items->Delete(0);
                        }
                    }
                }
            }
        }
    }
    catch (...)
    {
	ReportErrorToHost("ClipboardChange");
    }
}
                                                                    // Start Old Trim Code
                                                                    /* if (ClipboardTextLength > ClipLineLength)
                                                                    {
                                                                        ListItem = AdvConfigForm->lvwClipboard->Items->Add();
                                                                        ListItem->Caption = ClipboardText.SetLength(ClipLineLength);
                                                                    }
                                                                    else
                                                                    {
                                                                        ListItem = AdvConfigForm->lvwClipboard->Items->Add();
                                                                        ListItem->Caption = ClipboardText;
                                                                    } */
                                                                    // End Old Trim Code
//---------------------------------------------------------------------------
void __fastcall TAdvConfigForm::cmdStartClick(TObject *Sender)
{
    try
    {
        int MsgBoxResult = Application->MessageBox("Start recording clipboard text to KlipKeeper?", "Start Recording?", MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2);
        if (MsgBoxResult == IDYES) /* Yes */
        {
            AdvConfigForm->ClipboardMonitor->Enabled = true;
            AdvConfigForm->cmdStart->Enabled = false;
            AdvConfigForm->cmdStop->Enabled = true;
            AdvConfigForm->SaveSettings();
            callbackfp_set_strval(hostptr,"window.hide","");
        }
        else
        {
            callbackfp_set_strval(hostptr,"setsearch","kk `");
        }
    }
    catch (...)
    {
	ReportErrorToHost("StartRecordClick");
    }
}
//---------------------------------------------------------------------------
void __fastcall TAdvConfigForm::cmdStopClick(TObject *Sender)
{
    try
    {
        int MsgBoxResult = Application->MessageBox("Stop recording clipboard text to KlipKeeper?", "Stop Recording?", MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2);
        if (MsgBoxResult == IDYES) /* Yes */
        {
            AdvConfigForm->ClipboardMonitor->Enabled = false;
            AdvConfigForm->cmdStop->Enabled = false;
            AdvConfigForm->cmdStart->Enabled = true;
            AdvConfigForm->SaveSettings();
            callbackfp_set_strval(hostptr,"window.hide","");
        }
        else
        {
            callbackfp_set_strval(hostptr,"setsearch","kk `");
        }
    }
    catch (...)
    {
	ReportErrorToHost("EndRecordClick");
    }	
}
//---------------------------------------------------------------------------
void __fastcall TAdvConfigForm::cmdBrowseClick(TObject *Sender)
{
    // Make sure the open dialog successfully executed.
    if (AdvConfigForm->ChooseSound->Execute() == true)
    {
        AdvConfigForm->txtPlaySound->Text = AdvConfigForm->ChooseSound->FileName;
    }
}
//---------------------------------------------------------------------------
void __fastcall TAdvConfigForm::cmdClearSelectedClick(TObject *Sender)
{
    try
    {
        // Make sure at least 1 item is selected before deleting the selected items.
        if (AdvConfigForm->lvwClipboard->Selected!=NULL)
        {
            if (AdvConfigForm->lvwClipboard->SelCount == AdvConfigForm->lvwClipboard->Items->Count)
            {
                // Start deleting all entries.
                AdvConfigForm->lvwClipboard->Items->Clear();
                // End deleting entries and set focus to the clip list.
                AdvConfigForm->lvwClipboard->SetFocus();
            }
            else
            {
                // Start deleting selected entry or entries.
                for (DeleteSelected = AdvConfigForm->lvwClipboard->Items->Count - 1; DeleteSelected > -1; DeleteSelected--)
                {
                    if (AdvConfigForm->lvwClipboard->Items->Item[DeleteSelected]->Selected == true)
                    {
                        AdvConfigForm->lvwClipboard->Items->Delete(AdvConfigForm->lvwClipboard->Items->Item[DeleteSelected]->Index);
                    }
                }
                // End deleting selected entry or entries and set focus to the Klip list.
                AdvConfigForm->lvwClipboard->SetFocus();
            }
        }
        else
        {
            Application->MessageBox("You have not selected any entry or entries to clear.", "Error: No entry or entries selected!", MB_OK | MB_ICONERROR);
        }
    }
    catch (...)
    {
    	ReportErrorToHost("ClearSelected");
    }	
}
//---------------------------------------------------------------------------
void __fastcall TAdvConfigForm::cmdClearAllClick(TObject *Sender)
{
    try
    {
        if (AdvConfigForm->lvwClipboard->Items->Count > 0)
        {
            int MsgBoxResult = Application->MessageBox("Clear all KlipKeeper entries?", "Clear all entries?", MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2);
            if (MsgBoxResult == IDYES) /* Yes */
            {
                AdvConfigForm->lvwClipboard->Items->Clear();
                callbackfp_set_strval(hostptr,"window.hide","");
            }
        }
        else
        {
            Application->MessageBox("No entries in KlipKeeper to clear.", "No entries.", MB_OK | MB_ICONINFORMATION);
        }
    }
    catch (...)
    {
	ReportErrorToHost("ClearAll");
    }	
}
//---------------------------------------------------------------------------
void __fastcall TAdvConfigForm::mnuClearSelectedClick(TObject *Sender)
{
    AdvConfigForm->cmdClearSelected->Click();
}
//---------------------------------------------------------------------------
void __fastcall TAdvConfigForm::mnuFileClearAllClick(TObject *Sender)
{
    AdvConfigForm->cmdClearAll->Click();
}
//---------------------------------------------------------------------------

void __fastcall TAdvConfigForm::mnuFileCloseClick(TObject *Sender)
{
    Close();
}
//---------------------------------------------------------------------------
