//-----------------------------------------------------------------------
// JrPlugin_MyPlugin
//-----------------------------------------------------------------------
// System includes
#include <vcl.h>
#include <windows.h>
#include <stdio.h>
#include <string>
#include <clipbrd.hpp>
//-----------------------------------------------------------------------
// BCB (Borland C++ Builder) Hint
#pragma hdrstop
//-----------------------------------------------------------------------
// our partner header which includes the funcs we need in the right way
#include "JrPlugin_MyPlugin.h"
//-----------------------------------------------------------------------
// see JrPluginFuncs_FARR.h for function definitions we will be implementing here
//-----------------------------------------------------------------------
// BCB advanced configuration form
#include "AdvConfigFormUnit.h"
//-----------------------------------------------------------------------
// global state info
E_SearchStateT current_searchstate=E_SearchState_Stopped;
E_ResultAvailableStateT resultsavailabletype=E_ResultAvailableState_None;
int numresultsavailable=0;
BOOL current_lockstate=FALSE;
bool isready=true;
bool shuttingdown=false;

//-----------------------------------------------------------------------
// KlipKeeper Below
//-----------------------------------------------------------------------
AnsiString NormalIconFile; // Normal Klip Icon File
AnsiString WebIconFile; // URL Klip Icon File
TReplaceFlags MyFlags; // For StringReplace lines of code.
TListItem *ListItemToAdd; // For adding a listview item.
AnsiString ClipboardText; // String for Clipboard Text
AnsiString NewListItemText = ""; // Copy of Clipboard Text before replacements made.
AnsiString PathCaption; // Path Caption.
AnsiString ClipItem; // Clip Item.
AnsiString ClipItemLength; // Trims Clip Item to a new variable with new length/
//-----------------------------------------------------------------------
// KlipKeeper Above
//-----------------------------------------------------------------------
// farr-specific function pointer so we can call to inform the host when we have results
Fp_GlobalPluginCallback_NotifySearchStateChanged callbackfp_notifysearchstatechanged=NULL;
//-----------------------------------------------------------------------
// These functions are our plugin-specific versions of the generic
//  functions exported by the generic DLL shell.
// See JrPlugin_GenericShell.h and .cpp for descriptions of what these
//  functions are supposed to do.
//-----------------------------------------------------------------------
BOOL MyPlugin_DllEntryPoint(HINSTANCE hinst, unsigned long reason, void* lpReserved)
{
    switch (reason)
    {
      case DLL_PROCESS_ATTACH:
        // create forms on load
        if (AdvConfigForm==NULL)
        {
        // create the advanced config form
        AdvConfigForm=new TAdvConfigForm(NULL);
        }
        break;

      case DLL_PROCESS_DETACH:
        break;

      case DLL_THREAD_ATTACH:
        break;

      case DLL_THREAD_DETACH:
        break;
    }
	// success
	return TRUE;
}
//-----------------------------------------------------------------------

BOOL MyPlugin_DoInit()
{
    try
    {
        // set icon names so we can easily access them in the future
    	NormalIconFile = AnsiString(iconfpath) + "KlipKeeperNormal.ico";
    	WebIconFile = AnsiString(iconfpath) + "KlipKeeperUrl.ico";

        // Set StringReplace flags.
        MyFlags = MyFlags << rfReplaceAll << rfIgnoreCase;
        
        // load advanced options?
        AdvConfigForm->LoadSettings();
        // now use settings to do any work
        AdvConfigForm->SettingsHaveChanged();
        // now open up KlipKeeper entries file
        AdvConfigForm->OpenKlipKeeperDataFileToEntries();

	// shutting down flag
	shuttingdown=false;

        // success
        return TRUE;
    }
    catch (...)
    {
	ReportErrorToHost("PluginStart");
        return TRUE;
    }
}
//-----------------------------------------------------------------------
BOOL MyPlugin_DoShutdown()
{
    try
    {
	shuttingdown=true;
        // Delete the advanced config form
        if (AdvConfigForm!=NULL)
        {
            AdvConfigForm->ClipboardMonitor->Enabled = false;
            // now save KlipKeeper entries file
            AdvConfigForm->SaveEntriesToKlipKeeperDataFile();

            delete AdvConfigForm;
            AdvConfigForm=NULL;
        }
        // success
        return TRUE;
    }
    catch (...)
    {
	ReportErrorToHost("PluginShutdown");
        return TRUE;
    }
}
//-----------------------------------------------------------------------
BOOL MyPlugin_GetStrVal(const char* varname,char *destbuf, int maxlen)
{
    // FARR: default values for FARR fields
    if (strcmp(varname,DEF_FieldName_RegexStr)==0)
        {
        strcpy(destbuf,ThisPlugin_FARR_DefaultRegex);
        return TRUE;
        }
    if (strcmp(varname,DEF_FieldName_RegexFilterStr)==0)
            {
            strcpy(destbuf,ThisPlugin_FARR_DefaultRegexFilter);
            return TRUE;
            }
    if (strcmp(varname,DEF_FieldName_KeywordStr)==0)
            {
            strcpy(destbuf,ThisPlugin_FARR_DefaultKeywordStr);
            return TRUE;
            }
    if (strcmp(varname,DEF_FieldName_ScoreStr)==0)
            {
            strcpy(destbuf,ThisPlugin_FARR_DefaultScoreStr);
            return TRUE;
            }
    if (strcmp(varname,DEF_FieldName_AliasStr)==0)
            {
            strcpy(destbuf,ThisPlugin_FARR_DefaultAlias);
            return TRUE;
            }

    // not found
    return FALSE;
}
//-----------------------------------------------------------------------
BOOL MyPlugin_SetStrVal(const char* varname, void *val)
{
    // farr host will pass us function pointer we will call
    if (strcmp(varname,DEF_FieldName_NotifySearchCallbackFp)==0)
            callbackfp_notifysearchstatechanged = (Fp_GlobalPluginCallback_NotifySearchStateChanged)val;

    return FALSE;
}
//-----------------------------------------------------------------------
BOOL MyPlugin_SupportCheck(const char* testname, int version)
{
    // ATTN: we support farr interface
    if (strcmp(testname,DEF_FARRAPI_IDENTIFIER)==0)
            return TRUE;

    // otherwise we don't support it
    return FALSE;
}
//-----------------------------------------------------------------------
BOOL MyPlugin_DoAdvConfig()
{
    // Form style?
    bool doshowmodal = true;
    //
    // create it if not created yet
    if (AdvConfigForm==NULL)
            AdvConfigForm=new TAdvConfigForm(NULL);
    //
    // show modal or just show?
    if (doshowmodal)
    {
        AdvConfigForm->ShowModal();
    }
    else
    {
        AdvConfigForm->Show();
    }

    // success
    return TRUE;
}


BOOL MyPlugin_DoShowReadMe()
{
    // by default show the configured readme file
    std::string fname = std::string(dlldir)+std::string("\\")+std::string(ThisPlugin_ReadMeFile);
    ShellExecute(NULL, "open", fname.c_str(), NULL,NULL, SW_SHOWNORMAL);
    // success
    return TRUE;
}
//-----------------------------------------------------------------------
BOOL MyPlugin_SetState(int newstateval)
{
    // usually there is nothing to do here

    // success
    return TRUE;
}
//-----------------------------------------------------------------------
// These functions are FARR-SPECIFIC and have no counterparts in the
//  generic DLL shell.
//
// Note from the function declarations that these functions are EXPORTED
//  from the DLL so that the host can call them.
//-----------------------------------------------------------------------
// Host informs us that a search is begining and what the searchstring is
// The Plugin DLL could do all "searching" here (if there is indeed any to do)
// And compute all results within this function, OR simply begin a threaded asynchronous search now.
//
// Normally the string you care about is searchstring_lc_nokeywords, thats the search string with all special +modifier keywords removed, and
//  the alias keyword for this plugin removed, so all thats left is the string that should effect the results of the search.
//
// returns TRUE only if the plugin decides now that no more searching should be done by any other plugin or builtin
//
// NOTE: if asynchronous searching is to be done, make sure to only set current_searchstate=E_SearchState_Stopped when done!
//-----------------------------------------------------------------------
PREFUNCDEF BOOL EFuncName_Inform_SearchBegins(const char* searchstring_raw, const char *searchstring_lc_nokeywords, BOOL explicitinvocation)
{
    if (!explicitinvocation)
            return FALSE;
    // call farr search - nothing to do really for this plugin but return all results
    return DoFarrSearchBegin(searchstring_lc_nokeywords);
}
//-----------------------------------------------------------------------
// Host informs us that regular expression match **HAS** occured, and to begin doing that we need to do on such a match
// The Plugin DLL could do all "searching" here (if there is indeed any to do)
// And compute all results within this function, OR simply begin a threaded asynchronous search now.
//
// To access the group capture strings, use regexcharps[1] to match the first group, etc.
//
// returns TRUE only if the plugin decides now that no more searching should be done by any other plugin or builtin
//
// NOTE: if asynchronous searching is to be done, make sure to only set current_searchstate=E_SearchState_Stopped when done!
//
PREFUNCDEF BOOL EFuncName_Inform_RegexSearchMatch(const char* searchstring_raw, const char *searchstring_lc_nokeywords, int regexgroups, char** regexcharps)
{
    // call farr search - nothing to do really for this plugin but return all results
    return DoFarrSearchBegin(searchstring_lc_nokeywords);
}
//-----------------------------------------------------------------------
// Host informs us that a search has completed or been interrupted, and we should stop any processing we might be doing assynchronously
//
// NOTE: make sure to set current_searchstate=E_SearchState_Stopped;
//-----------------------------------------------------------------------
PREFUNCDEF void EFuncName_Inform_SearchEnds()
{
    // stop search and set this
    current_searchstate=E_SearchState_Stopped;

    // ATTN: test clear results
    numresultsavailable=0;
    resultsavailabletype=E_ResultAvailableState_None;

    // notify host that our state has changed
    ExecuteCallback_SearchStateChanged();

    return;
}
//-----------------------------------------------------------------------
// Host informs us that our window is about to appear
PREFUNCDEF void EFuncName_Inform_WindowIsHiding(HWND hwndp)
{
    return;
}
//-----------------------------------------------------------------------
// Host informs us that our window is about to disappear
//-----------------------------------------------------------------------
PREFUNCDEF void EFuncName_Inform_WindowIsUnHiding(HWND hwndp)
{
    return;
}
//-----------------------------------------------------------------------
// Host is asking us if we want a certain feature or call
// We should use a switch function here to tell what is being asked
//
// Returns TRUE or FALSE depending on whether we want the feature
//-----------------------------------------------------------------------
PREFUNCDEF BOOL EFuncName_Ask_WantFeature(E_WantFeaturesT featureid)
{
    switch (featureid)
    {
        case E_WantFeatures_searchinput_regex:
                // do we want to search on regular expression matches
                return TRUE;
        case E_SupportFeatures_searchinput_explicit:
                // do we want to search on alias match
                return TRUE;
        case E_SupportFeatures_searchinput_all:
                // do we want to search on all search expressions
                return FALSE;
        case E_SupportFeatures_tryhandle_trigger:
                // do we want to try to handle triggers when a result is launched
                return TRUE;
        case E_SupportFeatures_addinfo_files:
                // do we want to try to handle triggers when a result is launched
                return FALSE;
        case E_SupportFeatures_scoreall_files:
                // do we want to try to score EVERY file considered (SLOWS DOWN SEARCH!)
                return FALSE;
        case E_SupportFeatures_scoregood_files:
                // do we want to try to score files after other scoring has determined they are a viable result
                return FALSE;
        case E_SupportFeatures_dominate_results:
                // do we want our results to dominate and hide any others
                return FALSE;
    }

    // fell through - so not supported
    return FALSE;
}
//-----------------------------------------------------------------------
// Host wants to know the search state of the plugin
// returns:  E_SearchState_Stopped=0 || E_SearchState_Searching=1
//
// NOTE: this value will be based on a dll global variable
//-----------------------------------------------------------------------
PREFUNCDEF E_SearchStateT EFuncName_Ask_SearchState()
{
    // this will be tracked elsewhere
    return current_searchstate;
}
//-----------------------------------------------------------------------
// Host wants to know what kind if any results we have available
//
// NOTE: return is from (see JrPluginFuncs_FARR.h) enum E_ResultAvailableStateT { E_ResultAvailableState_None=0, E_ResultAvailableState_ItemResuls=1, E_ResultAvailableState_DisplayText=2, E_ResultAvailableState_DisplayHtml=3, E_ResultAvailableState_Grid=4, E_ResultAvailableState_WinTakeover=5 };
//-----------------------------------------------------------------------
PREFUNCDEF E_ResultAvailableStateT EFuncName_Ask_IsAnyResultAvailable()
{
    // this will be tracked elsewhere
    return resultsavailabletype;
}
//-----------------------------------------------------------------------
// Host wants to know how many item results are available
//-----------------------------------------------------------------------
PREFUNCDEF int EFuncName_Ask_HowManyItemResultsAreReady()
{
    // this will be tracked elsewhere
    return numresultsavailable;
}
//-----------------------------------------------------------------------
// Host calls this before iterating through the result requests
// It will be TRUE on lock before retrieving
//  and then FALSE when all done retrieving
// In this way the dll should refrain from altering the results until retrieval is done
// One way to do this for asynchronous searching is to MOVE results to a return store
//  on TRUE lock, and then free on FALSE.
//
// Returns TRUE on success
//
//-----------------------------------------------------------------------
PREFUNCDEF BOOL EFuncName_Request_LockResults(BOOL dolock)
{
    // set lock
    current_lockstate=dolock;

    if (dolock==false)
            {
            // on unlocking they have retrieved all the results, so CLEAR the results or they will be found again
            numresultsavailable=0;
            resultsavailabletype=E_ResultAvailableState_None;
            // notify host that our state has changed
            ExecuteCallback_SearchStateChanged();
            }

    // success
    return TRUE;
}
//-----------------------------------------------------------------------
// Host is requesting one of our results
// Fill the destination buffers without exceding maxlen
//
// resultpostprocmodep is from enum E_ResultPostProcessingT { E_ResultPostProcessing_ImmediateDisplay = 0, E_ResultPostProcessing_AddScore = 1, E_ResultPostProcessing_MatchAgainstSearch = 2};
//
// Returns TRUE on success
//-----------------------------------------------------------------------
PREFUNCDEF BOOL EFuncName_Request_ItemResultByIndex(int resultindex, char *destbuf_path, char *destbuf_caption, char *destbuf_groupname, char *destbuf_iconfilename, void** tagvoidpp, int maxlen, E_ResultPostProcessingT *resultpostprocmodep, int *scorep, E_EntryTypeT *entrytypep)
{
    int ClipItemNumber = (AdvConfigForm->lvwClipboard->Items->Count - resultindex)-1;

    ClipItem = AdvConfigForm->lvwClipboard->Items->Item[ClipItemNumber]->Caption;
    PathCaption = "The length of this klip is " + AnsiString(ClipItem.Length()) + " characters.";
    
    if (ClipItem.Length() > 60)
    {
    	ClipItemLength = ClipItem;
    	ClipItemLength.SetLength(60);
        ClipItem = ClipItemLength + "... (" + ClipItem.Length() + ")";
    }

    // clear values sample:
    strcpy(destbuf_groupname,"");

    *tagvoidpp = (void*) ClipItemNumber;


    // how result is handled
    // return E_ResultPostProcessing_ImmediateDisplay if we have done the filtering and the result should be displayed
    // return E_ResultPostProcessing_AddScore if the host should evaluate the result and add a score to it but still be forced to display it
    // return E_ResultPostProcessing_MatchAgainstSearch if the host should evaluate the result and decide whether to show it based on match of search words
    //
    // in this case we just return all entries and let host do the matching
    *resultpostprocmodep=E_ResultPostProcessing_MatchAgainstSearch;

    // do we want to modify the score assigned by host?  usually not.
    // *scorep+=100;

    // type (file is same as url)
    *entrytypep=E_EntryType_UNKNOWN;

    // ok fill the result data
    strncpy(destbuf_path, PathCaption.c_str(), maxlen);
    strncpy(destbuf_caption, ClipItem.c_str(), maxlen);

    int wpos = ClipItem.Pos("://");

    if ( (wpos > 0 && wpos < 10) || ClipItem.SubString(0,4) == "www.")
    {
        strncpy(destbuf_iconfilename, WebIconFile.c_str(), maxlen);
    }
    else
    {
        strncpy(destbuf_iconfilename, NormalIconFile.c_str(), maxlen);
    }

    // ok filled one
    return TRUE;
}
//-----------------------------------------------------------------------
// Host is asking us to return a char* to a LONG result string
//
// NOTE: It is up to us to free this char* when the lock is set to false above OR
//  on the next call, or on search end, but we MUST keep it valid until then.
//-----------------------------------------------------------------------
PREFUNCDEF BOOL EFuncName_Request_TextResultCharp(char **charp)
{
    // signify we have none
    *charp=NULL;

    // none returned
    return FALSE;
}
//-----------------------------------------------------------------------
// Host is calling us with a result item about to be triggered
// We can choose here to takeover the launching of it, OR simply do something and let normal launching continue
//
// Return TRUE to takeover launching and prevent all other further launching
// or FALSE to continue launching after we return
//-----------------------------------------------------------------------
PREFUNCDEF BOOL EFuncName_Allow_ProcessTrigger(const char* destbuf_path, const char* destbuf_caption, const char* destbuf_groupname, int pluginid,int thispluginid, int score, E_EntryTypeT entrytype, void* tagvoidp, BOOL *closeafterp)
{
    // does this plugin want to take over launching of this result?
    try
    {
        if (thispluginid!=pluginid)
        {
        // if we didnt create it, let default handler do it
        return FALSE;
        }
        else
        {
        // this plugin created this item, does it want to launch it specially?
            int ClipItemNumber = (int) (tagvoidp);
            
            if (ClipItemNumber < AdvConfigForm->lvwClipboard->Items->Count)
            {
            	ClipboardText = AdvConfigForm->lvwClipboard->Items->Item[ClipItemNumber]->Caption;
                NewListItemText = ClipboardText;
                AdvConfigForm->lvwClipboard->Items->Delete(ClipItemNumber);

                ClipboardText = StringReplace(ClipboardText, Def_SlashrReplacement , "\r", MyFlags);
                ClipboardText = StringReplace(ClipboardText, Def_SlashnReplacement , "\n", MyFlags);
        	//
                if (AdvConfigForm->cmdStart->Enabled == true)
                {
                    ListItemToAdd = AdvConfigForm->lvwClipboard->Items->Add();
                    ListItemToAdd->Caption = NewListItemText;
                }
                else if (AdvConfigForm->cmdStop->Enabled == true)
                {
                    AdvConfigForm->ClipboardMonitor->Enabled = false;
                    Clipboard()->SetTextBuf(ClipboardText.c_str());
                    AdvConfigForm->ClipboardMonitor->Enabled = true;
                }
            }
         }
        return TRUE;
    }
    catch (...)
    {
	ReportErrorToHost("ResultLaunch");
        return FALSE;
    }
}
//-----------------------------------------------------------------------
// Host is asking us if we want to modify the score of an item
// Original score in in *score, we can now modify it if we want
//
// Return TRUE if we modified it, or FALSE if not
//
// NOTE: set score to -9999 to eliminate it from results
//-----------------------------------------------------------------------
PREFUNCDEF BOOL EFuncName_Do_AdjustResultScore(const char* itempath, int *scorep)
{
    // we didnt modify it
    return FALSE;
}
//-----------------------------------------------------------------------
// Helper Functions the plugin calls
//-----------------------------------------------------------------------
void ExecuteCallback_SearchStateChanged()
{
    // tell the host that our state or resultcount has changed
    if (callbackfp_notifysearchstatechanged)
    {
        callbackfp_notifysearchstatechanged(hostptr,numresultsavailable,current_searchstate);
    }
}
//-----------------------------------------------------------------------
BOOL DoFarrSearchBegin(const char *searchstring_lc_nokeywords)
{
    try {
        // FARR search -- we just return test results and let the farr program filter them based on what user types
        if (!isready)
        {
            // statusbar
            callbackfp_set_strval(hostptr,"statusbar","KlipKeeper plugin is busy...");
            // search can continue by others?
            return FALSE;
        }
        else if (strcmp(searchstring_lc_nokeywords,"about")==0 || strcmp(searchstring_lc_nokeywords,"help")==0 || strcmp(searchstring_lc_nokeywords,"`")==0)
        {
            if (AdvConfigForm->cmdStart->Enabled == false) {
                AdvConfigForm->lblRefreshClipboardMonitor->Caption = "YES";
                AdvConfigForm->ClipboardMonitor->Enabled = false;
                AdvConfigForm->ClipboardMonitor->Enabled = true;
                AdvConfigForm->lblRefreshClipboardMonitor->Caption = "NO";
            }
            // Set height of the richedit control.
            callbackfp_set_strval(hostptr,"window.richeditheight",(char*)260);
            callbackfp_set_strval(hostptr,"statusbar","KlipKeeper Commands");
                        
            AnsiString AboutPlugin;
            AboutPlugin = AnsiString(ThisPlugin_DisplayName)+ " - "+AnsiString(ThisPlugin_VersionString)+ " (" +AnsiString(ThisPlugin_ReleaseDateString)+") by "+AnsiString(ThisPlugin_Author)+"\r\n\r\n";
            if (AdvConfigForm->cmdStart->Enabled == false)
            {
                AboutPlugin+= " KlipKeeper Recording = ON\r\n";
            }
            else if (AdvConfigForm->cmdStop->Enabled == false)
            {
                AboutPlugin+= " KlipKeeper Recording = OFF\r\n";
            }
            if (AdvConfigForm->chkPlaySound->Checked == true)
            {
                AboutPlugin+= " KlipKeeper Klip Sound = ON\r\n";
            }
            else
            {
                AboutPlugin+= " KlipKeeper Klip Sound = OFF\r\n";
            }
            AboutPlugin+= " Sound File = " + AdvConfigForm->txtPlaySound->Text + "\r\n";
            AboutPlugin+= " Max Klips = " + AnsiString(AdvConfigForm->txtMaxNumberClips->Text) + "\r\n";
            AboutPlugin+= " Max Klip Length = " + AnsiString(AdvConfigForm->txtMaxLengthClips->Text) + "\r\n";
            AboutPlugin+= " Total Klips Saved = " + AnsiString(AdvConfigForm->lvwClipboard->Items->Count) + "\r\n\r\n";
            AboutPlugin+= " kk `  -- Currently viewing.\r\n";
            AboutPlugin+= " kk `t -- Toggles recording all text that the clipboard receives.\r\n";
            AboutPlugin+= " kk `p -- Opens up the gui of the configuration.\r\n";
            AboutPlugin+= " kk `c -- Clears all KlipKeeper entries.\r\n";

            // Set height of the richedit control.
            callbackfp_set_strval(hostptr,"window.richeditheight",(char*)260);
            callbackfp_set_strval(hostptr,"window.richeditmode", AboutPlugin.c_str());
            callbackfp_set_strval(hostptr,"statusbar","KlipKeeper Commands");
            if (strcmp(searchstring_lc_nokeywords,"about")==0 || strcmp(searchstring_lc_nokeywords,"help")==0)
            {
            	callbackfp_set_strval(hostptr,"setsearch","kk `");
            }
            // search can continue by others?
            return FALSE;
        }
        else if (strcmp(searchstring_lc_nokeywords,"`p")==0 || strcmp(searchstring_lc_nokeywords,"`pref")==0)
        {
            callbackfp_set_strval(hostptr,"window.hide","");
            AdvConfigForm->Show();
            AdvConfigForm->lblCalledFromKKPrefs->Caption = "YES";
            // search can continue by others?
            return FALSE;
        }
        else if (strcmp(searchstring_lc_nokeywords,"`c")==0)
        {
            AdvConfigForm->cmdClearAll->Click();
            // search can continue by others?
            return FALSE;
        }
        else if (strcmp(searchstring_lc_nokeywords,"`t")==0 || strcmp(searchstring_lc_nokeywords,"`start")==0 || strcmp(searchstring_lc_nokeywords,"`stop")==0 && AdvConfigForm->cmdStart->Enabled == true)
        {
            if (AdvConfigForm->cmdStart->Enabled == true) {
                AdvConfigForm->cmdStart->Click();
            }
            else
            {
            	AdvConfigForm->cmdStop->Click();
            }
            // search can continue by others?
            return FALSE;
        }

        // start and end of search state
        current_searchstate=E_SearchState_Searching;

        // ok now results are available right away
        numresultsavailable=AdvConfigForm->lvwClipboard->Items->Count;
        resultsavailabletype=E_ResultAvailableState_ItemResuls;
        // done
        current_searchstate=E_SearchState_Stopped;

        // notify host that our state has changed
        ExecuteCallback_SearchStateChanged();

        // search can continue by others
        return FALSE;
    }
    catch (...)
    {
	ReportErrorToHost("SearchBegin");
        return FALSE;
    }
}
//-----------------------------------------------------------------------
