//---------------------------------------------------------------------------
// AdvConfigFormUnit.cpp
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// System includes and C++ Builder Stuff
#include <vcl.h>
#pragma hdrstop
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Application includes
#include "JrPlugin_MyPlugin.h"
#include "AdvConfigFormUnit.h"
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// C++ Builder stuff
#pragma package(smart_init)
#pragma link "BrwsFldr"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// Global Form Instance Pointer
TAdvConfigForm *AdvConfigForm=NULL;
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
__fastcall TAdvConfigForm::TAdvConfigForm(TComponent* Owner)
	: TForm(Owner)
{
	// nothing to do in constructor
	initialized=false;
}
//---------------------------------------------------------------------------





//---------------------------------------------------------------------------
void TAdvConfigForm::DoInitialize()
{
	// called from PopUpWisdom.cpp
	if (initialized)
		return;

	quote_activecount=0;

	// load settings
	LoadSettings();

	// randomize
	RandomizeInit();

	// recan books
	ReScanBookDir();

	// stupid *(&^#(#*&#$& freaks if we dont do this
	if (CheckBoxFirstUse->Checked || quote_activecount<=0)
		{
		// show the advanced dialog on first use?
		//Show();
		CheckBoxFirstUse->Checked=false;
		}

	// ok we are initialized
	initialized=true;
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
void TAdvConfigForm::RandomizeInit()
{
	srand((unsigned)time(0));
}
//---------------------------------------------------------------------------



//-----------------------------------------------------------------------
void __fastcall TAdvConfigForm::ButtonOkClick(TObject *Sender)
{
	// save settings
	SaveDisabledList();
	SaveSettings();
	Close();
	ReScanBookDir();
}
//---------------------------------------------------------------------------




//-----------------------------------------------------------------------
void __fastcall TAdvConfigForm::FormClose(TObject *Sender, TCloseAction &Action)
{
	// save any manual changes to parser vars? or this is done automatically elsewhere

	// ATTN: we need to hide if we create the form with Application->CreateForm since its main form and wont hide otherwise
	Action=caHide;
	Hide();
}
//---------------------------------------------------------------------------









//-----------------------------------------------------------------------
bool TAdvConfigForm::SaveSettings()
{
	// save adv options to file
	bool bretv=false;
	AnsiString appdir=AnsiString(dlldir);
	AnsiString filename=appdir+"advoptions.cfg";
	AnsiString filenametemp=filename+".tmp";

	// open temp ini file
        TMemIniFile *ini;
        ini = new TMemIniFile(filenametemp);
        if (!ini)
                return false;

	// save settings to temp file
	try
		{
		// save info
		bretv=SaveConfigIni(ini);
		}
	catch (...)
		{
		// MyShowMessage("Error loading "+AnsiString(ThisPlugin_DisplayName)+" plugin configuration file '"+filename+"'.");
		}

	ini->UpdateFile();

	// close ini
	delete ini;

	// ok success, so now copy it over the real existing settings file
	if (bretv && FileExists(filenametemp))
		{
		if (FileExists(filename))
			{
			// make backup
			if (FileExists(filename+".bak"))
				DeleteFile((filename+".bak").c_str());
			CopyFile(filename.c_str(),(filename+".bak").c_str(),true);
			DeleteFile(filename.c_str());
			}
		RenameFile(filenametemp.c_str(),filename.c_str());
		}

	// success
	return bretv;
}


bool TAdvConfigForm::SaveConfigIni(TMemIniFile *ini)
{
	// save ini stuff

	ini->WriteString("Information","File Identification",AnsiString(ThisPlugin_DisplayName));
	ini->WriteString("Information","Version",AnsiString(ThisPlugin_VersionString));

	// make a string of the disabled book memo
	AnsiString disabledbooklist = MemoDisabledBooks->Lines->Text;
	FindReplaceField(disabledbooklist,"\r\n",";;;");
	ini->WriteString("Settings","DisabledBookList",disabledbooklist);
	// save first use
	ini->WriteBool("Settings","FirstUse",CheckBoxFirstUse->Checked);
	//
	ini->WriteString("Settings","BookDir",EditBookDir->Text);

	// success
	return true;
}
//-----------------------------------------------------------------------



//-----------------------------------------------------------------------
bool TAdvConfigForm::LoadSettings()
{
	// load adv options from file
	bool bretv=false;
	AnsiString appdir=AnsiString(dlldir);
	AnsiString filename=appdir+"advoptions.cfg";

	if (!FileExists(filename))
		return false;

        TMemIniFile *ini;
        ini = new TMemIniFile(filename);
        if (!ini)
        	{
		// file not found
                return false;
                }

	// load settings
	try
		{
		// load info
		AnsiString version;
		version=ini->ReadString("Information","File Identification","");
		if (version!=AnsiString(ThisPlugin_DisplayName))
			{
			ShowMessage("Error: "+filename+" is not a configuration file for this plugin. Please delete it if this problem persists.");
			}
		else
			{
			bretv=LoadConfigIni(ini);
			}
		}
	catch (...)
		{
		ShowMessage("Error loading "+AnsiString(ThisPlugin_DisplayName)+" plugin configuration file '"+filename+"'.");
		}

	// close ini
	delete ini;

	// success
	return bretv;
}


bool TAdvConfigForm::LoadConfigIni(TMemIniFile *ini)
{
	// load ini stuff
	AnsiString FileVersion=ini->ReadString("Information","Version","");

	// make a string of the disabled book memo
	AnsiString disabledbooklist;
	disabledbooklist=ini->ReadString("Settings","DisabledBookList","");
	FindReplaceField(disabledbooklist,";;;","\r\n");
	MemoDisabledBooks->Lines->Text = disabledbooklist;

	// save first use
	CheckBoxFirstUse->Checked=ini->ReadBool("Settings","FirstUse",CheckBoxFirstUse->Checked);
	//
	EditBookDir->Text=ini->ReadString("Settings","BookDir",EditBookDir->Text);

	// success
	return true;
}
//-----------------------------------------------------------------------





//-----------------------------------------------------------------------
void TAdvConfigForm::SettingsHaveChanged()
{
	// settings have changed, now deal with it
}
//-----------------------------------------------------------------------




//---------------------------------------------------------------------------
int TAdvConfigForm::FindReplaceField(AnsiString &str,AnsiString findstr,AnsiString repstr)
{
	int len=findstr.Length();
	int slen=str.Length();
	int pos;
	int replacements=0;
	for (;;)
		{
		pos=str.Pos(findstr);
		if (pos<=0)
			break;
		str=str.SubString(1,pos-1)+repstr+str.SubString(pos+len,(slen-(pos+len))+1);
		slen=str.Length();
		++replacements;
		}
	return replacements;
}
//---------------------------------------------------------------------------
















//---------------------------------------------------------------------------
void __fastcall TAdvConfigForm::EditThisBookItemClick(TObject *Sender)
{
	// grab file name and shell out to edit it
	BookStore* bookstorep=get_selectedbookstorep();
	if (bookstorep==NULL)
		return;
	// now get filename and edit
	std::string filename=bookstorep->get_filename();
	ShellExecute(NULL, "open", filename.c_str(), NULL, NULL, SW_SHOWNORMAL);
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
void __fastcall TAdvConfigForm::VisitWebpageforBookItemClick(TObject *Sender)
{
	// grab file name and shell out to edit it
	BookStore* bookstorep=get_selectedbookstorep();
	if (bookstorep==NULL)
		return;
	std::string urlstring=bookstorep->get_url();
	if (urlstring!="")
		{
		ShellExecute(NULL, "open", urlstring.c_str(), NULL, NULL, SW_SHOWNORMAL);
		}
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
void __fastcall TAdvConfigForm::MenuItemExportMultilinePercentsClick(TObject *Sender)
{
	// export multiline with % between
	ExportBook(true,"\n%\n");
}


void __fastcall TAdvConfigForm::MenuItemExportSingleLinesClick(TObject *Sender)
{
	ExportBook(false,"\n");
}
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
void TAdvConfigForm::ExportBook(bool userealnewlines,AnsiString separator)
{
	bool wasquotesloaded;

	// get current book
	BookStore* bookstorep=get_selectedbookstorep();
	if (bookstorep==NULL)
		return;
	// now add info to display
	std::string linestr=bookstorep->get_title();
	std::string fname = bookstorep->get_filename();
	SaveDialogExport->Title="Exported "+AnsiString(linestr.c_str())+" As..";
        //
	AnsiString ansifname=ExtractFileName(AnsiString(fname.c_str()));
	ansifname="exported_"+ansifname;
	SaveDialogExport->FileName=ansifname;

	if (SaveDialogExport->Execute())
		{
		// ok export
		ansifname=SaveDialogExport->FileName;
		bookstorep->ExportBook(ansifname.c_str(),userealnewlines,separator.c_str());
		}
}
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
void __fastcall TAdvConfigForm::RescanBookDirectoryItemClick(TObject *Sender)
{
	SaveDisabledList();
	ReScanBookDir();
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
void TAdvConfigForm::SaveDisabledList()
{
	// save list of all disabled items to hidden memo
	MemoDisabledBooks->Lines->Clear();
	int entries=CheckListBoxBooks->Items->Count;
	for (int index=0;index<entries;++index)
		{
		if (CheckListBoxBooks->State[index]==cbUnchecked)
			{
			// add disabled item title
			MemoDisabledBooks->Lines->Append(CheckListBoxBooks->Items->Strings[index]);
			}
		}
}
//---------------------------------------------------------------------------





//---------------------------------------------------------------------------
BookStore* TAdvConfigForm::get_selectedbookstorep()
{
	// return pointer to currently selected book
	int bookindex=CheckListBoxBooks->ItemIndex;
	if (bookindex==-1)
		return NULL;
	return get_bookbyindex(bookindex);
}
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
void __fastcall TAdvConfigForm::CheckListBoxBooksClick(TObject *Sender)
{
	// user selects an item, so we show info about the book in the box
	RichEditBookInfo->Lines->Clear();
	BookStore* bookstorep=get_selectedbookstorep();
	if (bookstorep==NULL)
		return;
	// now add info to display
	std::string linestr=bookstorep->get_title() + " (" + JR_StringifyLong(bookstorep->get_quotecount())+ " entries)";
	if (bookstorep->get_quotesloaded())
		linestr+=" - cached";
	else
		linestr+=" - on disk";
	RichEditBookInfo->Lines->Append(linestr.c_str());
	RichEditBookInfo->Lines->Append((bookstorep->get_url()).c_str());
	RichEditBookInfo->Lines->Append((bookstorep->get_info()).c_str());
}
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
void __fastcall TAdvConfigForm::FormShow(TObject *Sender)
{
	CheckListBoxBooksClick(NULL);
}
//---------------------------------------------------------------------------














//---------------------------------------------------------------------------
void TAdvConfigForm::ReScanBookDir()
{
	// collect all text files in book dir
	quotefiles.set_FileMask("*.txt",false,false);

	bookdir = EditBookDir->Text;

	AnsiString realbookdir = bookdir;
	if (!DirectoryExists(realbookdir))
		{
		realbookdir = basedir+bookdir;
		if (!DirectoryExists(bookdir))
			realbookdir = basedir+"\\"+bookdir;
		}

	quotefiles.set_BaseDirectory(realbookdir.c_str());
	//
	quotefiles.Clear();
	quotefiles.RescanForFiles();
	// redisable the disabled
	DisableBooks();
	// ok now fill the options form checklist
	FillBookChecklist();
	// ok now read full quotes from enabled books
	CacheEnabledBookQuotes(true);
	// recalc available quotes
	RecalcQuoteCount();
}
//---------------------------------------------------------------------------








//---------------------------------------------------------------------------
void TAdvConfigForm::DisableBooks()
{
	// walk through our disabled list and disable any books on it
	ConfigEntry* entryp;
	BookStore* bookstorep;
	AnsiString titlestr;
	int entries=quotefiles.get_EntryCount();
	for (int index=0;index<entries;++index)
		{
		entryp=quotefiles.get_Entry(index);
		bookstorep=entryp->get_bookstorep();
		titlestr=AnsiString((bookstorep->get_title()).c_str());
		if (titlestr=="")
			{
			titlestr=AnsiString((entryp->get_LabelString()).c_str());
			}
		if (MemoDisabledBooks->Lines->IndexOf(titlestr)>-1 || CheckBoxFirstUse->Checked)
			{
			// found on disable list so disable
			bookstorep->set_enabled(false);
			}
		else
			{
			// not found on disable list so enable
			bookstorep->set_enabled(true);
			}
		}
}


void TAdvConfigForm::RecalcQuoteCount()
{
	quote_activecount=0L;
	//
	ConfigEntry* entryp;
	BookStore* bookstorep;
	AnsiString titlestr;
	int entries=quotefiles.get_EntryCount();
	for (int index=0;index<entries;++index)
		{
		entryp=quotefiles.get_Entry(index);
		bookstorep=entryp->get_bookstorep();
		if (bookstorep->get_enabled())
			{
			quote_activecount+=bookstorep->get_quotecount();
			}
		}
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
void TAdvConfigForm::FillBookChecklist()
{
	// fill book list with books we found
	CheckListBoxBooks->Clear();
	// fill
	ConfigEntry* entryp;
	BookStore* bookstorep;
	AnsiString titlestr;
	int entries=quotefiles.get_EntryCount();
	for (int index=0;index<entries;++index)
		{
		entryp=quotefiles.get_Entry(index);
		bookstorep=entryp->get_bookstorep();
		titlestr=AnsiString((bookstorep->get_title()).c_str());
		if (titlestr=="")
			titlestr=AnsiString((entryp->get_LabelString()).c_str());
		CheckListBoxBooks->Items->Append(titlestr);
		if (bookstorep->get_enabled())
			CheckListBoxBooks->State[index]=cbChecked;
		else
			CheckListBoxBooks->State[index]=cbUnchecked;
		}
}


void TAdvConfigForm::CacheEnabledBookQuotes(bool forcereload)
{
	// force enabled books into memory, force unenabled out
	// if forcereload==true then force re-reading of files
	// if forcereload==false then if book is already in memory do nothing
	ConfigEntry* entryp;
	BookStore* bookstorep;
	AnsiString titlestr;
	int entries=quotefiles.get_EntryCount();
	for (int index=0;index<entries;++index)
		{
		entryp=quotefiles.get_Entry(index);
		bookstorep=entryp->get_bookstorep();
		if (bookstorep->get_enabled())
			{
			if (!(bookstorep->get_quotesloaded()))
				bookstorep->NeedsQuotes(true);
			if (forcereload)
				{
				bookstorep->UnloadQuotes();
				bookstorep->NeedsQuotes(true);
				}
			}
		else
			{
			bookstorep->NeedsQuotes(false);
			}
		}
}
//---------------------------------------------------------------------------








//---------------------------------------------------------------------------
AnsiString TAdvConfigForm::GetQuoteByIndex(int quotenum, BookStore* &bookstorep)
{
	// get a quote by its absolute index

	// now we need to grab it
	long quoteoffset=quotenum;
	long thiscount;

	// reset
	AnsiString quotestring;

	// walk through like roulette
	ConfigEntry* entryp;
	AnsiString titlestr;
	int entries=quotefiles.get_EntryCount();
	for (int index=0;index<entries;++index)
		{
		entryp=quotefiles.get_Entry(index);
		bookstorep=entryp->get_bookstorep();
		if (bookstorep->get_enabled())
			{
			thiscount=bookstorep->get_quotecount();
			if (thiscount>=quoteoffset)
				{
				// ok we found the book we want!
				quotestring=AnsiString((bookstorep->get_quotebyindex(quoteoffset)).c_str());
				break;
				}
			else
				{
				bookstorep=NULL;
				quoteoffset-=thiscount;
				}
			}
		}
	// return found string
	return quotestring;
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
void __fastcall TAdvConfigForm::ButtonBrowseBookDirClick(TObject *Sender)
{
	//
	bookdir = EditBookDir->Text;
	AnsiString realbookdir = bookdir;
	if (!DirectoryExists(realbookdir))
		{
		realbookdir = basedir+bookdir;
		if (!DirectoryExists(bookdir))
			realbookdir = basedir+"\\"+bookdir;
		}

	MyBrowseForFolder->Path=realbookdir;
	if (MyBrowseForFolder->Execute())
		{
		EditBookDir->Text=MyBrowseForFolder->Path;
		}
}
//---------------------------------------------------------------------------

