//---------------------------------------------------------------------------
// JRbcbConfigCollection
//  manage a set of config files in a menu
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#include "JRConfigCollection.h"
#include "JRutils.h"
#include <fstream>
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
std::string ConfigEntry::MakeLabelFromFileName(std::string fullfilename,std::string in_BaseDir)
{
	// make label from full filename
	const char* charp=fullfilename.c_str();
	char c;
	int len=strlen(charp);
	int startpos=0;
	int endpos=len;

	if (in_BaseDir!="")
		{
		int bl=in_BaseDir.length();
		if (bl<=fullfilename.length())
			startpos=in_BaseDir.length()+1;
		for (int i=startpos;i<len;++i)
			{
			c=charp[i];
			if (c=='.')
				{
				endpos=i;
				break;
				}
			}
		}
	else
		{
		// find last \ and first following .
		for (int i=0;i<len;++i)
			{
			c=charp[i];
			if (c=='\\' || c=='/')
				{
				startpos=i+1;
				endpos=len;
				}
			else if (c=='.' && endpos==len)
				endpos=i;
			}
		}

	// get labelstring
	std::string labelstr;
	if (endpos-startpos>0)
		labelstr=std::string(&charp[startpos],endpos-startpos);
	return labelstr;
}
//---------------------------------------------------------------------------








//---------------------------------------------------------------------------
void JrConfigManager::Clear()
{
	// walk through and delete
	std::vector<ConfigEntry*> ::iterator iter;
	for (iter=ConfigEntries.begin();iter!=ConfigEntries.end();++iter)
		{
		// delete the pointed to node
		delete (*iter);
		}
	ConfigEntries.clear();
}
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
void JrConfigManager::RescanForFiles()
{
	// clear first
	Clear();

	// call templated objectmember function callback to find files
	RecursiveFileFinder<JrConfigManager>(this,BaseDirectory,FileMask,recursive,onlyfolders);
}
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
ConfigEntry* JrConfigManager::AddAFile(std::string fullfilename)
{
	// callback for discovering files

	// create it
	ConfigEntry* entryp=new ConfigEntry(fullfilename,BaseDirectory);
	// is it a dupe
	entryp->set_isDupeLabel(IsLabelDuped(entryp));
	// add it
	ConfigEntries.push_back(entryp);
	return entryp;
}


bool JrConfigManager::IsLabelDuped(ConfigEntry* entryp)
{
	// see if any other entries have same label
	std::vector<ConfigEntry*> ::iterator iter;
	for (iter=ConfigEntries.begin();iter!=ConfigEntries.end();++iter)
		{
		// delete the pointed to node
		if (*iter==entryp)
			continue;
		if ((*iter)->get_LabelString() == entryp->get_LabelString())
			return true;
		}
	return false;
}
//---------------------------------------------------------------------------










//---------------------------------------------------------------------------
bool ConfigEntry::LoadBook(const char *filename, bool loadinmemory, bool loadcounts)
{
	// open the file and load all strings
	bookstore.LoadBook(filename,loadinmemory,loadcounts);
        // success
        return true;
}

void ConfigEntry::ClearBook()
{
	// clear any existing quotes
	bookstore.ClearBook();
}
//---------------------------------------------------------------------------





















//---------------------------------------------------------------------------
void BookStore::InitBlankBook(std::string fname)
{
	// clear contents
	ClearBook();

	std::string dummystring=fname;
	JR_SplitStringRight(dummystring,book_title,'\\');
	book_info="No meta info found in this quote file.";
	book_urltitle="No url found for this quote file.";
	book_url=""; //http://www.google.com";
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
bool BookStore::LoadBook(const char *filename, bool loadinmemory, bool loadcounts)
{
	// clear any existing lines
	bool fortunemode=false;
	std::string line,fortuneline;
	bool lastwasnewline=true;

	InitBlankBook(filename);

	// load files
	std::ifstream infile;
	infile.open(filename, std::ios_base::in);
	if (!infile)
		{
		return false;
		}

	// set book file name
	bookfilename=std::string(filename);

	// read book
	quotesinfile=0L;
//	while (getline(infile, line, '\n'))
	while (getline(infile, line))
		{
		// ATTN: do we need to trim \n?
		if (line=="" && fortunemode==false)
			continue;
/*
		if (strncmp(line.c_str(),"%OPTIONS:",9)==0)
			{
			std::string optionstr;
			optionstr=std::string(line.c_str()+9);
			if (optionstr=="no")
				{
				}
			continue;
			}
*/


		if ((line=="%" || line=="%%") && fortunemode==false)
			{
			// it's a fortune file!, we need to restart
			fortunemode=true;
			quotesinfile=0L;
			quotestrings.clear();
			infile.seekg (0,std::ios::beg);
			continue;
			}

		if (quotesinfile==0L)
			{
			// first line *should* be info
			if (strncmp(line.c_str(),"PopUpWisdom",strlen("PopUpWisdom"))==0)
				{
				ParseQuoteFileInfoString(line);
				continue;
				}
			if (!loadcounts && fortunemode==false)
				{
				continue;
//				quotesinfile=-1L;
				break;
				}
			}

		if (fortunemode)
			{
			if (line!="%" && line!="%%")
				{
				if (loadinmemory)
					{
					// part of a quote, so accumulate it
					JR_MyTrimStringEnd(line);
					if (fortuneline!="")
						{
						if (line=="")
							{
							fortuneline+="\\n";
							lastwasnewline=true;
							}
						else if (line=="\\n")
							{
							fortuneline+="\\n";
							lastwasnewline=true;
							line="";	
							}
						else if (line[0]==' ' || line[0]=='\t')
							{
							fortuneline+="\\n";
							lastwasnewline=false;
							}
						else
							{
							if (!lastwasnewline)
								fortuneline+=" ";
							lastwasnewline=false;
							}
						}
					fortuneline+=line;
					}
				}
			else
				{
				// got a new quote
				// TRIM the fortuneline
				JR_MyTrimString(fortuneline);
				if (fortuneline!="")
					{
					if (loadinmemory)
						{
						quotestrings.push_back(fortuneline);
						fortuneline="";
						}
					++quotesinfile;
					}
				}
			}
		else
			{
			if (loadinmemory)
				quotestrings.push_back(line);
			++quotesinfile;
			}
		}

	// close file
	infile.close();

	// end line
	if (fortunemode)
		{
		JR_MyTrimString(fortuneline);
		if (fortuneline!="")
			{
			if (loadinmemory)
				{
				quotestrings.push_back(fortuneline);
				fortuneline="";
				}
			++quotesinfile;
			}
		}

	if (loadinmemory)
		quotesloaded=true;
	else
		quotesloaded=false;


	if (loadinmemory && quotesinfile==0)
		{
		fortuneline="No quotes found in file "+std::string(filename)+".";
		quotestrings.push_back(fortuneline);
		fortuneline="";
		++quotesinfile;
		}

	// ATTN: TEST
	if (quotesinfile==-1L)
		return false;

	// file will be closed on exit
	return true;
}


void BookStore::ClearBook()
{
	// remove all info
	UnloadQuotes();
	quotesinfile=0L;
}


void BookStore::UnloadQuotes()
{
	// remove quotes from memory
	quotestrings.clear();
	quotesloaded=false;
}


void BookStore::NeedsQuotes(bool needinmemory)
{
	if (needinmemory && quotesloaded==false)
		LoadBook(bookfilename.c_str(),true,true);
	else if (!needinmemory && quotesloaded==true)
		{
		UnloadQuotes();
		}
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
std::string BookStore::get_quotebyindex(long index)
{
	NeedsQuotes(true);
	if (index>=0 && index<quotesinfile && quotesloaded)
		return quotestrings[index];
	return "quotes could not be loaded.";
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
bool BookStore::ParseQuoteFileInfoString(std::string infostring)
{
	//
	// should be PopUpWisdom|book_title|book_info|book_urltitle|book_url
	std::string dummmystring;
	JR_SplitStringLeft(infostring,dummmystring,'|');
	JR_SplitStringLeft(infostring,book_title,'|');
	JR_SplitStringLeft(infostring,book_info,'|');
	JR_SplitStringLeft(infostring,book_urltitle,'|');
	JR_SplitStringLeft(infostring,book_url,'|');

	// success
	return true;
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
bool BookStore::ExportBook(std::string fname, bool userealnewlines, std::string separator)
{
	bool wasquotesloaded = get_quotesloaded();
	std::string quotelinestr;

	// load quotes
	NeedsQuotes(true);

	if (quotesinfile<0)
		return false;

        // open file
	std::ofstream outfile(fname.c_str(), std::ios_base::out);
	if (!outfile)
		return false;

	// replaced
	std::string escapedrn="\\r\\n";
	std::string escapedr="\\r";
	std::string escapedn="\\n";
	std::string slashrn="\r\n";
	std::string slashr="\r";
	std::string slashn="\n";


	// export them
	for (long index=0;index<quotesinfile;++index)
		{
	        quotelinestr=get_quotebyindex(index);

		// replace newlines
		if (!userealnewlines)
			{
			JR_FindReplaceField(quotelinestr,slashrn,escapedn,true);
			JR_FindReplaceField(quotelinestr,slashr,escapedn,true);
			JR_FindReplaceField(quotelinestr,slashn,escapedn,true);
			}
		else
			{
			JR_FindReplaceField(quotelinestr,escapedrn,slashn,true);
			JR_FindReplaceField(quotelinestr,escapedr,slashn,true);
			JR_FindReplaceField(quotelinestr,escapedn,slashn,true);
			}

		outfile << quotelinestr << separator;
		}

	// file will close itself

	// unload if wasnt loaded
	if (!wasquotesloaded)
		{
		UnloadQuotes();
		}

	// success
	return true;
}
//---------------------------------------------------------------------------

