//---------------------------------------------------------------------------
// JRbcbConfigCollection
//  manage a set of config files in a menu
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#ifndef JRConfigCollectionH
#define JRConfigCollectionH
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#include <string>
#include <vector>
//
#include "JRrfilefinder.h"
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
class BookStore
{
protected:
	std::vector<std::string> quotestrings;
	std::string bookfilename;
	std::string book_title,book_url,book_urltitle,book_info;
	bool quotesloaded;
	long quotesinfile;
	bool enabled;
public:
	BookStore() {quotesloaded=false;quotesinfile=-1L;enabled=true;};
	~BookStore() {ClearBook();};
public:
	void InitBlankBook(std::string fname);
	void ClearBook();
	void UnloadQuotes();
	bool LoadBook(const char *filename, bool loadinmemory, bool loadcounts);
public:
	// accessors
	long get_quotecount() {if (quotesinfile==-1L) LoadBook(bookfilename.c_str(),false,true); return quotesinfile;};
	std::string get_title() {return book_title;};
	std::string get_url() {return book_url;};
	std::string get_urltitle() {return book_urltitle;};
	std::string get_info() {return book_info;};
	std::string get_filename() {return bookfilename;};
	std::string set_title(std::string val) {book_title=val;};
	bool get_enabled() {return enabled;};
	void set_enabled(bool val) {enabled=val;};
	bool get_quotesloaded() {return quotesloaded;};
public:
	std::string get_quotebyindex(long index);
public:
	void NeedsQuotes(bool needinmemory);
	bool ParseQuoteFileInfoString(std::string infostring);
public:
	bool ExportBook(std::string fname, bool userealnewlines, std::string separator);
};
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
class ConfigEntry
{
friend class JrConfigManager;
protected:
	std::string LabelString;
	std::string FileName;
	bool isDupeLabel;
protected:
	BookStore bookstore;
protected:
	ConfigEntry(std::string in_FileName) {FileName=in_FileName;LabelString=MakeLabelFromFileName(FileName);isDupeLabel=false;LoadBook(FileName.c_str(),false,false);if (bookstore.get_title()=="") bookstore.set_title(LabelString);};
	ConfigEntry(std::string in_FileName,std::string in_BaseDir) {FileName=in_FileName;LabelString=MakeLabelFromFileName(FileName,in_BaseDir);isDupeLabel=false;LoadBook(FileName.c_str(),false,false);if (bookstore.get_title()=="") bookstore.set_title(LabelString);};
	~ConfigEntry() {ClearBook();};
public:
	std::string get_LabelString() {return LabelString;};
	std::string get_FileName() {return FileName;};
	void set_isDupeLabel(bool val) {isDupeLabel=val;};
	bool get_isDupeLabel() {return isDupeLabel;};
public:
	static std::string MakeLabelFromFileName(std::string fullfilename,std::string in_BaseDir="");
public:
	void ClearBook();
	bool LoadBook(const char *filename, bool loadinmemory, bool loadcounts);
	class BookStore* get_bookstorep() {return &bookstore;};
};
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
class JrConfigManager
{
protected:
	std::vector<ConfigEntry*> ConfigEntries;
	std::string FileMask;
	std::string BaseDirectory;
	bool onlyfolders;
	bool recursive;
public:
	JrConfigManager() {;};
	~JrConfigManager() {;};
public:
	void Clear();
	void RescanForFiles();
public:
	void set_FileMask(std::string val,bool inonlyfolders,bool inrecursive) {FileMask=val;onlyfolders=inonlyfolders;recursive=inrecursive;};
	void set_BaseDirectory(std::string val) {BaseDirectory=val;};	
public:
	// callback for discovering files
	ConfigEntry* AddAFile(std::string fullfilename);
public:
	ConfigEntry* get_Entry(int index) {return ConfigEntries[index];};
	int get_EntryCount() {return ConfigEntries.size();};
public:
	bool IsLabelDuped(ConfigEntry* entryp);
};
//---------------------------------------------------------------------------














//---------------------------------------------------------------------------
#endif
//---------------------------------------------------------------------------
