using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.Xml.XPath;
using FARRPlugin;

namespace TempPlugin
{
    public class FARRAltTab : FARRPluginBase
    {
        List<KeyValuePair<IntPtr, string>> m_DesktopWindowsCaptions = null;

        public FARRAltTab()
        {
        }

        protected override void SetGeneralInfoParameters(ref string DisplayName, ref string VersionString, ref string ReleaseDateString,
                                                         ref string Author, ref string HomepageURL, ref string UpdateURL,
                                                         ref string ShortDescription, ref string LongDescription, ref string AdvConfigString,
                                                         ref string ReadMeString, ref string IconFilename)
        {
            DisplayName = "FARR AltTab";
            VersionString = GetValueFromDCUpdate(EDCUpdateValues.Version);
            ReleaseDateString = "Nov 23rd, 2007";
            Author = "Vitaly Belman";
            HomepageURL = GetValueFromDCUpdate(EDCUpdateValues.WebPage);
            UpdateURL = GetValueFromDCUpdate(EDCUpdateValues.UpdateFile);
            ShortDescription = "An alternative to Windows ALT+TAB";
            LongDescription = "An alternative to Windows ALT+TAB that allows you to search the window by name";
            IconFilename = "FARRAltTab.ico";
        }

        protected override void SetSearchParameters(ref string DefaultAlias, ref string DefaultRegex, ref string DefaultRegexFilter, ref string DefaultKeyword, ref string DefaultScore)
        {
            DefaultAlias = "tab";
            DefaultRegex = "^tab (.+)";
            DefaultRegexFilter = "$$1";
            DefaultScore = "300";
        }
        protected override void SetFeatureParamaters(Dictionary<EWantFeatures, bool> FeatureDictionary)
        {
            FeatureDictionary[EWantFeatures.SearchInputRegex] = true;
            FeatureDictionary[EWantFeatures.SearchInputExplicit] = true;
            FeatureDictionary[EWantFeatures.TryHandleTrigger] = true;
        }

        public override void GetItemResultByIndex(int ResultIndex, ref string Path, ref string Caption, ref string Group, ref string IconFile, ref FARRPluginBase.EResultPostProcessing ResultPostProcessing, ref int ScoreChange, ref FARRPluginBase.EEntryType EntryType)
        {
            Path = Convert.ToString(m_DesktopWindowsCaptions[ResultIndex].Key);
            Caption = m_DesktopWindowsCaptions[ResultIndex].Value;
            IconFile = EnumWindows.GetProcessPath(m_DesktopWindowsCaptions[ResultIndex].Key);

            // We are sure we want to show the window
            ResultPostProcessing = EResultPostProcessing.AddScore;
            EntryType = EEntryType.File;
        }

        public override bool SearchBegins(string SearchStringRaw, string SearchStringKeyKeywords, bool ExplicitInvocation)
        {
            return GetDesktopWindows("");
        }

        private bool GetDesktopWindows(string ContainedCaption)
        {
            ExecuteFARRCommand("setshowallmode", "");

            EnumWindows.GetDesktopWindowsCaptions(ContainedCaption, out m_DesktopWindowsCaptions);

            m_SearchState = ESearchState.Stopped;
            if (m_DesktopWindowsCaptions.Count > 0)
            {
                m_ResultsAvailableState = EResultsAvailableState.ItemResults;
                m_ResultsAmount = m_DesktopWindowsCaptions.Count;
            }
            else
            {
                ClearResults();
            }

            SearchStateChanged();

            // Continue searching
            return false;
        }

        public override bool RegexSearchMatch(string SearchStringRaw, string SearchStringKeyKeywords, int RegexGroupsCount, string[] RegexGroups)
        {
            if (RegexGroupsCount == 2)
            {
                GetDesktopWindows(RegexGroups[1]);
                return true;
            }
            else
                return false;
        }

        public override void SearchEnds()
        {
            ClearResults();
        }

        public override bool LockResults()
        {
            

            return true;
        }

        public override bool UnlockResults()
        {
            ClearResults();
            SearchStateChanged();

            return true;
        }

        public override bool ProcessTrigger(string Path, string Caption, string Group, int PluginId, int ThisPlugId, int Score, EEntryType EntryType, out bool CloseAfterOperation)
        {
            if (PluginId == ThisPlugId)
            {
                CloseAfterOperation = true;

                EnumWindows.SwitchToWindow((IntPtr)Convert.ToInt32(Path));
                return true;
            }
            else
            {
                CloseAfterOperation = true;
                return false;
            }
        }
        
          
    }
}
