﻿using System;
using System.Collections.Generic;
using System.Text;
using FARRPlugin;
using iTunesLib;
using System.Text.RegularExpressions;

namespace FARRTunes.Lists
{
    class ActionsList : List<Actions.ActionItem>
    {
        protected ActionsList(string ActivatorRegex)
        {
            m_ActivatorRegex = ActivatorRegex + " {0,1}";
            m_SourceList = new List<Actions.ActionItem>();
        }

        // Notice that in dynamic lists it won't be just the list count
        public virtual int GetActionsCount(string KeyKeywords)
        {
            KeyKeywords = GetSearch(KeyKeywords);

            foreach (Actions.ActionItem Item in SourceList)
            {
                if (Item.Caption.ToLower().Contains(KeyKeywords))
                    this.Add(Item);
            }

            return this.Count;
        }

        protected string GetSearch(string KeyKeywords)
        {
            Regex RegMatch = new Regex(m_ActivatorRegex);
            return RegMatch.Replace(KeyKeywords, "");
        }

        public Actions.ActionItem FindByCaption(string Caption)
        {
            foreach (Actions.ActionItem Item in this)
            {
                if (Item.Caption == Caption)
                    return Item;
            }

            return null;
        }

        public string ActivatorRegex { get { return m_ActivatorRegex; } }
        string m_ActivatorRegex;

        internal virtual void ClearTemporaryItems()
        {
            this.Clear();
        }

        virtual public FARRPluginBase.EResultsAvailableState ResultsType
        {
            get { return FARRPluginBase.EResultsAvailableState.ItemResults; }
        }

        // Keeps all the original action items of the list
        List<Actions.ActionItem> m_SourceList;
        public List<Actions.ActionItem> SourceList { get { return m_SourceList; } }
    }

    class MainActionsList : ActionsList
    {
        public MainActionsList()
            : base(Constants.Tunes)
        {
            this.SourceList.Add(new Actions.TunesSong());
            this.SourceList.Add(new Actions.TunesPlayPause());
            this.SourceList.Add(new Actions.TunesNext());
            this.SourceList.Add(new Actions.TunesPrevious());
            this.SourceList.Add(new Actions.SearchChanger("Playlists", Constants.Playlists, "playlist.ico", "pls"));
            this.SourceList.Add(new Actions.SearchChanger("Search", Constants.Search, "search.ico", "src"));
        }
    }

    class PlaylistsList : ActionsList
    {
        public PlaylistsList()
            : base(Constants.Playlists)
        {
            iTunesAppClass TunesCom = TunesHelper.GetTunesCom();

            IITPlaylistCollection Playlists = TunesCom.Sources.get_ItemByName("Library").Playlists;
            foreach (IITPlaylist Playlist in Playlists)
            {
                this.SourceList.Add(new Actions.TunesPlaylist(Playlist));
            }
        }
    }

    class SongActionsList : ActionsList
    {
        public SongActionsList()
            : base(Constants.CurrentTune)
        {
            iTunesAppClass TunesCom = TunesHelper.GetTunesCom();

            this.SourceList.Add(new Actions.TunesSong());
            this.SourceList.Add(new Actions.SearchChanger("Rate song", Constants.CurrentTuneRate, "favorits.ico", "rt"));
            this.SourceList.Add(new Actions.SearchChanger("Lyrics", Constants.CurrentTuneLyrics, "lyrics.ico", "lyr"));
        }


    }

    class SongRateList : ActionsList
    {
        public SongRateList()
            : base(Constants.CurrentTuneRate)
        {
            this.SourceList.Add(new Actions.RateSongItem(1));
            this.SourceList.Add(new Actions.RateSongItem(2));
            this.SourceList.Add(new Actions.RateSongItem(3));
            this.SourceList.Add(new Actions.RateSongItem(4));
            this.SourceList.Add(new Actions.RateSongItem(5));
            this.SourceList.Add(new Actions.RateSongItem(0));
        }
    }

    class SongLyricsList : ActionsList
    {
        public SongLyricsList()
            : base(Constants.CurrentTuneLyrics)
        {
            this.SourceList.Add(new Actions.LyricsItem());
        }
    }

    class SongSearchList : ActionsList
    {
        public SongSearchList()
            : base(Constants.Search)
        {

        }

        public override int GetActionsCount(string KeyKeywords)
        {
            KeyKeywords = GetSearch(KeyKeywords);

            if (KeyKeywords != "")
            {
                iTunesAppClass TunesCom = TunesHelper.GetTunesCom();

                IITTrackCollection Tracks = TunesCom.LibraryPlaylist.Search(KeyKeywords, ITPlaylistSearchField.ITPlaylistSearchFieldAll);
                if (Tracks != null)
                {
                    foreach (IITTrack Track in Tracks)
                    {
                        this.Add(new Actions.TunesSong(Track));

                        if (this.Count > 10)
                            break;
                    }
                }

                return this.Count;
            }
            else
            {
                return 0;
            }
        }
    }


}
