unit MyPlugin;

{
  This unit contains all the things Farr expects

  It is recommended to put custom helper functions/procedures in the MyPluginCustomFuncs unit to keeps things clean
}

interface

uses
  Sysutils,
  Windows,
  Dialogs,
  ShellApi,
  FarrPluginGlobals,
  FarrPluginHelpers,
  MyPluginCustomFuncs;

  function DoInit():BOOL;
  function DoShutDown():BOOL;
  function SupportCheck(const testname:PChar; version:integer):BOOL;
  function DoAdvConfig():BOOL;
  function DoShowReadMe():BOOL;
  function GetStrVal(const varname:PChar; destbuf:PChar; maxlen:integer):BOOL;
  function SetStrVal(const varname:PChar; val:Pointer):BOOL;
  function SetState(newstateval:E_PluginStateT):BOOL;
  function AskWantFeature(featureid:E_WantFeaturesT):BOOL;
  function RequestLockResults(dolock:BOOL):BOOL;
  function RequestItemResultByIndex(resultindex:integer; destbuf_path:PChar; destbuf_caption:PChar; destbuf_groupname:PChar; destbuf_iconfilename:PChar; tagvoidpp:PPointer; maxlen:integer; resultpostprocmodep:PE_ResultPostProcessingT; scorep:PInteger; entrytypep:PE_EntryTypeT):BOOL;
  function InformRegexSearchMatch(const searchstring_raw:PChar; const searchstring_lc_nokeywords:PChar; regexgroups:integer; regexcharps:P_PCharArray):BOOL;
  function AllowProcessTrigger(const destbuf_path:PChar; const destbuf_caption:PChar; const destbuf_groupname:PChar; pluginid:integer; thispluginid:integer; score:integer; entrytype:E_EntryTypeT; tagvoidp:Pointer; closeafterp:PBOOL):BOOL;
  function InformSearchBegins(const searchstring_raw:PChar; const searchstring_lc_nokeywords:PChar; explicitinvocation:BOOL):BOOL;

  procedure InformSearchEnds();

  // START - To be tested
  function AskSearchState():E_SearchStateT;
  function AskIsAnyResultAvailable():E_ResultAvailableStateT;
  function AskHowManyItemResultsAreReady():integer;
  function RequestTextResultCharp(charp:P_PCharArray):BOOL;
  function DoAdjustResultScore(const itempath:PChar; scorep:PInteger):BOOL;
  function DoAddInfo(const itempath:PChar):BOOL;

  procedure InformWindowIsHiding(hwndp:HWND);
  procedure InformWindowIsUnHiding(hwndp:HWND);
  // END - To be tested

const
  // Customize these constants, they will show up in the FARR plugin configuration
  DisplayName:PChar = 'Locate32 Plugin';
  Version:PChar = '0.1';
  ReleaseDate:PChar = '09/23/2007';
  Author:PChar = 'O. Tijhuis';
  UpdateURL:PChar = 'http://www.somesite.com/pluginupdate';
  HomepageURL:PChar = 'http://www.somesite.com/plugin';
  ShortDescription:PChar = 'Locate32 Integration Plugin';
  LongDescription:PChar = 'Locate32 Plugin (source also serves as framework for Delphi developers)';
  AdvConfig:PChar = 'Advanced Configuration'; // Set this to a non-empty string if advanced configuration is supported
  ReadMe:PChar = 'View Help File'; // Set this to a non-empty string if ReadMe is supported
  ReadMeFile:string = 'README.txt'; // Your plugin readme filename
  IconFilename:PChar = 'Locate32.ico'; // Set this to the icon's filename if you supply an icon file
  StatusBar:PChar = 'FARRv2 Delphi Framework';
  DefaultAlias:PChar = ''; // NOTE: can be empty but InformSearchBegins won't get called then
  DefaultRegex:PChar ='^loc (.*)$';
  DefaultRegexFilter:PChar = ''; // $$1 for instance
  DefaultKeywordStr:PChar = ''; // default keywords
  DefaultScoreStr:PChar = '50';

implementation

  function DoInit():BOOL;
  begin
    // use this for loading settings for instance
    MyPluginCustomFuncs.LoadSettings();
    Result := True;
  end;

  function DoShutDown():BOOL;
  begin
    // use this for saving settings for instance
    inifile.Free;
    Result := True;
  end;

  function SupportCheck(const testname:PChar; version:integer):BOOL;
  begin
    // We support FARR interface
    if(testname = DEF_FARRAPI_IDENTIFIER) then
    begin
      Result := True;
    end
    else
      Result := False;
  end;

  function DoAdvConfig():BOOL;
  begin
    MyPluginCustomFuncs.AdvConfigForm();
    Result := True;
  end;

  function DoShowReadme():BOOL;
  begin
    // Open readme file
    ShellExecute(0, 'open', PChar(Concat(dlldir,'\',ReadMeFile)), nil, nil, SW_SHOWNORMAL);
    Result := True;
  end;

  function SetState(newstateval:E_PluginStateT):BOOL;
  begin
    // Usually nothing to do here
    Result := True;
  end;

  {
    Host informs us that a search is begining and what the searchstring is
    The Plugin DLL could do all "searching" here (if there is indeed any to do)
    And compute all results within this function, OR simply begin a threaded asynchronous search now.

    Normally the string you care about is searchstring_lc_nokeywords, thats the search string with all special +modifier keywords removed, and
    the alias keyword for this plugin removed, so all thats left is the string that should effect the results of the search.

    returns TRUE only if the plugin decides now that no more searching should be done by any other plugin or builtin

    NOTE: if asynchronous searching is to be done, make sure to only set current_searchstate=E_SearchState_Stopped when done!

    EXTRA NOTES: 1) Called when alias matches
  }
  function InformSearchBegins(const searchstring_raw:PChar; const searchstring_lc_nokeywords:PChar; explicitinvocation:BOOL):BOOL;
  begin
    if(not explicitinvocation) then
      Result := False
    else
      Result := MyPluginCustomFuncs.DoSearch(searchstring_lc_nokeywords);
  end;

  {
    Host informs us that regular expression match **HAS** occured, and to begin doing that we need to do on such a match
    The Plugin DLL could do all "searching" here (if there is indeed any to do)
    And compute all results within this function, OR simply begin a threaded asynchronous search now.

    To access the group capture strings, use regexcharps[1] to match the first group, etc.

    returns TRUE only if the plugin decides now that no more searching should be done by any other plugin or builtin

    NOTE: if asynchronous searching is to be done, make sure to only set current_searchstate=E_SearchState_Stopped when done!
  }
  function InformRegexSearchMatch(const searchstring_raw:PChar; const searchstring_lc_nokeywords:PChar; regexgroups:integer; regexcharps:P_PCharArray):BOOL;
  begin
    if(regexgroups > 0) then
    begin
      Result := MyPluginCustomFuncs.DoSearch(regexcharps^[1]);
    end
    else
      Result := False;
  end;

  // Host will be asking which features to use
  function AskWantFeature(featureid:E_WantFeaturesT):BOOL;
  begin

    if(featureid = E_WantFeatures_searchinput_regex) then
    begin
      // do we want to search on regular expression matches
      Result := True;
    end
    else if(featureid = E_SupportFeatures_searchinput_explicit) then
    begin
      // do we want to search on alias match
      Result := True;
    end
    else if(featureid = E_SupportFeatures_searchinput_all) then
    begin
      // do we want to search on all search expressions
      Result := False;
    end
    else if(featureid = E_SupportFeatures_tryhandle_trigger) then
    begin
      // do we want to try to handle triggers when a result is launched
      Result := True;
    end
    else if(featureid = E_SupportFeatures_addinfo_files) then
    begin
      // ???
      Result := False;
    end
    else if(featureid = E_SupportFeatures_scoreall_files) then
    begin
      // do we want to try to score EVERY file considered (SLOWS DOWN SEARCH!)
      Result := False;
    end
    else if(featureid = E_SupportFeatures_scoregood_files) then
    begin
      // do we want to try to score files after other scoring has determined they are a viable result
      Result := False;
    end
    else if(featureid = E_SupportFeatures_dominate_results) then
    begin
      // do we want our results to dominate and hide any others
      Result := False;
    end
    else
    begin
      // fell through - so not supported
      Result := False;
    end;
  end;

  function AskSearchState():E_SearchStateT;
  begin
//    ShowMessage('AskSearchState');
    Result := searchstate;
  end;

  function AskIsAnyResultAvailable():E_ResultAvailableStateT;
  begin
//    ShowMessage('AskIsAnyResultAvailable');
    Result := resultsavailabletype;
  end;

  function AskHowManyItemResultsAreReady():integer;
  begin
//    ShowMessage('AskHowManyItemResultsAreReady');
    Result := numresultsavailable;
  end;

  {
    Host calls this before iterating through the result requests
    It will be TRUE on lock before retrieving
    and then FALSE when all done retrieving
    In this way the dll should refrain from altering the results until retrieval is done
    One way to do this for asynchronous searching is to MOVE results to a return store
    on TRUE lock, and then free on FALSE.

    Returns TRUE on success
  }
  function RequestLockResults(dolock:BOOL):BOOL;
  begin
    // set lock
    lockstate := dolock;

    if(dolock = False) then
    begin
      // on unlocking they have retrieved all the results, so CLEAR the results or they will be found again
      numresultsavailable := 0;
      resultsavailabletype := E_ResultAvailableState_None;
      // notify host that our state has changed
      ExecuteCallbackSearchStateChanged();
    end;

    // success
    Result := True;
  end;

  {
    Host is requesting one of our results
    Fill the destination buffers without exceding maxlen

    resultpostprocmodep is from enum E_ResultPostProcessingT : E_ResultPostProcessing_ImmediateDisplay = 0, E_ResultPostProcessing_AddScore = 1, E_ResultPostProcessing_MatchAgainstSearch = 2

    Returns TRUE on success
  }
  function RequestItemResultByIndex(resultindex:integer; destbuf_path:PChar; destbuf_caption:PChar; destbuf_groupname:PChar; destbuf_iconfilename:PChar; tagvoidpp:PPointer; maxlen:integer; resultpostprocmodep:PE_ResultPostProcessingT; scorep:PInteger; entrytypep:PE_EntryTypeT):BOOL;
  begin
  	// clear values sample:
	  StrCopy(destbuf_groupname,'');
  	tagvoidpp^ := NIL;
    {
      how result is handled
      return E_ResultPostProcessing_ImmediateDisplay if we have done the filtering and the result should be displayed
      return E_ResultPostProcessing_AddScore if the host should evaluate the result and add a score to it but still be forced to display it
      return E_ResultPostProcessing_MatchAgainstSearch if the host should evaluate the result and decide whether to show it based on match of search words
    }
	  resultpostprocmodep^ := E_ResultPostProcessing_AddScore;

    {
      do we want to modify the score assigned by host?  usually not.
      scorep^ := scorep^ + 100;

      type (file is same as url)
    }
  	entrytypep^ := E_EntryType_FILE;

   	// ok fill the result data
    // In this cases it's a filename that was found by locate32
    tagvoidpp^ := PChar(ExtractFileName(results[resultindex]));

    // Return the path to the file
  	StrLCopy(destbuf_path,PChar(results[resultindex]),maxlen);
    // Return the caption
    // In this case the filename without extension
	  StrLCopy(destbuf_caption,PChar(ChangeFileExt(ExtractFileName(results[resultindex]),'')),maxlen);

   	// IF you want to return an icon for this item do so here by returning icon filename
    // Here the filename is returned and the icon belonging to the filetype is used by FARR
    StrLCopy(destbuf_iconfilename,PChar(results[resultindex]),maxlen);

  	// ok filled one
    Result := True;
  end;

  {
    Host is asking us to return a char* to a LONG result string

    NOTE: It is up to us to free this char* when the lock is set to false above OR
    on the next call, or on search end, but we MUST keep it valid until then.
  }
  function RequestTextResultCharp(charp:P_PCharArray):BOOL;
  begin
  	// signify we have none
    charp := NIl;

    // None returned
    Result := False;
  end;

  {
    Host is calling us with a result item about to be triggered
    We can choose here to takeover the launching of it, OR simply do something and let normal launching continue

    Return TRUE to takeover launching and prevent all other further launching
    or FALSE to continue launching after we return
  }
  function AllowProcessTrigger(const destbuf_path:PChar; const destbuf_caption:PChar; const destbuf_groupname:PChar; pluginid:integer; thispluginid:integer; score:integer; entrytype:E_EntryTypeT; tagvoidp:Pointer; closeafterp:PBOOL):BOOL;
  begin
    // Check if this plugin created the result
  	if (thispluginid <> pluginid) then
    begin
      // If not, don't handle it
      Result := False
    end
    else
    begin
      // In this case we don't want to handle it either
      Result := False;
    end;
  end;

  {
   Host is asking us if we want to modify the score of an item
   Original score in in *score, we can now modify it if we want

   Return TRUE if we modified it, or FALSE if not

   NOTE: set score to -9999 to eliminate it from results
  }
  function DoAdjustResultScore(const itempath:PChar; scorep:PInteger):BOOL;
  begin
    // ShowMessage('DoAdjustResultScore');
    // We didn't modify it
    Result := False;
  end;

  function DoAddInfo(const itempath:PChar):BOOL;
  begin
    // ShowMessage('DoAddInfo');
    Result := True;
  end;

  {
    Host informs us that a search has completed or been interrupted, and we should stop any processing we might be doing assynchronously

    NOTE: make sure to set current_searchstate=E_SearchState_Stopped;
  }
  procedure InformSearchEnds();
  begin
    // stop search and set this
    searchstate := E_SearchState_Stopped;

    // clear results
    numresultsavailable := 0;
    resultsavailabletype := E_ResultAvailableState_None;

    ExecuteCallbackSearchStateChanged();
  end;

  // Host informs us that our window is about to disappear
  procedure InformWindowIsHiding(hwndp:HWND);
  begin
  end;

  // Host informs us that our window is about to appear
  procedure InformWindowIsUnHiding(hwndp:HWND);
  begin
  end;

  function SetStrVal(const varname:PChar; val:Pointer):BOOL;
  begin
    if(varname = DEF_FieldName_NotifySearchCallbackFp) then
    begin
      Host_NotifySearchStateChanged := val;
      Result := False;
    end
    else if(varname = DEF_FieldName_EnterInRichEdit) then
    begin
      // Enter was pressed in rich edit mode
      // ShowMessage('Enter was pressed in rich edit mode');
      Result := False;
    end
    else
      Result := False;
  end;

  function GetStrVal(const varname:PChar; destbuf:PChar; maxlen:integer):BOOL;
  begin
    // No need to edit the next block of code, these are standard (can be customized in the const area at the top of this unit)
    // **** Start of block ****
    if(varname = DEF_FieldName_DisplayName) then
    begin
      StrCopy(destbuf, DisplayName);
      Result := True;
    end
    else if(varname = DEF_FieldName_VersionString) then
    begin
      StrCopy(destbuf, Version);
      Result := True;
    end
    else if(varname = DEF_FieldName_ReleaseDateString) then
    begin
      StrCopy(destbuf, ReleaseDate);
      Result := True;
    end
    else if(varname = DEF_FieldName_Author) then
    begin
      StrCopy(destbuf, Author);
      Result := True;
    end
    else if(varname = DEF_FieldName_UpdateURL) then
    begin
      StrCopy(destbuf, UpdateURL);
      Result := True;
    end
    else if(varname = DEF_FieldName_HomepageURL) then
    begin
      StrCopy(destbuf, HomepageURL);
      Result := True;
    end
    else if(varname = DEF_FieldName_ShortDescription) then
    begin
      StrCopy(destbuf, ShortDescription);
      Result := True;
    end
    else if(varname = DEF_FieldName_LongDescription) then
    begin
      StrCopy(destbuf, LongDescription);
      Result := True;
    end
    else if(varname = DEF_FieldName_AdvConfigString) then
    begin
      StrCopy(destbuf, AdvConfig);
      Result := True;
    end
    else if(varname = DEF_FieldName_ReadMeString) then
    begin
      StrCopy(destbuf, ReadMe);
      Result := True;
    end
    else if(varname = DEF_FieldName_IconFilename) then
    begin
      StrCopy(destbuf, IconFilename);
      Result := True;
    end
    else if(varname = DEF_FieldName_AliasStr) then
    begin
      StrCopy(destbuf, DefaultAlias);
      Result := True;
    end
    else if(varname = DEF_FieldName_RegexStr) then
    begin
      StrCopy(destbuf, DefaultRegex);
      Result := True;
    end
    else if(varname = DEF_FieldName_RegexFilterStr) then
    begin
      StrCopy(destbuf, DefaultRegexFilter);
      Result := True;
    end
    else if(varname = DEF_FieldName_KeywordStr) then
    begin
      StrCopy(destbuf, DefaultKeywordStr);
      Result := True;
    end
    else if(varname = DEF_FieldName_ScoreStr) then
    begin
      StrCopy(destbuf, DefaultScoreStr);
      Result := True;
    end
    else
    begin
      Result := False;
    end;

    // **** End of block ****

    // Enter custom code at the end if needed
  end;

end.
