unit MyPluginCustomFuncs;

interface
uses
  FarrPluginGlobals,
  FarrPluginHelpers,
  Classes,
  SysUtils,
  Controls,
  IniFiles,
  Dialogs, // Debug purposes
  AdvConfigForm,
  Windows,
  Forms,
  RegExpr;

  function DoSearch(searchstring:PChar):BOOL;
  function GetDosOutput(const CommandLine:string): string;
  procedure AdvConfigForm();
  procedure LoadSettings();
  procedure SaveSettings();
  function EditSearchString(searchstring:PChar):string;

implementation

  function EditSearchString(searchstring:PChar):string;
  var
    objRegEx:TRegExpr;
    outstring:string;
    optionterminatorpos:integer;
  begin
    optionterminatorpos := 0;
    outstring := searchstring;

    objRegEx := TRegExpr.Create;
    try
      // Casesensitive!
      objRegEx.ModifierI := False;

      // Following is a work in progress, not to be used just yet
      {
      objRegEx.Expression := ' -- ';
      if(objRegEx.Exec(outstring)) then
      begin
        optionterminatorpos := objRegEx.MatchPos[0];
      end;

      objRegEx.Expression := ' -s +| -s$';
      if(objRegEx.Exec(outstring)) then
      begin
        ShowMessage('match');
      end;

      // Remove -s option from the searchstring, it can't be used
      objRegEx.Expression := ' -s +| -s$';
      if(objRegEx.Exec(outstring)) then
      begin
        outstring := objRegEx.Replace(outstring,' ',True);
      end;

      // Remove -i option from the searchstring, it can't be used
      objRegEx.Expression := ' -i +| -i$';
      if(objRegEx.Exec(searchstring)) then
      begin
        outstring := objRegEx.Replace(outstring,' ',True);
      end;

      // Remove -p option from the searchstring if no extra data is given yet
      objRegEx.Expression := ' -p *$| -p *[a-zA-Z]+$';
      if(objRegEx.Exec(outstring)) then
      begin
        outstring := objRegEx.Replace(outstring,'',True);
      end;
      }

      objRegEx.Expression := ' -ln[:]?[0-9]+ | -ln[:]?[0-9]+$';
      if(not objRegEx.Exec(outstring)) then
      begin
        outstring := outstring + ' -ln:' + IntToStr(locatemaxfoundfiles);
      end;

    finally
      objRegEx.Free;
    end;
    Result := outstring;
  end;

  function DoSearch(searchstring:PChar):BOOL;
  var
    splitresults:TStrings;
    x:integer;
    output:string;
  begin
    // FARR search
    if(not isready) then
    begin
      // statusbar
      Host_SetStrVal(hostptr,'statusbar','Plugin is busy searching...');
      // search can continue by others? (False : Yes  ,  True : No)
      Result := True;
    end
    // If searchstring is empty then return no results
    else if((searchstring = ' ') or (searchstring = '')) then
    begin

      // ok now results are available right away
      // IMPORTANT: here is where you modify the code to specify how many results are available
      numresultsavailable := 0;
      resultsavailabletype := E_ResultAvailableState_None;

          // done
      searchstate := E_SearchState_Stopped;

      // notify host that our state has changed
      ExecuteCallbackSearchStateChanged();

      Result := True;
    end
    else
    begin
      // start and end of search state
      searchstate := E_SearchState_Searching;

      while(not FileExists(locateexelocation)) do
      begin
        ShowMessage('Locate.exe does not exist in location: ' + locateexelocation);
        AdvConfigForm();
      end;

      searchstring := PChar(EditSearchString(searchstring));
      output := GetDosOutput(locateexelocation + ' ' + searchstring);
      splitresults := TStringList.Create;
      ExtractStrings([Char(#13)],[' '],PChar(output),splitresults);

      {
          TODO: there should be no need to copy the results to another variable again
      }
      SetLength(results,splitresults.Count);
      for x := 0 to splitresults.Count - 1 do
      begin
        results[x] := splitresults[x];
      end;

      // ok now results are available right away
      // IMPORTANT: here is where you modify the code to specify how many results are available
      numresultsavailable := length(results);
      resultsavailabletype := E_ResultAvailableState_ItemResults;

      // done
      searchstate := E_SearchState_Stopped;

      // notify host that our state has changed
      ExecuteCallbackSearchStateChanged();

      // search can continue by others? (False : Yes  ,  True : No)
      Result := True;
    end;
  end;

  // With thanks to John W. Long (article http://www.delphi3000.com/articles/article_2112.asp)
  function GetDosOutput(const CommandLine:string): string;
  var
    SA: TSecurityAttributes;
    SI: TStartupInfo;
    PI: TProcessInformation;
    StdOutPipeRead, StdOutPipeWrite: THandle;
    WasOK: Boolean;
    Buffer: array[0..255] of Char;
    BytesRead: Cardinal;
    Line: String;
  begin
    Application.ProcessMessages;
    with SA do
    begin
      nLength := SizeOf(SA);
      bInheritHandle := True;
      lpSecurityDescriptor := nil;
    end;
    // create pipe for standard output redirection
    CreatePipe(StdOutPipeRead,  // read handle
               StdOutPipeWrite, // write handle
               @SA,             // security attributes
               0                // number of bytes reserved for pipe - 0 default
              );
    try
      // Make child process use StdOutPipeWrite as standard out,
      // and make sure it does not show on screen.
      with SI do
      begin
        FillChar(SI, SizeOf(SI), 0);
        cb := SizeOf(SI);
        dwFlags := STARTF_USESHOWWINDOW or STARTF_USESTDHANDLES;
        wShowWindow := SW_HIDE;
        hStdInput := GetStdHandle(STD_INPUT_HANDLE); // don't redirect stdinput
        hStdOutput := StdOutPipeWrite;
        hStdError := StdOutPipeWrite;
    end;

    // launch the command line compiler
    WasOK := CreateProcess(nil, PChar(CommandLine), nil, nil, True, 0, nil, nil, SI, PI);

    // Now that the handle has been inherited, close write to be safe.
    // We don't want to read or write to it accidentally.
    CloseHandle(StdOutPipeWrite);
    // if process could be created then handle its output
    if not WasOK then
      raise Exception.Create('Could not execute command line!')
    else
      try
        // get all output until dos app finishes
        Line := '';
        repeat
          // read block of characters (might contain carriage returns and line feeds)
          WasOK := ReadFile(StdOutPipeRead, Buffer, 255, BytesRead, nil);

          // has anything been read?
          if BytesRead > 0 then
          begin
            // finish buffer to PChar
            Buffer[BytesRead] := #0;
            // combine the buffer with the rest of the last run
            Line := Line + Buffer;
          end;
        until not WasOK or (BytesRead = 0);
        // wait for console app to finish (should be already at this point)
        WaitForSingleObject(PI.hProcess, INFINITE);
      finally
        // Close all remaining handles
        CloseHandle(PI.hThread);
        CloseHandle(PI.hProcess);
      end;
      finally
        result:=Line;
        CloseHandle(StdOutPipeRead);
      end;
  end;

  procedure AdvConfigForm();
  var
    frm:TfrmAdvConfig;
  begin
    frm := TfrmAdvConfig.Create(NIL);
    try
      frm.txtLocation.Text := locateexelocation;
      frm.txtMaxFoundFiles.Text := IntToStr(locatemaxfoundfiles);
      if(frm.ShowModal = mrOk) then
      begin
        locateexelocation := frm.txtLocation.Text;
        locatemaxfoundfiles := StrToInt(frm.txtMaxFoundFiles.Text);
        SaveSettings();
      end;
    finally
      frm.Free;
    end;
  end;

  procedure LoadSettings();
  begin
    inifile := TIniFile.Create(Concat(dlldir,'\','MyPlugin.ini'));
    locateexelocation := IniFile.ReadString('Global','LocateExeLocation','');
    locatemaxfoundfiles := IniFile.ReadInteger('Global','LocateMaxFoundFiles',15);
  end;

  procedure SaveSettings();
  begin
    inifile.WriteString('Global','LocateExeLocation',locateexelocation);
    inifile.WriteInteger('Global','LocateMaxFoundFiles',locatemaxfoundfiles);
  end;
end.
