#include "stdafx.h"
#include <windows.h>
#include "nwconnections.h"

/**************************************************************************
private function prototypes
**************************************************************************/
LPITEMIDLIST GetNWConnPidl(LPSHELLFOLDER);
BOOL GetNWConnContextMenu(LPCTSTR, IContextMenu**);
BOOL GetItemIdName(LPSHELLFOLDER, LPITEMIDLIST, DWORD, LPTSTR, UINT);
BOOL GetItemIdDefaultMenuID(LPSHELLFOLDER, LPITEMIDLIST pidl, HMENU&, UINT&, IContextMenu*&);
BOOL GetItemDefMenuTextById(HMENU, UINT, LPTSTR, UINT);
HRESULT IContextMenu_GetCommandString(IContextMenu*, UINT_PTR, UINT, UINT*, LPWSTR, UINT);
void RemoveAmpersand(LPTSTR str);

/**************************************************************************
functions implementation
**************************************************************************/
void EnumNWConnItems(NWConnList* pList)
{
  USES_CONVERSION;

  if (!pList) return;  // sanity check

  pList->clear();

  HRESULT        hr;
  LPMALLOC pMalloc = NULL; // memory manager, for freeing up PIDLs
  LPSHELLFOLDER  pDesktop, pNWConns;
  LPITEMIDLIST   pidlTemp, pidlNWConns;
  LPENUMIDLIST   pEnum;
  DWORD          dwRetrieved;   

  if(FAILED(SHGetMalloc(&pMalloc)))
    return;

  if(FAILED(SHGetDesktopFolder(&pDesktop)))
    return;

  pidlNWConns = GetNWConnPidl(pDesktop);

  if(pidlNWConns)
  {
    //get the IShellFolder for Network Connections
    hr = pDesktop->BindToObject(  pidlNWConns, 
      NULL, 
      IID_IShellFolder, 
      (LPVOID*)&pNWConns);

    if(SUCCEEDED(hr)) 
    {
      if(SUCCEEDED(pNWConns->EnumObjects(NULL, SHCONTF_FOLDERS | SHCONTF_NONFOLDERS, &pEnum)))
      {
        //enumerate the item's PIDLs
        do
        {
          hr = pEnum->Next(1, &pidlTemp, &dwRetrieved);          

          if(SUCCEEDED(hr) && dwRetrieved)
          {
            //check it's name
            TCHAR          szTemp[MAX_PATH+1] = {0};
            GetItemIdName(pNWConns, pidlTemp, SHGDN_NORMAL, szTemp, sizeof(szTemp));

            HMENU hMenu = NULL;
            UINT defMenuID = -1;
            IContextMenu* pContextMenu = NULL;

            if (GetItemIdDefaultMenuID(pNWConns, pidlTemp, hMenu, defMenuID, pContextMenu))
            {
              if (hMenu && pContextMenu && defMenuID != (UINT)-1) // sanity check
              {
                // NOTE: "Status" verb cannot be invoked by any process other than "explorer.exe" due to a shell bug.
                // (see: http://groups.google.com/group/microsoft.public.windowsxp.network_web/browse_thread/thread/fa38af52e9dbf6a3/f96b8089d334437c)
                // So for those PIDLs which have default verbs as "Status", I switched them to invoke "Properties" instead. 
                if (defMenuID == 5) defMenuID = 9;

                NWConnection nwConnItem = {0};

                // 1. populate connName
                _tcsncpy(nwConnItem.connName, szTemp, MAX_PATH);

                // 2. populate helpText
                WCHAR helpTextW[MAX_PATH+1] = {0};
                if (FAILED(IContextMenu_GetCommandString(pContextMenu, defMenuID - 1, GCS_VERBW, NULL, helpTextW, MAX_PATH))) 
                {
                  lstrcpynW(helpTextW, L"No help available.", MAX_PATH);
                }                
                _tcsncpy(nwConnItem.helpText, W2T(helpTextW), MAX_PATH);
                RemoveAmpersand(nwConnItem.helpText);

                // 3. populate menuText
                TCHAR menuText[MAX_PATH+1] = {0};                  
                GetItemDefMenuTextById(hMenu, defMenuID, menuText, MAX_PATH);                
                _tcsncpy(nwConnItem.menuText, menuText, MAX_PATH);
                RemoveAmpersand(nwConnItem.menuText);

                // 4. populate cmdId
                nwConnItem.menuCmdId = defMenuID;
                pList->push_back(nwConnItem);

                pContextMenu->Release();
                DestroyMenu(hMenu);
              }
            }
            pMalloc->Free(pidlTemp);
          }
        }
        while(SUCCEEDED(hr) && dwRetrieved);

        pEnum->Release();
      }

      pNWConns->Release();
    }
    pMalloc->Free(pidlNWConns);
  }

  pDesktop->Release();
  pMalloc->Release();
}

BOOL ExecNWConn(const TCHAR* connName, UINT cmdID)
{  
  BOOL bSuccess = FALSE;
  IContextMenu* pContextMenu = NULL; 
  
  if (GetNWConnContextMenu(connName, &pContextMenu) && pContextMenu)
  {
    CMINVOKECOMMANDINFO info = { 0 };
    info.cbSize = sizeof(info);
    info.hwnd = NULL;
    info.lpVerb = MAKEINTRESOURCEA(cmdID - 1);           
    pContextMenu->InvokeCommand(&info);
    pContextMenu->Release();
    bSuccess = TRUE;
  }

  return bSuccess;
}

LPITEMIDLIST GetNWConnPidl(LPSHELLFOLDER pDesktop)
{
  HRESULT        hr;
  LPITEMIDLIST   pidlNWConns;

  if(!pDesktop)
    return NULL;

  hr = SHGetSpecialFolderLocation(NULL, CSIDL_CONNECTIONS, &pidlNWConns);

  if(FAILED(hr)) 
    return NULL;

  return pidlNWConns;
}

BOOL GetNWConnContextMenu(LPCTSTR lpszName, IContextMenu** ppContextMenu)
{
  BOOL           bSuccess = FALSE;
  HRESULT        hr;
  LPMALLOC       pMalloc;    
  LPSHELLFOLDER  pDesktop, pNWConns;
  LPITEMIDLIST   pidlTemp, pidlNWConns;
  LPENUMIDLIST   pEnum;
  DWORD          dwRetrieved;
  TCHAR          szTemp[MAX_PATH+1] = {0};

  if (FAILED(SHGetMalloc(&pMalloc)))
    return FALSE;

  if (FAILED(SHGetDesktopFolder(&pDesktop)))
    return FALSE;

  pidlNWConns = GetNWConnPidl(pDesktop);

  if (pidlNWConns)
  {
    //get the IShellFolder for NWConns
    hr = pDesktop->BindToObject(pidlNWConns, NULL, IID_IShellFolder, (LPVOID*)&pNWConns);

    if(SUCCEEDED(hr)) 
    {
      if(SUCCEEDED(pNWConns->EnumObjects(NULL, SHCONTF_FOLDERS | SHCONTF_NONFOLDERS, &pEnum)))
      {
        //enumerate the item's PIDLs, looking for the one requested
        bool bFoundMatch = false;
        do
        {
          hr = pEnum->Next(1, &pidlTemp, &dwRetrieved);

          //check it's name
          if (SUCCEEDED(hr) && dwRetrieved)
          {
            GetItemIdName(pNWConns, pidlTemp, SHGDN_NORMAL, szTemp, sizeof(szTemp));
            bFoundMatch = (_tcsicmp(szTemp, lpszName) == 0);

            if (bFoundMatch)
            {              
              LPCITEMIDLIST apidl[] = { pidlTemp };  
              HRESULT hr = pNWConns->GetUIObjectOf(NULL, 1, apidl, IID_IContextMenu, NULL, reinterpret_cast<LPVOID *>(ppContextMenu));
              if (SUCCEEDED(hr))
              {
                bSuccess = TRUE;
              }
            }

            pMalloc->Free(pidlTemp);
          }          
        }
        while(SUCCEEDED(hr) && dwRetrieved && !bFoundMatch);

        pEnum->Release();
      }
      pNWConns->Release();
    }
    pMalloc->Free(pidlNWConns);
  }

  pDesktop->Release();
  pMalloc->Release();

  return bSuccess;
}

BOOL GetItemIdName(LPSHELLFOLDER pFolder, 
                   LPITEMIDLIST pidl, 
                   DWORD dwFlags, 
                   LPTSTR pszName, 
                   UINT cchMax)
{
  BOOL     fSuccess = TRUE;
  STRRET   str;

  if(SUCCEEDED(pFolder->GetDisplayNameOf(pidl, dwFlags, &str))) 
  {
    if (SUCCEEDED(StrRetToBuf(&str, pidl, pszName, cchMax)))
    {
      fSuccess = TRUE;
    }
  } 
  else 
  {
    fSuccess = FALSE;
  }

  return (fSuccess);
}

BOOL GetItemIdDefaultMenuID(LPSHELLFOLDER pFolder, 
                            LPITEMIDLIST pidl,
                            HMENU& hMenu,    // out
                            UINT& defMenuId, // out
                            IContextMenu*& pContextMenu) //out
{
  BOOL     fSuccess = FALSE;  

  hMenu     = NULL;
  defMenuId = -1;

  LPCITEMIDLIST apidl[] = { pidl };  
  HRESULT hr = pFolder->GetUIObjectOf(NULL, 1, apidl, IID_IContextMenu, NULL, reinterpret_cast<LPVOID *>(&pContextMenu));
  if (SUCCEEDED(hr))
  {
    hMenu = CreatePopupMenu();    
    if (hMenu && SUCCEEDED(pContextMenu->QueryContextMenu(hMenu, 0, 1, 0x7FFF, CMF_DEFAULTONLY))) 
    {
      defMenuId = GetMenuDefaultItem(hMenu, FALSE, 0);
      fSuccess = TRUE;
    }
    else
    {
      pContextMenu->Release();
      if (hMenu) DestroyMenu(hMenu);
      hMenu = NULL;
    }  
  }

  return (fSuccess);
}

BOOL GetItemDefMenuTextById(HMENU hMenu, UINT id, LPTSTR pszText, UINT cchMax)
{
  BOOL     fSuccess = FALSE;

  MENUITEMINFO mii = {0}; 
  mii.cbSize = sizeof(mii); 
  mii.fMask = MIIM_TYPE | MIIM_ID; 
  mii.dwTypeData = pszText;
  mii.cch = cchMax;
  mii.fType = MFT_STRING;

  if (GetMenuItemInfo(hMenu, id, FALSE, &mii) && 
    !(mii.fType & MFT_SEPARATOR) && IS_INTRESOURCE(mii.wID) && 
    (mii.wID >= 1))
  {
    fSuccess = TRUE;
  }

  return (fSuccess);
}

HRESULT IContextMenu_GetCommandString(IContextMenu *pcm, UINT_PTR idCmd, UINT uFlags,
                                      UINT *pwReserved, LPWSTR pszName, UINT cchMax)
{
  // Callers are expected to be using Unicode.
  if (!(uFlags & GCS_UNICODE)) return E_INVALIDARG;

  // Some context menu handlers have off-by-one bugs and will
  // overflow the output buffer. Let's artificially reduce the
  // buffer size so a one-character overflow won't corrupt memory.
  if (cchMax <= 1) return E_FAIL;
  cchMax--;

  // First try the Unicode message.  Preset the output buffer
  // with a known value because some handlers return S_OK without
  // doing anything.
  pszName[0] = L'\0';

  HRESULT hr = pcm->GetCommandString(idCmd, uFlags, pwReserved,
    (LPSTR)pszName, cchMax);

  if (SUCCEEDED(hr) && pszName[0] == L'\0') 
  {
    // Rats, a buggy IContextMenu handler that returned success
    // even though it failed.
    hr = E_NOTIMPL;
  }

  if (FAILED(hr)) 
  {
    // try again with ANSI - pad the buffer with one extra character
    // to compensate for context menu handlers that overflow by
    // one character.
    LPSTR pszAnsi = (LPSTR)LocalAlloc(LMEM_FIXED, (cchMax + 1) * sizeof(CHAR));
    if (pszAnsi) 
    {
      pszAnsi[0] = '\0';
      hr = pcm->GetCommandString(idCmd, uFlags & ~GCS_UNICODE, pwReserved, pszAnsi, cchMax);

      if (SUCCEEDED(hr) && pszAnsi[0] == '\0') 
      {
        // Rats, a buggy IContextMenu handler that returned success
        // even though it failed.
        hr = E_NOTIMPL;
      }

      if (SUCCEEDED(hr)) 
      {
        if (MultiByteToWideChar(CP_ACP, 0, pszAnsi, -1,
          pszName, cchMax) == 0) {
            hr = E_FAIL;
        }
      }

      LocalFree(pszAnsi);
    } 
    else 
    {
      hr = E_OUTOFMEMORY;
    }
  }
  return hr;
}

void RemoveAmpersand(LPTSTR str)
{
  TCHAR* pSrc = str;
  TCHAR* pDest = str;

  while (*pSrc)
  {
    if (*pSrc != _T('&'))
    {
      *pDest = *pSrc;
      pDest = _tcsinc(pDest);
      pSrc = _tcsinc(pSrc);
    }
    else
    {
      pSrc = _tcsinc(pSrc);
    }
  }
  *(pDest) = NULL;
}