/**************************************************************************
#include statements
**************************************************************************/
#include "stdafx.h"
#include <windows.h>
#include "winspool.h"
#include "Printers.h"

/**************************************************************************
private function prototypes
**************************************************************************/
#define MAX_DATA 4096

typedef BOOL (__stdcall * pFuncGetDefaultPrinter)(
  OUT LPTSTR pszBuffer,   // printer name buffer
  LPDWORD pcchBuffer  // size of name buffer
  );


typedef BOOL (__stdcall * pFuncSetDefaultPrinter)(
  IN LPCTSTR pszPrinter   // printer name 
  );

LPITEMIDLIST GetPrintersPidl(LPSHELLFOLDER);
LPITEMIDLIST GetThisPrinter(LPCTSTR, LPMALLOC);
LPITEMIDLIST GetAddPrinterItem(LPMALLOC);
BOOL OpenThisPrinter(LPCTSTR);
BOOL GetItemIdName(LPSHELLFOLDER, LPITEMIDLIST, DWORD, LPTSTR, UINT);
BOOL GetItemIdFriendlyName(LPSHELLFOLDER, LPITEMIDLIST, LPTSTR, UINT);
LPITEMIDLIST Pidl_Concatenate(LPMALLOC, LPCITEMIDLIST, LPCITEMIDLIST);
UINT Pidl_GetSize(LPCITEMIDLIST);
LPITEMIDLIST Pidl_Create(LPMALLOC, UINT);
inline LPITEMIDLIST Pidl_GetNextItem(LPCITEMIDLIST);
BOOL Pidl_GetDisplayName(LPSHELLFOLDER, LPITEMIDLIST, DWORD, LPTSTR, UINT);

/**************************************************************************
global variables
**************************************************************************/



/**************************************************************************

OpenPrinterProperties()

**************************************************************************/
BOOL OpenPrinterProperties(LPCTSTR lpszPrinterName)
{
  LPMALLOC          pMalloc;
  LPITEMIDLIST      pidlPrinter;
  SHELLEXECUTEINFO  sei;

  if(FAILED(SHGetMalloc(&pMalloc)))
    return FALSE;

  pidlPrinter = GetThisPrinter(lpszPrinterName, pMalloc);

  if(!pidlPrinter)
  {
    return FALSE;
  }

  ZeroMemory(&sei, sizeof(sei));
  sei.cbSize     = sizeof(SHELLEXECUTEINFO);
  sei.fMask      = SEE_MASK_INVOKEIDLIST | SEE_MASK_FLAG_NO_UI;
  sei.lpIDList   = pidlPrinter;
  sei.lpVerb     = TEXT("Properties");
  sei.nShow      = SW_SHOWDEFAULT;

  ShellExecuteEx(&sei);

  pMalloc->Free(pidlPrinter);

  return TRUE;
}


/**************************************************************************

EnumPrinterItems()

**************************************************************************/
void EnumPrinterItems(FarrPrinterList* pList)
{
  if (!pList) return;

  pList->clear();

  HRESULT        hr;
  LPSHELLFOLDER  pDesktop, pPrinters;
  LPITEMIDLIST   pidlTemp, pidlPrinters;
  LPENUMIDLIST   pEnum;
  DWORD          dwRetrieved;
  TCHAR          szTemp[MAX_PATH+1] = {0};
  int            i = 0;

  TCHAR          defPrinter[MAX_PATH+1] = {0};
  DWORD          defPrinterBufSize = MAX_PATH;
  DPGetDefaultPrinter(defPrinter, &defPrinterBufSize);

  if(FAILED(SHGetDesktopFolder(&pDesktop)))
    return;

  pidlPrinters = GetPrintersPidl(pDesktop);

  if(pidlPrinters)
  {
    //get the IShellFolder for Printers
    hr = pDesktop->BindToObject(  pidlPrinters, 
      NULL, 
      IID_IShellFolder, 
      (LPVOID*)&pPrinters);

    if(SUCCEEDED(hr)) 
    {
      if(SUCCEEDED(pPrinters->EnumObjects(NULL, SHCONTF_FOLDERS | SHCONTF_NONFOLDERS, &pEnum)))
      {
        //enumerate the item's PIDLs
        do
        {
          hr = pEnum->Next(1, &pidlTemp, &dwRetrieved);

          //check it's name
          GetItemIdName(pPrinters, pidlTemp, SHGDN_NORMAL, szTemp, sizeof(szTemp));

          if(i && SUCCEEDED(hr) && dwRetrieved)
          {
            FarrPrinter printerItem = {0};

            // printer name
            _tcsncpy(printerItem.printerName, szTemp, MAX_PATH);

            // printer friendly name
            if (GetItemIdFriendlyName(pPrinters, pidlTemp, szTemp, sizeof(szTemp)))
            {
              _tcsncpy(printerItem.printerFriendlyName, szTemp, MAX_PATH);
            }

            if (_tcslen(printerItem.printerFriendlyName))
            {
              printerItem.bIsDefault = (_tcsicmp(printerItem.printerFriendlyName, defPrinter) == 0);
            }
            else
            {
              printerItem.bIsDefault = false;
            }

            pList->push_back(printerItem);
          }

          i = 1;
        }
        while(SUCCEEDED(hr) && dwRetrieved);

        pEnum->Release();
      }

      pPrinters->Release();
    }
  }

  pDesktop->Release();
}

/**************************************************************************

CreatePrinterShortcut()

Parameters:

LPTSTR lpszPrinterName -   The display name of the existing printer 
to create the shortcut for.
LPTSTR lpszShortcutName -  The name of the shortcut. The .LNK 
extension will be added to the end of 
this name if it is not present.

Returns - Boolean indicating succes or failure.

**************************************************************************/

BOOL CreatePrinterShortcut(LPTSTR lpszPrinterName, LPTSTR lpszShortcutName)
{
  HRESULT        hr;
  IShellLink     *pShellLink; 
  BOOL           bReturn;
  LPITEMIDLIST   pidlPrinter;
  LPMALLOC       pMalloc;

  if(FAILED(SHGetMalloc(&pMalloc)))
    return FALSE;

  pidlPrinter = GetThisPrinter(lpszPrinterName, pMalloc);

  if(!pidlPrinter)
  {
    return FALSE;
  }

  CoInitialize(NULL);

  //create a new IShellLink interface
  hr = CoCreateInstance(  CLSID_ShellLink, 
    NULL, 
    CLSCTX_INPROC_SERVER, 
    IID_IShellLink, 
    (LPVOID*)&pShellLink); 

  bReturn = FALSE;

  if(SUCCEEDED(hr)) 
  { 
    IPersistFile* pPersistFile; 

    //set the PIDL for the shortcut target
    hr = pShellLink->SetIDList(pidlPrinter); 

    if(SUCCEEDED(hr)) 
    {
      // Query IShellLink for the IPersistFile interface for saving the 
      // shortcut in persistent storage. 
      hr = pShellLink->QueryInterface(IID_IPersistFile, (LPVOID*)&pPersistFile); 

      if(SUCCEEDED(hr)) 
      { 
        WCHAR wsz[MAX_PATH]; 

        //make sure that the link name ends with a ".LNK" extension
        if(lstrcmpi(lpszShortcutName + lstrlen(lpszShortcutName) - 4, ".lnk"))
        {
          //add the proper extension
          lstrcat(lpszShortcutName, ".lnk");
        }

        // Ensure that the string is ANSI. 
        MultiByteToWideChar( CP_ACP, 
          0, 
          lpszShortcutName, 
          -1, 
          wsz, 
          MAX_PATH); 

        // Save the link by calling IPersistFile::Save. 
        if(SUCCEEDED(pPersistFile->Save(wsz, TRUE)))
          bReturn = TRUE;

        pPersistFile->Release(); 
      } 

      pShellLink->Release(); 
    }
  } 

  pMalloc->Free(pidlPrinter);

  return bReturn;
}

/**************************************************************************

OpenThisPrinter()

**************************************************************************/

BOOL OpenThisPrinter(LPCTSTR lpszPrinterName)
{
  LPMALLOC          pMalloc;
  LPITEMIDLIST      pidlPrinter;
  SHELLEXECUTEINFO  sei;

  if(FAILED(SHGetMalloc(&pMalloc)))
    return FALSE;

  pidlPrinter = GetThisPrinter(lpszPrinterName, pMalloc);

  if(!pidlPrinter)
  {
    return FALSE;
  }

  ZeroMemory(&sei, sizeof(sei));
  sei.cbSize     = sizeof(SHELLEXECUTEINFO);
  sei.fMask      = SEE_MASK_INVOKEIDLIST | SEE_MASK_FLAG_NO_UI;
  sei.lpIDList   = pidlPrinter;
  sei.nShow      = SW_SHOWDEFAULT;

  ShellExecuteEx(&sei);

  pMalloc->Free(pidlPrinter);

  return TRUE;
}

/**************************************************************************

CreateAddPrinterShortcut()

Parameters:

LPTSTR lpszShortcutName -  The name of the shortcut. The .LNK 
extension will be added to the end of 
this name if it is not present.

Returns - Boolean indicating success or failure.

**************************************************************************/

BOOL CreateAddPrinterShortcut(LPTSTR lpszShortcutName)
{
  HRESULT        hr;
  IShellLink     *pShellLink; 
  BOOL           bReturn;
  LPITEMIDLIST   pidlPrinter;
  LPMALLOC       pMalloc;

  if(FAILED(SHGetMalloc(&pMalloc)))
    return FALSE;

  pidlPrinter = GetAddPrinterItem(pMalloc);

  if(!pidlPrinter)
  {
    return FALSE;
  }

  CoInitialize(NULL);

  //create a new IShellLink interface
  hr = CoCreateInstance(  CLSID_ShellLink, 
    NULL, 
    CLSCTX_INPROC_SERVER, 
    IID_IShellLink, 
    (LPVOID*)&pShellLink); 

  bReturn = FALSE;

  if(SUCCEEDED(hr)) 
  { 
    IPersistFile* pPersistFile; 

    //set the PIDL for the shortcut target
    hr = pShellLink->SetIDList(pidlPrinter); 

    if(SUCCEEDED(hr)) 
    {
      // Query IShellLink for the IPersistFile interface for saving the 
      // shortcut in persistent storage. 
      hr = pShellLink->QueryInterface(IID_IPersistFile, (LPVOID*)&pPersistFile); 

      if(SUCCEEDED(hr)) 
      { 
        WCHAR wsz[MAX_PATH]; 

        //make sure that the link name ends with a ".LNK" extension
        if(lstrcmpi(lpszShortcutName + lstrlen(lpszShortcutName) - 4, ".lnk"))
        {
          //add the proper extension
          lstrcat(lpszShortcutName, ".lnk");
        }

        // Ensure that the string is ANSI. 
        MultiByteToWideChar( CP_ACP, 
          0, 
          lpszShortcutName, 
          -1, 
          wsz, 
          MAX_PATH); 

        // Save the link by calling IPersistFile::Save. 
        if(SUCCEEDED(pPersistFile->Save(wsz, TRUE)))
          bReturn = TRUE;

        pPersistFile->Release(); 
      } 

      pShellLink->Release(); 
    }
  } 

  pMalloc->Free(pidlPrinter);

  return bReturn;
}

/**************************************************************************

ExecuteAddPrinter()

**************************************************************************/

BOOL ExecuteAddPrinter(void)
{
#if 1

  LPMALLOC          pMalloc;
  LPITEMIDLIST      pidlAddPrinter;
  SHELLEXECUTEINFO  sei;

  if(FAILED(SHGetMalloc(&pMalloc)))
    return FALSE;

  pidlAddPrinter = GetAddPrinterItem(pMalloc);

  if(!pidlAddPrinter)
  {
    return FALSE;
  }

  ZeroMemory(&sei, sizeof(sei));
  sei.cbSize     = sizeof(SHELLEXECUTEINFO);
  sei.fMask      = SEE_MASK_INVOKEIDLIST | SEE_MASK_FLAG_NO_UI;
  sei.lpIDList   = pidlAddPrinter;
  sei.nShow      = SW_SHOWDEFAULT;

  ShellExecuteEx(&sei);

  pMalloc->Free(pidlAddPrinter);

  return TRUE;

#else

  WinExec("rundll32 shell32,SHHelpShortcuts_RunDLL AddPrinter", SW_SHOWNORMAL);

#endif

  return TRUE;
}

/**************************************************************************

GetPrintersPidl()

Returns - 

**************************************************************************/

LPITEMIDLIST GetPrintersPidl(LPSHELLFOLDER pDesktop)
{
  HRESULT        hr;
  LPITEMIDLIST   pidlPrinters;

  if(!pDesktop)
    return NULL;

  hr = SHGetSpecialFolderLocation(NULL, CSIDL_PRINTERS, &pidlPrinters);

  if(FAILED(hr)) 
    return NULL;

  return pidlPrinters;
}

/**************************************************************************

GetAddPrinterItem()

**************************************************************************/

LPITEMIDLIST GetAddPrinterItem(LPMALLOC pMalloc)
{
  HRESULT        hr;
  LPSHELLFOLDER  pDesktop,
    pidlPrinters;
  LPITEMIDLIST   pidlTarget = NULL,
    pidlTemp,
    pidlPrinter;
  LPENUMIDLIST   pEnum;
  DWORD          dwRetrieved;

  if(!pMalloc)
    return NULL;

  if(FAILED(SHGetDesktopFolder(&pDesktop)))
    return NULL;

  pidlPrinter = GetPrintersPidl(pDesktop);

  if(pidlPrinter)
  {
    //get the IShellFolder for Printers
    hr = pDesktop->BindToObject(  pidlPrinter, 
      NULL, 
      IID_IShellFolder, 
      (LPVOID*)&pidlPrinters);

    if(SUCCEEDED(hr)) 
    {
      if(SUCCEEDED(pidlPrinters->EnumObjects(NULL, SHCONTF_FOLDERS | SHCONTF_NONFOLDERS, &pEnum)))
      {
        //get the first item's PIDL, this will be the item for adding a printer
        hr = pEnum->Next(1, &pidlTemp, &dwRetrieved);

        if(dwRetrieved && SUCCEEDED(hr)) 
        {
          //we need to concatenate the item's PIDL onto the Printers PIDL
          pidlTarget = Pidl_Concatenate(pMalloc, pidlPrinter, pidlTemp);
        }

        pEnum->Release();
      }
      pidlPrinters->Release();
    }
  }

  pDesktop->Release();

  return pidlTarget;
}

/**************************************************************************

GetThisPrinter()

**************************************************************************/

LPITEMIDLIST GetThisPrinter(LPCTSTR lpszName, LPMALLOC pMalloc)
{
  HRESULT        hr;
  LPSHELLFOLDER  pDesktop,
    pPrinters;
  LPITEMIDLIST   pidlTarget,
    pidlTemp,
    pidlPrinter;
  LPENUMIDLIST   pEnum;
  DWORD          dwRetrieved;
  char           szTemp[MAX_PATH];

  if(!pMalloc)
    return NULL;

  if(FAILED(SHGetDesktopFolder(&pDesktop)))
    return NULL;

  pidlPrinter = GetPrintersPidl(pDesktop);

  if(pidlPrinter)
  {
    //get the IShellFolder for Printers
    hr = pDesktop->BindToObject(  pidlPrinter, 
      NULL, 
      IID_IShellFolder, 
      (LPVOID*)&pPrinters);

    if(SUCCEEDED(hr)) 
    {
      if(SUCCEEDED(pPrinters->EnumObjects(NULL, SHCONTF_FOLDERS | SHCONTF_NONFOLDERS, &pEnum)))
      {
        //enumerate the item's PIDLs, looking for the one requested
        do
        {
          hr = pEnum->Next(1, &pidlTemp, &dwRetrieved);

          //check it's name
          GetItemIdName(pPrinters, pidlTemp, SHGDN_NORMAL, szTemp, sizeof(szTemp));
        }
        while(SUCCEEDED(hr) && dwRetrieved && (lstrcmpi(szTemp, lpszName)));


        if(dwRetrieved && SUCCEEDED(hr)) 
        {
          //we need to concatenate the item's PIDL onto the Printers PIDL
          pidlTarget = Pidl_Concatenate(pMalloc, pidlPrinter, pidlTemp);
        }

        pEnum->Release();
      }
      pPrinters->Release();
    }
  }

  pDesktop->Release();

  return pidlTarget;
}

/**************************************************************************

GetItemIdName()

**************************************************************************/

BOOL GetItemIdName(LPSHELLFOLDER pFolder, 
                   LPITEMIDLIST pidl, 
                   DWORD dwFlags, 
                   LPTSTR pszName, 
                   UINT cchMax)
{
  BOOL     fSuccess = TRUE;
  STRRET   str;

  if(SUCCEEDED(pFolder->GetDisplayNameOf(pidl, dwFlags, &str))) 
  {
    if (SUCCEEDED(StrRetToBuf(&str, pidl, pszName, cchMax)))
    {
      fSuccess = TRUE;
    }
  } 
  else 
  {
    fSuccess = FALSE;
  }

  return (fSuccess);
}


BOOL GetItemIdFriendlyName(LPSHELLFOLDER pFolder, 
                           LPITEMIDLIST pidl,
                           LPTSTR pszName, 
                           UINT cchMax)
{
  BOOL     fSuccess = TRUE;  

  LPCITEMIDLIST apidl[] = { pidl };
  IDataObject* pDataObject = NULL;

  if(SUCCEEDED(pFolder->GetUIObjectOf(NULL, 1, apidl, IID_IDataObject, NULL, reinterpret_cast<LPVOID *>(&pDataObject))))
  {    
    FORMATETC fmt = {0};
    fmt.cfFormat  = RegisterClipboardFormat(CFSTR_PRINTERGROUP);
    fmt.ptd       = 0; 
    fmt.dwAspect  = DVASPECT_CONTENT; 
    fmt.lindex    = -1; 
    fmt.tymed     = TYMED_HGLOBAL; 

    STGMEDIUM medium = {0};
    if (SUCCEEDED(pDataObject->GetData(&fmt, &medium)))
    {      
      // we asked for a global memory handle, must lock it for access
      LPCTSTR drop_files = (LPCTSTR)GlobalLock(medium.hGlobal);       
      DROPFILES *files = (DROPFILES*)drop_files;           
      UINT fileNameLen = DragQueryFile((HDROP)files, 0, pszName, cchMax);

      GlobalUnlock(medium.hGlobal);
      ReleaseStgMedium(&medium); // will call GlobalFree

      fSuccess = TRUE;
    }
    else
    {
      fSuccess = FALSE;
    }
    pDataObject->Release();
  } 
  else 
  {
    fSuccess = FALSE;
  }

  return (fSuccess);   
}

#ifdef UNICODE
#define GETDEFAULTPRINTER "GetDefaultPrinterW"
#else
#define GETDEFAULTPRINTER "GetDefaultPrinterA"
#endif

BOOL DPGetDefaultPrinter(LPTSTR pPrinterName, LPDWORD pdwBufferSize)
{
  BOOL bFlag;
  OSVERSIONINFO osv;
  TCHAR cBuffer[MAX_DATA];
  PRINTER_INFO_2 *ppi2 = NULL;
  DWORD dwNeeded = 0;
  DWORD dwReturned = 0;
  HMODULE hWinSpool = NULL;
  pFuncGetDefaultPrinter fnGetDefaultPrinter = NULL;

  // What version of Windows are you running?
  osv.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
  GetVersionEx(&osv);

  // If Windows 95 or 98, use EnumPrinters.
  if (osv.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
  {
    // The first EnumPrinters() tells you how big our buffer must
    // be to hold ALL of PRINTER_INFO_2. Note that this will
    // typically return FALSE. This only means that the buffer (the 4th
    // parameter) was not filled in. You do not want it filled in here.
    SetLastError(0);
    bFlag = EnumPrinters(PRINTER_ENUM_DEFAULT, NULL, 2, NULL, 0, &dwNeeded, &dwReturned);
    {
      if ((GetLastError() != ERROR_INSUFFICIENT_BUFFER) || (dwNeeded == 0))
        return FALSE;
    }

    // Allocate enough space for PRINTER_INFO_2.
    ppi2 = (PRINTER_INFO_2 *)GlobalAlloc(GPTR, dwNeeded);
    if (!ppi2)
      return FALSE;

    // The second EnumPrinters() will fill in all the current information.
    bFlag = EnumPrinters(PRINTER_ENUM_DEFAULT, NULL, 2, (LPBYTE)ppi2, dwNeeded, &dwNeeded, &dwReturned);
    if (!bFlag)
    {
      GlobalFree(ppi2);
      return FALSE;
    }

    // If specified buffer is too small, set required size and fail.
    if ((DWORD)lstrlen(ppi2->pPrinterName) >= *pdwBufferSize)
    {
      *pdwBufferSize = (DWORD)lstrlen(ppi2->pPrinterName) + 1;
      GlobalFree(ppi2);
      return FALSE;
    }

    // Copy printer name into passed-in buffer.
    lstrcpy(pPrinterName, ppi2->pPrinterName);

    // Set buffer size parameter to minimum required buffer size.
    *pdwBufferSize = (DWORD)lstrlen(ppi2->pPrinterName) + 1;
  }

  // If Windows NT, use the GetDefaultPrinter API for Windows 2000,
  // or GetProfileString for version 4.0 and earlier.
  else if (osv.dwPlatformId == VER_PLATFORM_WIN32_NT)
  {
    if (osv.dwMajorVersion >= 5) // Windows 2000 or later (use explicit call)
    {
      hWinSpool = LoadLibrary(_T("winspool.drv"));
      if (!hWinSpool)
        return FALSE;
      fnGetDefaultPrinter = (pFuncGetDefaultPrinter)GetProcAddress(hWinSpool, GETDEFAULTPRINTER);
      if (!fnGetDefaultPrinter)
      {
        FreeLibrary(hWinSpool);
        return FALSE;
      }

      bFlag = fnGetDefaultPrinter(pPrinterName, pdwBufferSize);
      FreeLibrary(hWinSpool);
      if (!bFlag)
        return FALSE;
    }

    else // NT4.0 or earlier
    {
      // Retrieve the default string from Win.ini (the registry).
      // String will be in form "printername,drivername,portname".
      if (GetProfileString("windows", "device", ",,,", cBuffer, MAX_DATA) <= 0)
        return FALSE;

      // Printer name precedes first "," character.
      strtok(cBuffer, ",");

      // If specified buffer is too small, set required size and fail.
      if ((DWORD)lstrlen(cBuffer) >= *pdwBufferSize)
      {
        *pdwBufferSize = (DWORD)lstrlen(cBuffer) + 1;
        return FALSE;
      }

      // Copy printer name into passed-in buffer.
      lstrcpy(pPrinterName, cBuffer);

      // Set buffer size parameter to minimum required buffer size.
      *pdwBufferSize = (DWORD)lstrlen(cBuffer) + 1;
    }
  }

  // Clean up.
  if (ppi2)
    GlobalFree(ppi2);

  return TRUE;
}


#ifdef UNICODE
#define SETDEFAULTPRINTER "SetDefaultPrinterW"
#else
#define SETDEFAULTPRINTER "SetDefaultPrinterA"
#endif

BOOL DPSetDefaultPrinter(LPTSTR pPrinterName)
{
  BOOL bFlag;
  OSVERSIONINFO osv;
  DWORD dwNeeded = 0;
  HANDLE hPrinter = NULL;
  PRINTER_INFO_2 *ppi2 = NULL;
  LPTSTR pBuffer = NULL;
  LONG lResult;
  HMODULE hWinSpool = NULL;
  pFuncSetDefaultPrinter fnSetDefaultPrinter = NULL;
  BOOL bNeedsNotify = TRUE;

  // What version of Windows are you running?
  osv.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
  GetVersionEx(&osv);

  if (!pPrinterName)
    return FALSE;

  // If Windows 95 or 98, use SetPrinter.
  if (osv.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
  {
    // Open this printer so you can get information about it.
    bFlag = OpenPrinter(pPrinterName, &hPrinter, NULL);
    if (!bFlag || !hPrinter)
      return FALSE;

    // The first GetPrinter() tells you how big our buffer must
    // be to hold ALL of PRINTER_INFO_2. Note that this will
    // typically return FALSE. This only means that the buffer (the 3rd
    // parameter) was not filled in. You do not want it filled in here.
    SetLastError(0);
    bFlag = GetPrinter(hPrinter, 2, 0, 0, &dwNeeded);
    if (!bFlag)
    {
      if ((GetLastError() != ERROR_INSUFFICIENT_BUFFER) || (dwNeeded == 0))
      {
        ClosePrinter(hPrinter);
        return FALSE;
      }
    }

    // Allocate enough space for PRINTER_INFO_2.
    ppi2 = (PRINTER_INFO_2 *)GlobalAlloc(GPTR, dwNeeded);
    if (!ppi2)
    {
      ClosePrinter(hPrinter);
      return FALSE;
    }

    // The second GetPrinter() will fill in all the current information
    // so that all you have to do is modify what you are interested in.
    bFlag = GetPrinter(hPrinter, 2, (LPBYTE)ppi2, dwNeeded, &dwNeeded);
    if (!bFlag)
    {
      ClosePrinter(hPrinter);
      GlobalFree(ppi2);
      return FALSE;
    }

    // Set default printer attribute for this printer.
    ppi2->Attributes |= PRINTER_ATTRIBUTE_DEFAULT;
    bFlag = SetPrinter(hPrinter, 2, (LPBYTE)ppi2, 0);
    if (!bFlag)
    {
      ClosePrinter(hPrinter);
      GlobalFree(ppi2);
      return FALSE;
    }

    // Tell all open programs that this change occurred. 
    // Allow each program 1 second to handle this message.
    lResult = (LONG)SendMessageTimeout(HWND_BROADCAST, WM_SETTINGCHANGE, 0L,
      (LPARAM)(LPCTSTR)"windows", SMTO_NORMAL, 1000, NULL);
  }

  // If Windows NT, use the SetDefaultPrinter API for Windows 2000,
  // or WriteProfileString for version 4.0 and earlier.
  else if (osv.dwPlatformId == VER_PLATFORM_WIN32_NT)
  {
    if (osv.dwMajorVersion >= 5) // Windows 2000 or later (use explicit call)
    {
      bNeedsNotify = FALSE;

      hWinSpool = LoadLibrary(_T("winspool.drv"));
      if (!hWinSpool)
        return FALSE;
      fnSetDefaultPrinter = (pFuncSetDefaultPrinter)GetProcAddress(hWinSpool, SETDEFAULTPRINTER);
      if (!fnSetDefaultPrinter)
      {
        FreeLibrary(hWinSpool);
        return FALSE;
      }

      bFlag = fnSetDefaultPrinter(pPrinterName);
      FreeLibrary(hWinSpool);
      if (!bFlag)
        return FALSE;
    }

    else // NT4.0 or earlier
    {
      // Open this printer so you can get information about it.
      bFlag = OpenPrinter(pPrinterName, &hPrinter, NULL);
      if (!bFlag || !hPrinter)
        return FALSE;

      // The first GetPrinter() tells you how big our buffer must
      // be to hold ALL of PRINTER_INFO_2. Note that this will
      // typically return FALSE. This only means that the buffer (the 3rd
      // parameter) was not filled in. You do not want it filled in here.
      SetLastError(0);
      bFlag = GetPrinter(hPrinter, 2, 0, 0, &dwNeeded);
      if (!bFlag)
      {
        if ((GetLastError() != ERROR_INSUFFICIENT_BUFFER) || (dwNeeded == 0))
        {
          ClosePrinter(hPrinter);
          return FALSE;
        }
      }

      // Allocate enough space for PRINTER_INFO_2.
      ppi2 = (PRINTER_INFO_2 *)GlobalAlloc(GPTR, dwNeeded);
      if (!ppi2)
      {
        ClosePrinter(hPrinter);
        return FALSE;
      }

      // The second GetPrinter() fills in all the current<BR/>
      // information.
      bFlag = GetPrinter(hPrinter, 2, (LPBYTE)ppi2, dwNeeded, &dwNeeded);
      if ((!bFlag) || (!ppi2->pDriverName) || (!ppi2->pPortName))
      {
        ClosePrinter(hPrinter);
        GlobalFree(ppi2);
        return FALSE;
      }

      // Allocate buffer big enough for concatenated string.
      // String will be in form "printername,drivername,portname".
      pBuffer = (LPTSTR)GlobalAlloc(GPTR,
        lstrlen(pPrinterName) +
        lstrlen(ppi2->pDriverName) +
        lstrlen(ppi2->pPortName) + 3);
      if (!pBuffer)
      {
        ClosePrinter(hPrinter);
        GlobalFree(ppi2);
        return FALSE;
      }

      // Build string in form "printername,drivername,portname".
      lstrcpy(pBuffer, pPrinterName);  lstrcat(pBuffer, ",");
      lstrcat(pBuffer, ppi2->pDriverName);  lstrcat(pBuffer, ",");
      lstrcat(pBuffer, ppi2->pPortName);

      // Set the default printer in Win.ini and registry.
      bFlag = WriteProfileString("windows", "device", pBuffer);
      if (!bFlag)
      {
        ClosePrinter(hPrinter);
        GlobalFree(ppi2);
        GlobalFree(pBuffer);
        return FALSE;
      }
    }

    // Tell all open programs that this change occurred. 
    // Allow each app 1 second to handle this message.
    if (bNeedsNotify)
    {
      lResult = (LONG)SendMessageTimeout(HWND_BROADCAST, WM_SETTINGCHANGE, 0L, 0L,
        SMTO_NORMAL, 1000, NULL);
    }    
  }

  // Clean up.
  if (hPrinter)
    ClosePrinter(hPrinter);
  if (ppi2)
    GlobalFree(ppi2);
  if (pBuffer)
    GlobalFree(pBuffer);

  return TRUE;
}

//
//  FUNCTION:   Pidl_Concatenate
//
//  PURPOSE:    Creates a new ITEMIDLIST with pidl2 appended to pidl1.
//
//  PARAMETERS:
//		piMalloc - Pointer to the allocator interface that should create the
//				   new ITEMIDLIST.
//      pidl1 	 - Pointer to an ITEMIDLIST that contains the root.
//		pidl2    - Pointer to an ITEMIDLIST that contains what should be 
//				   appended to the root.
//
//  RETURN VALUE:
//  	Returns a new ITEMIDLIST if successful, NULL otherwise.  
//

LPITEMIDLIST Pidl_Concatenate(   LPMALLOC piMalloc, 
                              LPCITEMIDLIST pidl1,
                              LPCITEMIDLIST pidl2)
{
  LPITEMIDLIST pidlNew;
  UINT cb1, cb2 = 0;

  //
  // Pidl1 can possibly be NULL if it points to the desktop.  Since we only
  // need a single NULL terminator, we remove the extra 2 bytes from the
  // size of the first ITEMIDLIST.
  //
  if (pidl1)
    cb1 = Pidl_GetSize(pidl1) - (2 * sizeof(BYTE));

  cb2 = Pidl_GetSize(pidl2);

  //
  // Create a new ITEMIDLIST that is the size of both pidl1 and pidl2, then
  // copy pidl1 and pidl2 to the new list.
  //
  pidlNew = Pidl_Create(piMalloc, cb1 + cb2);
  if (pidlNew)
  {
    if (pidl1)	
      CopyMemory(pidlNew, pidl1, cb1);

    CopyMemory(((LPBYTE)pidlNew) + cb1, pidl2, cb2);
  }

  return (pidlNew);
}


//
//  FUNCTION:   Pidl_GetSize 
//
//  PURPOSE:    Returns the total number of bytes in an ITEMIDLIST.
//
//  PARAMETERS:
//      pidl - Pointer to the ITEMIDLIST that you want the size of.
//

UINT Pidl_GetSize(LPCITEMIDLIST pidl)
{
  UINT cbTotal = 0;
  LPITEMIDLIST pidlTemp = (LPITEMIDLIST) pidl;

  if (pidlTemp)
  {
    while (pidlTemp->mkid.cb)
    {
      cbTotal += pidlTemp->mkid.cb;
      pidlTemp = Pidl_GetNextItem(pidlTemp);
    }	

    // Requires a 16 bit zero value for the NULL terminator
    cbTotal += 2 * sizeof(BYTE);
  }

  return (cbTotal);
}

//
//  FUNCTION:   Pidl_Create 
//
//  PURPOSE:    Creates a new ITEMIDLIST of the specified size.
//
//  PARAMETERS:
//      piMalloc - Pointer to the allocator interface that should allocate
//				   memory.
//		cbSize   - Size of the ITEMIDLIST to create.
//
//  RETURN VALUE:
//      Returns a pointer to the new ITEMIDLIST, or NULL if a problem occured.
//

LPITEMIDLIST Pidl_Create(LPMALLOC piMalloc, UINT cbSize)
{
  LPITEMIDLIST pidl = NULL;

  pidl = (LPITEMIDLIST) piMalloc->Alloc(cbSize);
  if (pidl)
    ZeroMemory(pidl, cbSize);

  return (pidl);
}


//
//  FUNCTION:   Pidl_GetNextItem 
//
//  PURPOSE:    Returns a pointer to the next item in the ITEMIDLIST.
//
//  PARAMETERS:
//      pidl - Pointer to an ITEMIDLIST to walk through
//

inline LPITEMIDLIST Pidl_GetNextItem(LPCITEMIDLIST pidl)
{
  if (pidl)
    return (LPITEMIDLIST) (LPBYTE)(((LPBYTE)pidl) + pidl->mkid.cb);
  else
    return (NULL);
}


//
//  FUNCTION:   Pidl_GetDisplayName 
//
//  PURPOSE:    Returns the display name for the item pointed to by pidl.  The
//				function assumes the pidl is relative to piFolder.  If piFolder
//				is NULL, the function assumes the item is fully qualified.
//
//  PARAMETERS:
//      piFolder - Pointer to the IShellFolder for the folder containing the 
//				   item.
//		pidl	 - Pointer to an ITEMIDLIST relative to piFolder that we want
//				   the display name for.
//		dwFlags  - Flags to pass to ISF::GetDisplayNameOf().
//		pszName  - Pointer to the string where the display name is returned.
//		cchMax   - Maximum number of characters in pszName.
//
//  RETURN VALUE:
//      Returns TRUE if successful, FALSE otherwise.
//

BOOL Pidl_GetDisplayName(LPSHELLFOLDER piFolder, LPITEMIDLIST pidl, 
                         DWORD dwFlags, LPTSTR pszName, UINT cchMax)
{
  BOOL fSuccess = TRUE;
  STRRET str;
  BOOL fDesktop = FALSE;

  //
  // Check to see if a parent folder was specified.  If not, get a pointer
  // to the desktop folder.
  //
  if (NULL == piFolder)	
  {
    HRESULT hr = SHGetDesktopFolder(&piFolder);
    if (FAILED(hr))
      return (FALSE);
  }

  if(!pidl)
    return FALSE;

  //
  // Get the display name from the folder.  Then do any conversions necessary
  // depending on the type of string returned.
  //
  if (NOERROR == piFolder->GetDisplayNameOf(pidl, dwFlags, &str))
  {
    if (SUCCEEDED(StrRetToBuf(&str, pidl, pszName, cchMax)))
    {
      fSuccess = TRUE;
    }
  }
  else
    fSuccess = FALSE;

  //
  // Free the desktop folder interface if we allocated it.
  //
  if (fDesktop)
  {
    piFolder->Release();
    piFolder = NULL;
  }

  return (fSuccess);
}
