#include "stdafx.h"
#include <windows.h>

#include "helpers.h"
#include "processlist.h"

#include <stdio.h>
#include "EnumProcess.h"

static CEnumProcess& GetEnumProcessClass()
{
  static CEnumProcess s_enumeration;
  return s_enumeration;
}

bool SortByProcessName(const PROCESS &a, const PROCESS &b) 
{
  return _tcsicmp(a.ProcessName.c_str(), b.ProcessName.c_str()) < 0;
}

BOOL EnableDebugPrivilege(BOOL fEnable) 
{
  // Enabling the debug privilege allows the application to see
  // information about service applications
  BOOL fOk = FALSE;    // Assume function fails
  HANDLE hToken;

  // Try to open this process's access token
  if (OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES, 
    &hToken)) {

      // Attempt to modify the "Debug" privilege
      TOKEN_PRIVILEGES tp;
      tp.PrivilegeCount = 1;
      LookupPrivilegeValue(NULL, SE_DEBUG_NAME, &tp.Privileges[0].Luid);
      tp.Privileges[0].Attributes = fEnable ? SE_PRIVILEGE_ENABLED : 0;
      AdjustTokenPrivileges(hToken, FALSE, &tp, sizeof(tp), NULL, NULL);
      fOk = (GetLastError() == ERROR_SUCCESS);
      CloseHandle(hToken);
  }
  return(fOk);
}

void GetProcessOwner(HANDLE hProcess, string& szOwner)
{  
  // default value
  szOwner = _T("");

  if (hProcess == NULL) return;

  // get process token
  HANDLE hToken = NULL;
  EnablePrivilege(SE_TCB_NAME);
  if (!::OpenProcessToken(hProcess, TOKEN_QUERY, &hToken))
    return;

  // Obtain the size of the user information in the token.
  DWORD cbti = 0;
  ::GetTokenInformation(hToken, TokenUser, NULL, 0, &cbti);

  // Call should have failed due to zero-length buffer.
  if (::GetLastError() == ERROR_INSUFFICIENT_BUFFER)
  {
    // Allocate buffer for user information in the token.
    PTOKEN_USER ptiUser = (PTOKEN_USER) ::HeapAlloc(GetProcessHeap(), 0, cbti);
    if (ptiUser != NULL)
    {
      // Retrieve the user information from the token.
      if (::GetTokenInformation(hToken, TokenUser, ptiUser, cbti, &cbti))
      {
        SID_NAME_USE   snu;
        TCHAR          szUser[_MAX_PATH];
        DWORD          chUser = _MAX_PATH;
        PDWORD         pcchUser = &chUser; 
        TCHAR          szDomain[_MAX_PATH];
        DWORD          chDomain = _MAX_PATH;
        PDWORD         pcchDomain = &chDomain;

        // Retrieve user name and domain name based on user's SID.
        if (
          ::LookupAccountSid(
          NULL, 
          ptiUser->User.Sid, 
          szUser, 
          pcchUser, 
          szDomain, 
          pcchDomain, 
          &snu
          )
          )
        {
          // build the owner string as \\DomainName\UserName
          // szOwner = _T("\\\\");
          szOwner += szDomain;
          szOwner += _T("\\");
          szOwner += szUser;
        }
      }

      // free buffer
      ::HeapFree(::GetProcessHeap(), 0, ptiUser);
    }
  }

  // don't forget to free process token
  ::CloseHandle(hToken); 
}

bool EnumProcessList(CProcessList& pList)
{
  pList.clear();

  bool ret = false;
  bool bToolHelpAvailable = false;
  TCHAR tmp[MAX_PATH+1] = {0};
  
  // enable debug priv. so we can probe into system services
  EnableDebugPrivilege(TRUE);

  // always use the static reference instead of creating new instances
  // to minimize loading/freeing library and setting up function pointers
  CEnumProcess& enumeration = GetEnumProcessClass();
  CEnumProcess::CProcessEntry pEntry;  
  
  // enumeration.SetMethod(ENUM_METHOD::PSAPI);  
  
  // for each process, augment additional information
  for (BOOL OK = enumeration.GetProcessFirst(&pEntry); OK; OK = enumeration.GetProcessNext(&pEntry) )
  {
    DWORD pid = pEntry.dwPID;
    
    // skip [System Idle Process]
    if (pid != 0)
    {
      CEnumProcess::CModuleEntry mEntry;      

      // first module is the full path of the executable
      // could have use GetModuleFileNameEx, but it doesn't work on 9x systems
      BOOL OK = enumeration.GetModuleFirst(pid, &mEntry);
      if (OK)
      {
        // translate from funky file names to win32 style file names
        TranslateFilename(mEntry.lpFilename, tmp);
      }

      string pOwner;
      HANDLE hProcess = GetProcessHandleWithEnoughRights(pEntry.dwPID, PROCESS_QUERY_INFORMATION | PROCESS_VM_READ);
      if (hProcess)
      {
        GetProcessOwner(hProcess, pOwner);
        CloseHandle(hProcess);
      }      

      PROCESS ps = {0};
      ps.dwProcessId = pid;
      ps.ProcessName = pEntry.lpFilename;
      ps.ProcessFullPath = tmp;
      ps.Owner = pOwner;      
      pList.push_back(ps);      
    }
  }

  EnableDebugPrivilege(FALSE);

  return ret;
}
