// ProcessKillerHelper.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include <windows.h>
#include <vdmdbg.h>
#include <atlbase.h>
#include <atlconv.h>
#include <shellapi.h>
#include "cmdlineargs.h"
#include "safetp.h"
#include "ProgressDlg.h"
#include "resource.h"

// progress bar (http://www.codeproject.com/miscctrl/nonprogressbar.asp)
#include "gscontrols.h"

// Combination of the two articles
// See: http://www.ddj.com/dept/windows/184416547
// See: http://support.microsoft.com/kb/178893

#define MAX_DATA  4096
#define TA_FAILED 0
#define TA_SUCCESS_CLEAN 1
#define TA_SUCCESS_KILL 2
#define TA_SUCCESS_16 3
#define WARNING_TEXT _T("WARNING: Terminating a process can cause undesired\nresults including loss of data and system instability. The\nprocess will not be given the chance to save its state or\ndata before it is terminated.  Are you sure you want to\nterminate the process?\n")

bool FFGWin(HWND hwnd);
BOOL CALLBACK TerminateAppEnum( HWND hwnd, LPARAM lParam );
DWORD WINAPI CleanTerminateApp( DWORD dwPID, DWORD dwTimeout, TCHAR* processName, bool bShowUI);
DWORD WINAPI ForceTerminateApp( DWORD dwPID, DWORD dwTimeout, TCHAR* processName, bool bConfirmBeforeTerminate );

BOOL CALLBACK TerminateAppEnum( HWND hwnd, LPARAM lParam )
{
  DWORD dwID ;

  GetWindowThreadProcessId(hwnd, &dwID) ;

  if(dwID == (DWORD)lParam)
  {
    //FFGWin(hwnd);

    // Check whether the window still exists and that it does not have an owner.
    if (IsWindow(hwnd) && !GetWindow(hwnd, GW_OWNER))
    {
      // Do not close disabled windows.
      if (!IsWindowEnabled(hwnd))
        return TRUE;
      else
        PostMessage(hwnd, WM_CLOSE, 0, 0);
    }
  }

  return TRUE ;
}

DWORD WINAPI CleanTerminateApp( DWORD dwPID, DWORD dwTimeout, TCHAR* processName, bool bShowUI)
{
  DWORD  dwRet = TA_FAILED;
  HANDLE hProcess = OpenProcess(PROCESS_TERMINATE|SYNCHRONIZE, FALSE, dwPID);

  if (hProcess == NULL || hProcess == INVALID_HANDLE_VALUE) 
  {    
    return TA_FAILED;
  }

  TCHAR buf[MAX_PATH+1] = {0};
  _sntprintf(buf, MAX_PATH, _T("Waiting for %s to shutdown..."), processName);
  CProgressDlg dlg(buf);

  if (bShowUI)
  {
    dlg.Show();
  } 

  // TerminateAppEnum() posts WM_CLOSE to all windows whose PID
  // matches your process's.
  // NOTE: if timeout is 0, assume we want to install kill
  if (dwTimeout > 0)
  {
    EnumWindows((WNDENUMPROC)TerminateAppEnum, (LPARAM) dwPID) ;
  }  

  // Wait on the handle. If it signals, great. If it times out,
  // then you kill it.
  if( WaitForSingleObject(hProcess, dwTimeout) != WAIT_OBJECT_0)
  {
    dwRet = TA_FAILED;    
  }
  else
  {
    dwRet = TA_SUCCESS_CLEAN ;    
  }

  dlg.Dismiss();

  CloseHandle(hProcess) ;

  return dwRet ;
}

DWORD WINAPI ForceTerminateApp( DWORD dwPID, DWORD dwTimeout, TCHAR* processName, bool bConfirmBeforeTerminate )
{
  DWORD  dwRet = TA_FAILED;
  HANDLE hProcess = OpenProcess(PROCESS_TERMINATE|SYNCHRONIZE, FALSE, dwPID);

  if (hProcess == NULL || hProcess == INVALID_HANDLE_VALUE) 
  {    
    return TA_FAILED;
  }

  TCHAR buf[MAX_DATA+1] = {0};
  int yesNo = IDYES;

  if (bConfirmBeforeTerminate)
  {
    DWORD ForegroundThreadID = GetWindowThreadProcessId(GetForegroundWindow(),NULL); 
    DWORD ThisThreadID = GetCurrentThreadId();
    AttachThreadInput(ThisThreadID, ForegroundThreadID, true);

    // show diff msg depending on if we tried a clean shutdown first
    if (dwTimeout == 0)
    {
      yesNo = MessageBox(NULL, WARNING_TEXT, _T("Confirmation"), MB_ICONWARNING | MB_YESNO | MB_SETFOREGROUND );
    }
    else
    {                
      _sntprintf(buf, MAX_DATA, _T("Timed out waiting for %s to shutdown..."), processName);
      yesNo = MessageBox(NULL, WARNING_TEXT, buf, MB_ICONWARNING | MB_YESNO | MB_SETFOREGROUND );          
    }
    AttachThreadInput(ThisThreadID, ForegroundThreadID, false);
    if (yesNo == IDNO) dwRet = TA_FAILED;
  }

  if (yesNo == IDYES)
  {
    OSVERSIONINFO vi = { sizeof(vi) };
    GetVersionEx(&vi);

    // Win9x
    if (vi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
    {
      if (TerminateProcess(hProcess,(DWORD)-1)) dwRet = TA_SUCCESS_KILL;      
    } 
    else // NT or above
    {
      if (SafeTerminateProcess(hProcess,(DWORD)-1)) dwRet = TA_SUCCESS_KILL;  
    }
  }

  CloseHandle(hProcess) ;

  return dwRet ;
}

int APIENTRY _tWinMain(HINSTANCE hInstance,
                       HINSTANCE hPrevInstance,
                       LPTSTR    lpCmdLine,
                       int       nCmdShow)
{
  UNREFERENCED_PARAMETER(hPrevInstance);
  UNREFERENCED_PARAMETER(lpCmdLine);

  USES_CONVERSION;

  // initialize progress bar class
  INITCOMMONCONTROLSEX InitCtrlEx;
  InitCtrlEx.dwSize = sizeof(INITCOMMONCONTROLSEX);
  InitCtrlEx.dwICC  = ICC_PROGRESS_CLASS;
  InitCommonControlsEx(&InitCtrlEx);
  InitializeGSControls();
  
  CmdLineArgs szArglist;

  int ret = 0;  
  int nArgs = szArglist.size(); 

  DWORD pid = 0;
  DWORD dwTimeout = 0;
  TCHAR processName[MAX_PATH+1] = {0};
  bool  bShowUI = false;
  bool  bConfirmBeforeTerminate = false;

  for (int i = 1; i < nArgs; i++) 
  {
    if (!_tcsicmp(szArglist[i], _T("/pid"))) 
    {
      if (i+1 < nArgs)
      {
        pid = _ttol(szArglist[i+1]);          
        i++;
      }      
    }
    else if (!_tcsicmp(szArglist[i], _T("/timeout")))
    {
      if (i+1 < nArgs)
      {
        dwTimeout = _ttol(szArglist[i+1]);          
        i++;
      }              
    }
    else if (!_tcsicmp(szArglist[i], _T("/name")))
    {
      if (i+1 < nArgs)
      {
        _tcsncpy(processName, szArglist[i+1], MAX_PATH);
        i++;
      }
    } 
    else if (!_tcsicmp(szArglist[i], _T("/confirm"))) 
    {
      bConfirmBeforeTerminate = true;
    }
    else if (!_tcsicmp(szArglist[i], _T("/showprogress"))) 
    {
      bShowUI = true;
    }
  }    

  DWORD dwRet = 0;

  if (pid != 0)
  {
    dwRet = CleanTerminateApp(pid, dwTimeout, processName, bShowUI);
    if (dwRet != TA_SUCCESS_CLEAN)
    {
      dwRet = ForceTerminateApp(pid, dwTimeout, processName, bConfirmBeforeTerminate);
    }
  }

  // uninitialize progress bar class
  UninitializeGSControls();

  return ret;
}

// Modified from http://www.delphi3000.com/articles/article_1775.asp
bool FFGWin(HWND hwnd)
{ 
  bool Ret; 

  DWORD nullvalue = 0; 

  DWORD ForegroundThreadID; 
  DWORD ThisThreadID; 
  DWORD timeout; 

  if(IsIconic(hwnd)) ShowWindow(hwnd, SW_RESTORE); 

  if(GetForegroundWindow()==hwnd) return true; 
  else 
  {
    OSVERSIONINFO vi = { sizeof(vi) };
    GetVersionEx(&vi);

    // Windows 98/2000 doesn't want to foreground a window when some other 
    // window has keyboard focus 
    if( 
      ((vi.dwPlatformId == VER_PLATFORM_WIN32_NT) && (vi.dwMajorVersion > 4)) || 
      ((vi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS) && ((vi.dwMajorVersion > 4) || ((vi.dwMajorVersion == 4) &&(vi.dwMinorVersion > 0)))) 
      ) 
    { 
      // Code from Karl E. Peterson, www.mvps.org/vb/sample.htm 
      // Converted to Delphi by Ray Lischner 
      // Published in The Delphi Magazine 55, page 16 

      Ret = false; 
      ForegroundThreadID = GetWindowThreadProcessId(GetForegroundWindow(),NULL); 
      ThisThreadID = GetWindowThreadProcessId(hwnd,NULL); 
      if( AttachThreadInput(ThisThreadID, ForegroundThreadID, true) ) 
      { 
        BringWindowToTop(hwnd); // IE 5.5 related hack 
        SetForegroundWindow(hwnd); 
        AttachThreadInput(ThisThreadID, ForegroundThreadID, false); 
        Ret = (GetForegroundWindow() == hwnd); 
      } 
      if(!Ret) 
      { 
        // Code by Daniel P. Stasinski 
        SystemParametersInfo(SPI_GETFOREGROUNDLOCKTIMEOUT, 0, &timeout, 0); 
        SystemParametersInfo(SPI_SETFOREGROUNDLOCKTIMEOUT, 0, &nullvalue, 
          SPIF_SENDCHANGE); 
        BringWindowToTop(hwnd); // IE 5.5 related hack 
        SetForegroundWindow(hwnd); 
        SystemParametersInfo(SPI_SETFOREGROUNDLOCKTIMEOUT, 0, 
          &timeout, SPIF_SENDCHANGE); 
      } 
    } 
    else 
    { 
      BringWindowToTop(hwnd); // IE 5.5 related hack 
      SetForegroundWindow(hwnd); 
    } 
    Ret = (GetForegroundWindow() == hwnd); 
  } 
  return Ret; 
}