// ProgressDlg.cpp : implementation file
//

#include "stdafx.h"
#include <CommCtrl.h>
#include <windowsx.h>
#include <process.h>

#include "resource.h"
#include "ProgressDlg.h"

void CProgressDlg::Show()
{
  if (!m_threadHandle)
  {
    unsigned threadAddr = 0;
    m_threadHandle = _beginthreadex(NULL, 0, CProgressDlg::TShowProgress, (void*)this, 0, &threadAddr);
    
    // wait for thread begin to execute
    WaitForSingleObject(m_hEvent1, INFINITE);    
  }
}

void CProgressDlg::ShowProgress()
{
  if (!m_hDlgWnd)
  {
    m_hDlgWnd = CreateDialog(NULL, MAKEINTRESOURCE(IDD_PROGRESSDLG), NULL, ProgressDlg_Proc);
    if (m_hDlgWnd != NULL)
    {
      SendMessage(m_hDlgWnd, WM_SETTEXT, 0, (LPARAM)m_Title);
      ShowWindow(m_hDlgWnd, SW_SHOW);
    }

    SetEvent(m_hEvent1);

    // wait for dismiss event while processing messages
    HANDLE waitEvents[] = { m_hEvent2 };
    const unsigned hcount = sizeof(waitEvents)/sizeof(HANDLE);

    while(true)
    {        
      DWORD dwWait =  MsgWaitForMultipleObjects( hcount, waitEvents, FALSE, INFINITE, QS_ALLINPUT);

      switch(dwWait)
      {      
      case WAIT_TIMEOUT:            // should never come here
        break;        
      case WAIT_OBJECT_0:           // exit event
        {
          if (m_hDlgWnd) DestroyWindow(m_hDlgWnd);
          m_hDlgWnd = NULL;
          return;
        }
      case WAIT_OBJECT_0+hcount:    // some type of msg, dispatch them
      default:                      // default
        {       
          MSG wMsg; 
          while(::PeekMessage(&wMsg, NULL, 0, 0, PM_REMOVE)) 
          {
            ::DispatchMessage(&wMsg);
          }
          break;
        }
      }
    }
  }
}

void CProgressDlg::Dismiss()
{
  // wait for thread to die if exists
  if (m_threadHandle)
  {
    SetEvent(m_hEvent2);  // signal dismiss event  
    WaitForSingleObject((HANDLE)m_threadHandle, INFINITE);    
    CloseHandle((void*)m_threadHandle);
    m_threadHandle = NULL;
  }  
}

unsigned __stdcall CProgressDlg::TShowProgress(void* pvargs)
{
  try
  {
    CProgressDlg* tArgs = (CProgressDlg*)pvargs;    
    if (tArgs)
    {
      tArgs->ShowProgress();
    }
  }
  catch (...)
  {   
  }
  return 0;
}

///////////////////////////////////////////////////////////////////////////////


BOOL ProgressDlg_OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam) 
{
  return(TRUE);
}


///////////////////////////////////////////////////////////////////////////////


INT_PTR WINAPI ProgressDlg_Proc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam) 
{
  switch (uMsg) 
  {
  case (WM_INITDIALOG): 
    return (SetDlgMsgResult(hwnd, uMsg,
            HANDLE_WM_INITDIALOG((hwnd), (wParam), (lParam), (ProgressDlg_OnInitDialog))));
  }
  return(FALSE);  
}