#ifndef CMDLINEARGS_H
#define CMDLINEARGS_H

// http://www.codeguru.com/cpp/w-p/win32/article.php/c1427/
// Enhanced to use TCHAR to be unicode compliant

#include <windows.h>
#include <tchar.h>
#include <vector>

class CmdLineArgs : public std::vector<TCHAR*>
{
public:
  CmdLineArgs ()
  {
    // Save local copy of the command line string, because
    // ParseCmdLine() modifies this string while parsing it.
    LPTSTR cmdline = GetCommandLine();
    m_cmdline = new TCHAR [_tcslen (cmdline) + 1];
    if (m_cmdline)
    {
      _tcscpy (m_cmdline, cmdline);
      ParseCmdLine(); 
    }
  }
  ~CmdLineArgs()
  {
    delete m_cmdline;
  }

private:
  LPTSTR  m_cmdline; // the command line string

  ////////////////////////////////////////////////////////////////////////////////
  // Parse m_cmdline into individual tokens, which are delimited by spaces. If a
  // token begins with a quote, then that token is terminated by the next quote
  // followed immediately by a space or terminator.  This allows tokens to contain
  // spaces.
  // This input string:     This "is" a ""test"" "of the parsing" alg"o"rithm.
  // Produces these tokens: This, is, a, "test", of the parsing, alg"o"rithm
  ////////////////////////////////////////////////////////////////////////////////
  void ParseCmdLine ()
  {
    enum { TERM  = '\0',
      QUOTE = '\"' };

    bool bInQuotes = false;
    TCHAR* pargs = m_cmdline;

    while (*pargs)
    {
      while (isspace (*pargs))        // skip leading whitespace
        pargs = _tcsinc(pargs);

      bInQuotes = (*pargs == QUOTE);  // see if this token is quoted

      if (bInQuotes)                  // skip leading quote
        pargs = _tcsinc(pargs);

      push_back (pargs);              // store position of current token

      // Find next token.
      // NOTE: Args are normally terminated by whitespace, unless the
      // arg is quoted.  That's why we handle the two cases separately,
      // even though they are very similar.
      if (bInQuotes)
      {
        // find next quote followed by a space or terminator
        while (*pargs && 
          !(*pargs == QUOTE && (_istspace (pargs[1]) || pargs[1] == TERM)))
          pargs = _tcsinc(pargs);
        if (*pargs)
        {
          *pargs = TERM;  // terminate token
          if (pargs[1])   // if quoted token not followed by a terminator
          {
            pargs = _tcsinc(pargs); // advance to next token
            pargs = _tcsinc(pargs);
          }
        }
      }
      else
      {
        // skip to next non-whitespace character
        while (*pargs && !_istspace (*pargs)) 
          pargs = _tcsinc(pargs);
        if (*pargs && _istspace (*pargs)) // end of token
        {
          *pargs = TERM;    // terminate token
          pargs = _tcsinc(pargs);    // advance to next token or terminator
        }
      }
    } // while (*pargs)
  } // ParseCmdLine()
}; // class CmdLineArgs


#endif // CMDLINEARGS_H