#pragma once

#include <windows.h>
#include <WinSvc.h>
#include <vector>
#include <string>
using namespace std;

struct CService;

typedef std::vector<CService> CServiceList;
typedef std::vector<CService>::iterator CServiceListItr;

enum Status
{
  SVC_STATUS_UNKNOWN = 0,
  SVC_STATUS_ACTIVE = 1,
  SVC_STATUS_INACTIVE = 2
};

enum SVCERR
{ 
  SVC_SUCCESS = 0,
  SVC_FAIL,
  SVC_TIMEDOUT
};

struct CService
{
  inline CService() : status(SVC_STATUS_UNKNOWN) {}
  inline ~CService() {}

  string name;
  string display_name;  
  Status status;  
};

class ServiceManager
{
public:
  ServiceManager(void) {}
  ~ServiceManager(void) {}
  bool GetServices(CServiceList& services, int status_mask = SVC_STATUS_ACTIVE|SVC_STATUS_INACTIVE);
  bool IsServiceDisabled(const TCHAR* srvName);
  bool ChangeServiceStartMode(const TCHAR* srvName, DWORD newStartType);
  SVCERR GetServiceDependencies(const TCHAR* srvName, CServiceList& services);
  SVCERR StartService(const TCHAR* srvName, int timeout = 0xFFFFFFFF);
  SVCERR StopService(const TCHAR* srvName, bool bStopDependencies = true, int timeout = 0xFFFFFFFF); // See Q245230

protected:
  bool GetServiceHandle(const TCHAR* svcName, DWORD scmAccess, DWORD svcAccess, SC_HANDLE* schSCManager, SC_HANDLE* schService);  
  void LogError(const TCHAR* funcName, DWORD dwErrorCode);
  void SleepWaitHint(DWORD dwWaitHint);
};
