#include "stdafx.h"
#include "service.h"
#include <string>
using namespace std;

#include "resource.h"
#include "ProgressDlg.h"
#include "JrPlugin_MyPlugin.h"

extern HMODULE GetThisModuleHandle();

struct ThreadProgressArgs
{  
  string title;
  HANDLE hEvent1;
  HANDLE hEvent2;
};

bool SendMsgDispatch()
{
  bool bret = true;
  MSG msg = {0};
  while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) 
  { 
    DispatchMessage(&msg);   
  }
  return bret;
}

unsigned __stdcall ShowProgress(void* pvargs)
{
  try
  {
    ThreadProgressArgs* tArgs = (ThreadProgressArgs*)pvargs;    
    if (tArgs)
    {      
      string title = tArgs->title;
      HANDLE hEvent1 = tArgs->hEvent1;
      HANDLE hEvent2 = tArgs->hEvent2;

      HWND hDlgWnd = CreateDialog(GetThisModuleHandle(), MAKEINTRESOURCE(IDD_PROGRESSDLG), NULL, ProgressDlg_Proc);
      if (hDlgWnd != NULL)
      {
        SendMessage(hDlgWnd, WM_SETTEXT, 0, (LPARAM)title.c_str());
        ShowWindow(hDlgWnd, SW_SHOW);
      }
      SetEvent(hEvent1);

      HANDLE waitEvents[] = { hEvent2 };
      const unsigned hcount = sizeof(waitEvents)/sizeof(HANDLE);

      while(true)
      {
        DWORD stopcheck;
        stopcheck =  MsgWaitForMultipleObjects( hcount, waitEvents, FALSE, INFINITE, QS_ALLINPUT);

        if ( ((stopcheck - WAIT_OBJECT_0) == 0)  || !SendMsgDispatch() )
        {
          if (hDlgWnd) DestroyWindow(hDlgWnd);
          return 0;
        }
      }
    }
  }
  catch (...)
  {
  }
  return 0;
}


_declspec (dllexport) void WINAPI ServiceCtrl(const TCHAR* svcName, const TCHAR* svcDisplayName, bool bStart)
{
  TCHAR buf[MAX_DATA+1] = {0};
  HANDLE hEvent1 = NULL;
  HANDLE hEvent2 = NULL;

  GetOption(_T("SETUP"), _T("ShowMsgBoxSuccess"), _T("FALSE"), buf, MAX_PATH);
  bool bShowMsgBoxSuccess = (_tcsicmp(buf, _T("TRUE")) == 0);

  GetOption(_T("SETUP"), _T("ShowMsgBoxError"), _T("TRUE"), buf, MAX_PATH);
  bool bShowMsgBoxError = (_tcsicmp(buf, _T("TRUE")) == 0);

  GetOption(_T("SETUP"), _T("ShowProgress"), _T("TRUE"), buf, MAX_PATH);
  bool bShowUI = (_tcsicmp(buf, _T("TRUE")) == 0);

  if (bShowUI)
  {
    ThreadProgressArgs tArgs;
    if (bStart)
    {
      _sntprintf(buf, MAX_DATA, _T("Starting %s (%s)..."), svcDisplayName, svcName);
      tArgs.title = buf;
    }
    else
    {
      _sntprintf(buf, MAX_DATA, _T("Stopping %s (%s)..."), svcDisplayName, svcName);
      tArgs.title = buf;
    }    
    tArgs.hEvent1 = hEvent1 = CreateEvent(NULL, false, false, NULL);  // wait for this event to indicate UI is ready
    tArgs.hEvent2 = hEvent2 = CreateEvent(NULL, false, false, NULL);  // set this event to dismiss UI

    unsigned threadAddr = 0;
    uintptr_t threadHandle = _beginthreadex(NULL, 0, ShowProgress, (void*)&tArgs, 0, &threadAddr);
    WaitForSingleObject(hEvent1, INFINITE);
    CloseHandle((void*)hEvent1);
    CloseHandle((void*)threadHandle);
  }

  if (bStart)
  {
    SVCERR ret = g_ServiceManager.StartService(svcName, 20000);

    if (bShowUI)
    {
      SetEvent(hEvent2);
    }

    switch (ret)
    {
    case SVC_TIMEDOUT:
      if (bShowMsgBoxError) MessageBox(NULL, _T("Operation Timed out.  Service did not respond in a timely fashion."), std::string(ThisPlugin_DisplayName).c_str(), MB_OK);      
      break;
    case SVC_FAIL:      
      _sntprintf(buf, MAX_DATA, _T("Failed to start %s.  Win32 Error Code: %d"), svcName, GetLastError());
      if (bShowMsgBoxError)  MessageBox(NULL, buf, ThisPlugin_DisplayName, MB_OK);      
      break;
    case SVC_SUCCESS:
      _sntprintf(buf, MAX_DATA, _T("%s (%s) has been started."), svcDisplayName, svcName);
      if (bShowMsgBoxSuccess) MessageBox(NULL, buf, ThisPlugin_DisplayName, MB_OK);
    }    
  }
  else
  {
    SVCERR ret = SVC_FAIL;
    ret = g_ServiceManager.StopService(svcName, true, 20000);    

    if (bShowUI)
    {
      SetEvent(hEvent2);
    }

    switch (ret)
    {
    case SVC_TIMEDOUT:
      if (bShowMsgBoxError) MessageBox(NULL, _T("Operation Timed out.  Service did not respond in a timely fashion."), std::string(ThisPlugin_DisplayName).c_str(), MB_OK);      
      break;
    case SVC_FAIL:
      _sntprintf(buf, MAX_DATA, _T("Failed to stop %s.  Win32 Error Code: %d"), svcName, GetLastError());
      if (bShowMsgBoxError) MessageBox(NULL, buf, ThisPlugin_DisplayName, MB_OK);
      break;
    case SVC_SUCCESS:
      _sntprintf(buf, MAX_DATA, _T("%s (%s) has been stopped."), svcDisplayName, svcName);
      if (bShowMsgBoxSuccess) MessageBox(NULL, buf, ThisPlugin_DisplayName, MB_OK);      
    }
  }
}